@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2023 NVIDIA Corporation
+    Copyright (c) 2015-2024 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -22,6 +22,7 @@
 *******************************************************************************/
 
 #include "uvm_hal.h"
+#include "uvm_global.h"
 #include "uvm_kvmalloc.h"
 
 #include "cla16f.h"
@@ -43,6 +44,10 @@
 #include "clc7b5.h"
 #include "clc86f.h"
 #include "clc8b5.h"
+#include "clc96f.h"
+#include "clc9b5.h"
+#include "clca6f.h"
+#include "clcab5.h"
 
 static int uvm_downgrade_force_membar_sys = 1;
 module_param(uvm_downgrade_force_membar_sys, uint, 0644);
@@ -70,16 +75,17 @@ static uvm_hal_class_ops_t ce_table[] =
             .semaphore_release = uvm_hal_maxwell_ce_semaphore_release,
             .semaphore_timestamp = uvm_hal_maxwell_ce_semaphore_timestamp,
             .semaphore_reduction_inc = uvm_hal_maxwell_ce_semaphore_reduction_inc,
+            .semaphore_target_is_valid = uvm_hal_maxwell_semaphore_target_is_valid,
             .offset_out = uvm_hal_maxwell_ce_offset_out,
             .offset_in_out = uvm_hal_maxwell_ce_offset_in_out,
             .phys_mode = uvm_hal_maxwell_ce_phys_mode,
             .plc_mode = uvm_hal_maxwell_ce_plc_mode,
             .memcopy_copy_type = uvm_hal_maxwell_ce_memcopy_copy_type,
-            .memcopy_is_valid = uvm_hal_ce_memcopy_is_valid_stub,
+            .memcopy_is_valid = uvm_hal_maxwell_ce_memcopy_is_valid,
             .memcopy_patch_src = uvm_hal_ce_memcopy_patch_src_stub,
             .memcopy = uvm_hal_maxwell_ce_memcopy,
             .memcopy_v_to_v = uvm_hal_maxwell_ce_memcopy_v_to_v,
-            .memset_is_valid = uvm_hal_ce_memset_is_valid_stub,
+            .memset_is_valid = uvm_hal_maxwell_ce_memset_is_valid,
             .memset_1 = uvm_hal_maxwell_ce_memset_1,
             .memset_4 = uvm_hal_maxwell_ce_memset_4,
             .memset_8 = uvm_hal_maxwell_ce_memset_8,
@@ -139,9 +145,9 @@ static uvm_hal_class_ops_t ce_table[] =
         .u.ce_ops = {
             .method_is_valid = uvm_hal_method_is_valid_stub,
             .plc_mode = uvm_hal_ampere_ce_plc_mode_c7b5,
-            .memcopy_is_valid = uvm_hal_ce_memcopy_is_valid_stub,
+            .memcopy_is_valid = uvm_hal_maxwell_ce_memcopy_is_valid,
             .memcopy_patch_src = uvm_hal_ce_memcopy_patch_src_stub,
-            .memset_is_valid = uvm_hal_ce_memset_is_valid_stub,
+            .memset_is_valid = uvm_hal_maxwell_ce_memset_is_valid,
         },
     },
     {
@@ -163,6 +169,16 @@ static uvm_hal_class_ops_t ce_table[] =
             .decrypt = uvm_hal_hopper_ce_decrypt,
         },
     },
+    {
+        .id = BLACKWELL_DMA_COPY_A,
+        .parent_id = HOPPER_DMA_COPY_A,
+        .u.ce_ops = {},
+    },
+    {
+        .id = BLACKWELL_DMA_COPY_B,
+        .parent_id = BLACKWELL_DMA_COPY_A,
+        .u.ce_ops = {},
+    },
 };
 
 // Table for GPFIFO functions.  Same idea as the copy engine table.
@@ -177,6 +193,7 @@ static uvm_hal_class_ops_t host_table[] =
             .sw_method_is_valid = uvm_hal_method_is_valid_stub,
             .wait_for_idle = uvm_hal_maxwell_host_wait_for_idle,
             .membar_sys = uvm_hal_maxwell_host_membar_sys,
+
             // No MEMBAR GPU until Pascal, just do a MEMBAR SYS.
             .membar_gpu = uvm_hal_maxwell_host_membar_sys,
             .noop = uvm_hal_maxwell_host_noop,
@@ -184,6 +201,7 @@ static uvm_hal_class_ops_t host_table[] =
             .semaphore_acquire = uvm_hal_maxwell_host_semaphore_acquire,
             .semaphore_release = uvm_hal_maxwell_host_semaphore_release,
             .semaphore_timestamp = uvm_hal_maxwell_host_semaphore_timestamp,
+            .semaphore_target_is_valid = uvm_hal_maxwell_semaphore_target_is_valid,
             .set_gpfifo_entry = uvm_hal_maxwell_host_set_gpfifo_entry,
             .set_gpfifo_noop = uvm_hal_maxwell_host_set_gpfifo_noop,
             .set_gpfifo_pushbuffer_segment_base = uvm_hal_maxwell_host_set_gpfifo_pushbuffer_segment_base_unsupported,
@@ -199,8 +217,8 @@ static uvm_hal_class_ops_t host_table[] =
             .clear_faulted_channel_method = uvm_hal_maxwell_host_clear_faulted_channel_method_unsupported,
             .clear_faulted_channel_register = uvm_hal_maxwell_host_clear_faulted_channel_register_unsupported,
             .access_counter_clear_all = uvm_hal_maxwell_access_counter_clear_all_unsupported,
-            .access_counter_clear_type = uvm_hal_maxwell_access_counter_clear_type_unsupported,
             .access_counter_clear_targeted = uvm_hal_maxwell_access_counter_clear_targeted_unsupported,
+            .access_counter_query_clear_op = uvm_hal_maxwell_access_counter_query_clear_op_unsupported,
             .get_time = uvm_hal_maxwell_get_time,
         }
     },
@@ -236,9 +254,6 @@ static uvm_hal_class_ops_t host_table[] =
             .replay_faults = uvm_hal_volta_replay_faults,
             .cancel_faults_va = uvm_hal_volta_cancel_faults_va,
             .clear_faulted_channel_method = uvm_hal_volta_host_clear_faulted_channel_method,
-            .access_counter_clear_all = uvm_hal_volta_access_counter_clear_all,
-            .access_counter_clear_type = uvm_hal_volta_access_counter_clear_type,
-            .access_counter_clear_targeted = uvm_hal_volta_access_counter_clear_targeted,
             .semaphore_timestamp = uvm_hal_volta_host_semaphore_timestamp,
         }
     },
@@ -250,6 +265,12 @@ static uvm_hal_class_ops_t host_table[] =
             .semaphore_release = uvm_hal_turing_host_semaphore_release,
             .clear_faulted_channel_method = uvm_hal_turing_host_clear_faulted_channel_method,
             .set_gpfifo_entry = uvm_hal_turing_host_set_gpfifo_entry,
+            .tlb_invalidate_all = uvm_hal_turing_host_tlb_invalidate_all,
+            .tlb_invalidate_va = uvm_hal_turing_host_tlb_invalidate_va,
+            .tlb_invalidate_test = uvm_hal_turing_host_tlb_invalidate_test,
+            .access_counter_clear_all = uvm_hal_turing_access_counter_clear_all,
+            .access_counter_clear_targeted = uvm_hal_turing_access_counter_clear_targeted,
+            .access_counter_query_clear_op = uvm_hal_turing_access_counter_query_clear_op,
         }
     },
     {
@@ -282,6 +303,23 @@ static uvm_hal_class_ops_t host_table[] =
             .set_gpfifo_pushbuffer_segment_base = uvm_hal_hopper_host_set_gpfifo_pushbuffer_segment_base,
         }
     },
+    {
+        .id = BLACKWELL_CHANNEL_GPFIFO_A,
+        .parent_id = HOPPER_CHANNEL_GPFIFO_A,
+        .u.host_ops = {
+            .tlb_invalidate_all = uvm_hal_blackwell_host_tlb_invalidate_all,
+            .tlb_invalidate_va = uvm_hal_blackwell_host_tlb_invalidate_va,
+            .tlb_invalidate_test = uvm_hal_blackwell_host_tlb_invalidate_test,
+            .access_counter_query_clear_op = uvm_hal_blackwell_access_counter_query_clear_op_gb100,
+        }
+    },
+    {
+        .id = BLACKWELL_CHANNEL_GPFIFO_B,
+        .parent_id = BLACKWELL_CHANNEL_GPFIFO_A,
+        .u.host_ops = {
+            .access_counter_query_clear_op = uvm_hal_blackwell_access_counter_query_clear_op_gb20x
+        }
+    },
 };
 
 static uvm_hal_class_ops_t arch_table[] =
@@ -293,7 +331,6 @@ static uvm_hal_class_ops_t arch_table[] =
             .mmu_mode_hal = uvm_hal_mmu_mode_maxwell,
             .enable_prefetch_faults = uvm_hal_maxwell_mmu_enable_prefetch_faults_unsupported,
             .disable_prefetch_faults = uvm_hal_maxwell_mmu_disable_prefetch_faults_unsupported,
-            .mmu_engine_id_to_type = uvm_hal_maxwell_mmu_engine_id_to_type_unsupported,
             .mmu_client_id_to_utlb_id = uvm_hal_maxwell_mmu_client_id_to_utlb_id_unsupported,
         }
     },
@@ -319,7 +356,6 @@ static uvm_hal_class_ops_t arch_table[] =
         .u.arch_ops = {
             .init_properties = uvm_hal_volta_arch_init_properties,
             .mmu_mode_hal = uvm_hal_mmu_mode_volta,
-            .mmu_engine_id_to_type = uvm_hal_volta_mmu_engine_id_to_type,
             .mmu_client_id_to_utlb_id = uvm_hal_volta_mmu_client_id_to_utlb_id,
         },
     },
@@ -329,7 +365,6 @@ static uvm_hal_class_ops_t arch_table[] =
         .u.arch_ops = {
             .init_properties = uvm_hal_turing_arch_init_properties,
             .mmu_mode_hal = uvm_hal_mmu_mode_turing,
-            .mmu_engine_id_to_type = uvm_hal_turing_mmu_engine_id_to_type,
         },
     },
     {
@@ -338,7 +373,6 @@ static uvm_hal_class_ops_t arch_table[] =
         .u.arch_ops = {
             .init_properties = uvm_hal_ampere_arch_init_properties,
             .mmu_mode_hal = uvm_hal_mmu_mode_ampere,
-            .mmu_engine_id_to_type = uvm_hal_ampere_mmu_engine_id_to_type,
             .mmu_client_id_to_utlb_id = uvm_hal_ampere_mmu_client_id_to_utlb_id,
         },
     },
@@ -355,10 +389,27 @@ static uvm_hal_class_ops_t arch_table[] =
         .u.arch_ops = {
             .init_properties = uvm_hal_hopper_arch_init_properties,
             .mmu_mode_hal = uvm_hal_mmu_mode_hopper,
-            .mmu_engine_id_to_type = uvm_hal_hopper_mmu_engine_id_to_type,
             .mmu_client_id_to_utlb_id = uvm_hal_hopper_mmu_client_id_to_utlb_id,
         },
     },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GB100,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GH100,
+        .u.arch_ops = {
+            .init_properties = uvm_hal_blackwell_arch_init_properties,
+            .mmu_mode_hal = uvm_hal_mmu_mode_blackwell,
+            .mmu_client_id_to_utlb_id = uvm_hal_blackwell_mmu_client_id_to_utlb_id,
+        }
+    },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GB200,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GB100,
+        .u.arch_ops = {
+            // Note that GB20x MMU behaves as Hopper MMU, so it inherits from
+            // Hopper's MMU, not from GB10x.
+            .mmu_mode_hal = uvm_hal_mmu_mode_hopper,
+        }
+    },
 };
 
 static uvm_hal_class_ops_t fault_buffer_table[] =
@@ -373,7 +424,8 @@ static uvm_hal_class_ops_t fault_buffer_table[] =
             .read_get = uvm_hal_maxwell_fault_buffer_read_get_unsupported,
             .write_get = uvm_hal_maxwell_fault_buffer_write_get_unsupported,
             .get_ve_id = uvm_hal_maxwell_fault_buffer_get_ve_id_unsupported,
-            .parse_entry = uvm_hal_maxwell_fault_buffer_parse_entry_unsupported,
+            .get_mmu_engine_type = uvm_hal_maxwell_fault_buffer_get_mmu_engine_type_unsupported,
+            .parse_replayable_entry = uvm_hal_maxwell_fault_buffer_parse_replayable_entry_unsupported,
             .entry_is_valid = uvm_hal_maxwell_fault_buffer_entry_is_valid_unsupported,
             .entry_clear_valid = uvm_hal_maxwell_fault_buffer_entry_clear_valid_unsupported,
             .entry_size = uvm_hal_maxwell_fault_buffer_entry_size_unsupported,
@@ -396,7 +448,7 @@ static uvm_hal_class_ops_t fault_buffer_table[] =
             .read_put = uvm_hal_pascal_fault_buffer_read_put,
             .read_get = uvm_hal_pascal_fault_buffer_read_get,
             .write_get = uvm_hal_pascal_fault_buffer_write_get,
-            .parse_entry = uvm_hal_pascal_fault_buffer_parse_entry,
+            .parse_replayable_entry = uvm_hal_pascal_fault_buffer_parse_replayable_entry,
             .entry_is_valid = uvm_hal_pascal_fault_buffer_entry_is_valid,
             .entry_clear_valid = uvm_hal_pascal_fault_buffer_entry_clear_valid,
             .entry_size = uvm_hal_pascal_fault_buffer_entry_size,
@@ -411,7 +463,8 @@ static uvm_hal_class_ops_t fault_buffer_table[] =
             .read_get = uvm_hal_volta_fault_buffer_read_get,
             .write_get = uvm_hal_volta_fault_buffer_write_get,
             .get_ve_id = uvm_hal_volta_fault_buffer_get_ve_id,
-            .parse_entry = uvm_hal_volta_fault_buffer_parse_entry,
+            .get_mmu_engine_type = uvm_hal_volta_fault_buffer_get_mmu_engine_type,
+            .parse_replayable_entry = uvm_hal_volta_fault_buffer_parse_replayable_entry,
             .parse_non_replayable_entry = uvm_hal_volta_fault_buffer_parse_non_replayable_entry,
             .get_fault_type = uvm_hal_volta_fault_buffer_get_fault_type,
         }
@@ -422,12 +475,15 @@ static uvm_hal_class_ops_t fault_buffer_table[] =
         .u.fault_buffer_ops = {
             .disable_replayable_faults = uvm_hal_turing_disable_replayable_faults,
             .clear_replayable_faults = uvm_hal_turing_clear_replayable_faults,
+            .get_mmu_engine_type = uvm_hal_turing_fault_buffer_get_mmu_engine_type,
         }
     },
     {
         .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GA100,
         .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_TU100,
-        .u.fault_buffer_ops = {}
+        .u.fault_buffer_ops = {
+            .get_mmu_engine_type = uvm_hal_ampere_fault_buffer_get_mmu_engine_type,
+        }
     },
     {
         .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_AD100,
@@ -439,8 +495,22 @@ static uvm_hal_class_ops_t fault_buffer_table[] =
         .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_AD100,
         .u.fault_buffer_ops = {
             .get_ve_id = uvm_hal_hopper_fault_buffer_get_ve_id,
+            .get_mmu_engine_type = uvm_hal_hopper_fault_buffer_get_mmu_engine_type,
         }
     },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GB100,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GH100,
+        .u.fault_buffer_ops = {
+            .get_fault_type = uvm_hal_blackwell_fault_buffer_get_fault_type,
+            .get_mmu_engine_type = uvm_hal_blackwell_fault_buffer_get_mmu_engine_type,
+        }
+    },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GB200,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GB100,
+        .u.fault_buffer_ops = {}
+    },
 };
 
 static uvm_hal_class_ops_t access_counter_buffer_table[] =
@@ -470,22 +540,19 @@ static uvm_hal_class_ops_t access_counter_buffer_table[] =
     {
         .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GV100,
         .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GP100,
-        .u.access_counter_buffer_ops = {
-            .enable_access_counter_notifications  = uvm_hal_volta_enable_access_counter_notifications,
-            .disable_access_counter_notifications = uvm_hal_volta_disable_access_counter_notifications,
-            .clear_access_counter_notifications = uvm_hal_volta_clear_access_counter_notifications,
-            .parse_entry = uvm_hal_volta_access_counter_buffer_parse_entry,
-            .entry_is_valid = uvm_hal_volta_access_counter_buffer_entry_is_valid,
-            .entry_clear_valid = uvm_hal_volta_access_counter_buffer_entry_clear_valid,
-            .entry_size = uvm_hal_volta_access_counter_buffer_entry_size,
-        }
+        .u.access_counter_buffer_ops = {}
     },
     {
         .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_TU100,
         .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GV100,
         .u.access_counter_buffer_ops = {
+            .enable_access_counter_notifications  = uvm_hal_turing_enable_access_counter_notifications,
             .disable_access_counter_notifications = uvm_hal_turing_disable_access_counter_notifications,
             .clear_access_counter_notifications = uvm_hal_turing_clear_access_counter_notifications,
+            .parse_entry = uvm_hal_turing_access_counter_buffer_parse_entry,
+            .entry_is_valid = uvm_hal_turing_access_counter_buffer_entry_is_valid,
+            .entry_clear_valid = uvm_hal_turing_access_counter_buffer_entry_clear_valid,
+            .entry_size = uvm_hal_turing_access_counter_buffer_entry_size,
         }
     },
     {
@@ -503,6 +570,16 @@ static uvm_hal_class_ops_t access_counter_buffer_table[] =
         .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_AD100,
         .u.access_counter_buffer_ops = {}
     },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GB100,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GH100,
+        .u.access_counter_buffer_ops = {}
+    },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GB200,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GB100,
+        .u.access_counter_buffer_ops = {}
+    },
 };
 
 static uvm_hal_class_ops_t sec2_table[] =
@@ -514,6 +591,7 @@ static uvm_hal_class_ops_t sec2_table[] =
             .decrypt = uvm_hal_maxwell_sec2_decrypt_unsupported,
             .semaphore_release = uvm_hal_maxwell_sec2_semaphore_release_unsupported,
             .semaphore_timestamp = uvm_hal_maxwell_sec2_semaphore_timestamp_unsupported,
+            .semaphore_target_is_valid = uvm_hal_maxwell_semaphore_target_is_valid,
         }
     },
     {
@@ -556,6 +634,16 @@ static uvm_hal_class_ops_t sec2_table[] =
             .decrypt = uvm_hal_hopper_sec2_decrypt,
         }
     },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GB100,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GH100,
+        .u.sec2_ops = {}
+    },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GB200,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GB100,
+        .u.sec2_ops = {}
+    },
 };
 
 static inline uvm_hal_class_ops_t *ops_find_by_id(uvm_hal_class_ops_t *table, NvU32 row_count, NvU32 id)
@@ -631,13 +719,19 @@ NV_STATUS uvm_hal_init_table(void)
         return status;
     }
 
-    status = ops_init_from_parent(host_table, ARRAY_SIZE(host_table), HOST_OP_COUNT, offsetof(uvm_hal_class_ops_t, u.host_ops));
+    status = ops_init_from_parent(host_table,
+                                  ARRAY_SIZE(host_table),
+                                  HOST_OP_COUNT,
+                                  offsetof(uvm_hal_class_ops_t, u.host_ops));
     if (status != NV_OK) {
         UVM_ERR_PRINT("ops_init_from_parent(host_table) failed: %s\n", nvstatusToString(status));
         return status;
     }
 
-    status = ops_init_from_parent(arch_table, ARRAY_SIZE(arch_table), ARCH_OP_COUNT, offsetof(uvm_hal_class_ops_t, u.arch_ops));
+    status = ops_init_from_parent(arch_table,
+                                  ARRAY_SIZE(arch_table),
+                                  ARCH_OP_COUNT,
+                                  offsetof(uvm_hal_class_ops_t, u.arch_ops));
     if (status != NV_OK) {
         UVM_ERR_PRINT("ops_init_from_parent(arch_table) failed: %s\n", nvstatusToString(status));
         return status;
@@ -680,7 +774,9 @@ NV_STATUS uvm_hal_init_gpu(uvm_parent_gpu_t *parent_gpu)
 
     class_ops = ops_find_by_id(ce_table, ARRAY_SIZE(ce_table), gpu_info->ceClass);
     if (class_ops == NULL) {
-        UVM_ERR_PRINT("Unsupported ce class: 0x%X, GPU %s\n", gpu_info->ceClass, parent_gpu->name);
+        UVM_ERR_PRINT("Unsupported ce class: 0x%X, GPU %s\n",
+                      gpu_info->ceClass,
+                      uvm_parent_gpu_name(parent_gpu));
         return NV_ERR_INVALID_CLASS;
     }
 
@@ -688,7 +784,9 @@ NV_STATUS uvm_hal_init_gpu(uvm_parent_gpu_t *parent_gpu)
 
     class_ops = ops_find_by_id(host_table, ARRAY_SIZE(host_table), gpu_info->hostClass);
     if (class_ops == NULL) {
-        UVM_ERR_PRINT("Unsupported host class: 0x%X, GPU %s\n", gpu_info->hostClass, parent_gpu->name);
+        UVM_ERR_PRINT("Unsupported host class: 0x%X, GPU %s\n",
+                      gpu_info->hostClass,
+                      uvm_parent_gpu_name(parent_gpu));
         return NV_ERR_INVALID_CLASS;
     }
 
@@ -696,7 +794,9 @@ NV_STATUS uvm_hal_init_gpu(uvm_parent_gpu_t *parent_gpu)
 
     class_ops = ops_find_by_id(arch_table, ARRAY_SIZE(arch_table), gpu_info->gpuArch);
     if (class_ops == NULL) {
-        UVM_ERR_PRINT("Unsupported GPU architecture: 0x%X, GPU %s\n", gpu_info->gpuArch, parent_gpu->name);
+        UVM_ERR_PRINT("Unsupported GPU architecture: 0x%X, GPU %s\n",
+                      gpu_info->gpuArch,
+                      uvm_parent_gpu_name(parent_gpu));
         return NV_ERR_INVALID_CLASS;
     }
 
@@ -704,7 +804,9 @@ NV_STATUS uvm_hal_init_gpu(uvm_parent_gpu_t *parent_gpu)
 
     class_ops = ops_find_by_id(fault_buffer_table, ARRAY_SIZE(fault_buffer_table), gpu_info->gpuArch);
     if (class_ops == NULL) {
-        UVM_ERR_PRINT("Fault buffer HAL not found, GPU %s, arch: 0x%X\n", parent_gpu->name, gpu_info->gpuArch);
+        UVM_ERR_PRINT("Fault buffer HAL not found, GPU %s, arch: 0x%X\n",
+                      uvm_parent_gpu_name(parent_gpu),
+                      gpu_info->gpuArch);
         return NV_ERR_INVALID_CLASS;
     }
 
@@ -714,7 +816,9 @@ NV_STATUS uvm_hal_init_gpu(uvm_parent_gpu_t *parent_gpu)
                                ARRAY_SIZE(access_counter_buffer_table),
                                gpu_info->gpuArch);
     if (class_ops == NULL) {
-        UVM_ERR_PRINT("Access counter HAL not found, GPU %s, arch: 0x%X\n", parent_gpu->name, gpu_info->gpuArch);
+        UVM_ERR_PRINT("Access counter HAL not found, GPU %s, arch: 0x%X\n",
+                      uvm_parent_gpu_name(parent_gpu),
+                      gpu_info->gpuArch);
         return NV_ERR_INVALID_CLASS;
     }
 
@@ -722,7 +826,9 @@ NV_STATUS uvm_hal_init_gpu(uvm_parent_gpu_t *parent_gpu)
 
     class_ops = ops_find_by_id(sec2_table, ARRAY_SIZE(sec2_table), gpu_info->gpuArch);
     if (class_ops == NULL) {
-        UVM_ERR_PRINT("SEC2 HAL not found, GPU %s, arch: 0x%X\n", parent_gpu->name, gpu_info->gpuArch);
+        UVM_ERR_PRINT("SEC2 HAL not found, GPU %s, arch: 0x%X\n",
+                      uvm_parent_gpu_name(parent_gpu),
+                      gpu_info->gpuArch);
         return NV_ERR_INVALID_CLASS;
     }
 
@@ -733,13 +839,11 @@ NV_STATUS uvm_hal_init_gpu(uvm_parent_gpu_t *parent_gpu)
 
 static void hal_override_properties(uvm_parent_gpu_t *parent_gpu)
 {
-    // Access counters are currently not supported in vGPU.
+    // Access counters are currently not supported in vGPU or Confidential
+    // Computing.
     //
     // TODO: Bug 200692962: Add support for access counters in vGPU
-    if (parent_gpu->virt_mode != UVM_VIRT_MODE_NONE)
-        parent_gpu->access_counters_supported = false;
-    // Access counters are not supported in CC.
-    else if (uvm_conf_computing_mode_enabled_parent(parent_gpu))
+    if ((parent_gpu->virt_mode != UVM_VIRT_MODE_NONE) || g_uvm_global.conf_computing_enabled)
         parent_gpu->access_counters_supported = false;
 }
 
@@ -760,6 +864,9 @@ void uvm_hal_tlb_invalidate_membar(uvm_push_t *push, uvm_membar_t membar)
 
     gpu = uvm_push_get_gpu(push);
 
+    // TLB invalidate on Blackwell+ GPUs should not use a standalone membar.
+    UVM_ASSERT(gpu->parent->rm_info.gpuArch < NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GB100);
+
     for (i = 0; i < gpu->parent->num_hshub_tlb_invalidate_membars; i++)
         gpu->parent->host_hal->membar_gpu(push);
 
@@ -794,7 +901,7 @@ uvm_membar_t uvm_hal_downgrade_membar_type(uvm_gpu_t *gpu, bool is_local_vidmem)
     // memory, including those from other processors like the CPU or peer GPUs,
     // must come through this GPU's L2. In all current architectures, MEMBAR_GPU
     // is sufficient to resolve ordering at the L2 level.
-    if (is_local_vidmem && !uvm_gpu_is_coherent(gpu->parent) && !uvm_downgrade_force_membar_sys)
+    if (is_local_vidmem && !uvm_parent_gpu_is_coherent(gpu->parent) && !uvm_downgrade_force_membar_sys)
         return UVM_MEMBAR_GPU;
 
     // If the mapped memory was remote, or if a coherence protocol can cache
@@ -865,7 +972,7 @@ const char *uvm_fault_access_type_string(uvm_fault_access_type_t fault_access_ty
 
 const char *uvm_fault_type_string(uvm_fault_type_t fault_type)
 {
-    BUILD_BUG_ON(UVM_FAULT_TYPE_COUNT != 16);
+    BUILD_BUG_ON(UVM_FAULT_TYPE_COUNT != 17);
 
     switch (fault_type) {
         UVM_ENUM_STRING_CASE(UVM_FAULT_TYPE_INVALID_PDE);
@@ -884,6 +991,7 @@ const char *uvm_fault_type_string(uvm_fault_type_t fault_type)
         UVM_ENUM_STRING_CASE(UVM_FAULT_TYPE_UNSUPPORTED_KIND);
         UVM_ENUM_STRING_CASE(UVM_FAULT_TYPE_REGION_VIOLATION);
         UVM_ENUM_STRING_CASE(UVM_FAULT_TYPE_POISONED);
+        UVM_ENUM_STRING_CASE(UVM_FAULT_TYPE_CC_VIOLATION);
         UVM_ENUM_STRING_DEFAULT();
     }
 }
@@ -914,14 +1022,16 @@ const char *uvm_mmu_engine_type_string(uvm_mmu_engine_type_t mmu_engine_type)
 void uvm_hal_print_fault_entry(const uvm_fault_buffer_entry_t *entry)
 {
     UVM_DBG_PRINT("fault_address:                    0x%llx\n", entry->fault_address);
-    UVM_DBG_PRINT("    fault_instance_ptr:           {0x%llx:%s}\n", entry->instance_ptr.address,
-                                                                     uvm_aperture_string(entry->instance_ptr.aperture));
+    UVM_DBG_PRINT("    fault_instance_ptr:           {0x%llx:%s}\n",
+                  entry->instance_ptr.address,
+                  uvm_aperture_string(entry->instance_ptr.aperture));
     UVM_DBG_PRINT("    fault_type:                   %s\n", uvm_fault_type_string(entry->fault_type));
     UVM_DBG_PRINT("    fault_access_type:            %s\n", uvm_fault_access_type_string(entry->fault_access_type));
     UVM_DBG_PRINT("    is_replayable:                %s\n", entry->is_replayable? "true": "false");
     UVM_DBG_PRINT("    is_virtual:                   %s\n", entry->is_virtual? "true": "false");
     UVM_DBG_PRINT("    in_protected_mode:            %s\n", entry->in_protected_mode? "true": "false");
-    UVM_DBG_PRINT("    fault_source.client_type:     %s\n", uvm_fault_client_type_string(entry->fault_source.client_type));
+    UVM_DBG_PRINT("    fault_source.client_type:     %s\n",
+                  uvm_fault_client_type_string(entry->fault_source.client_type));
     UVM_DBG_PRINT("    fault_source.client_id:       %d\n", entry->fault_source.client_id);
     UVM_DBG_PRINT("    fault_source.gpc_id:          %d\n", entry->fault_source.gpc_id);
     UVM_DBG_PRINT("    fault_source.mmu_engine_id:   %d\n", entry->fault_source.mmu_engine_id);
@@ -930,34 +1040,15 @@ void uvm_hal_print_fault_entry(const uvm_fault_buffer_entry_t *entry)
     UVM_DBG_PRINT("    timestamp:                    %llu\n", entry->timestamp);
 }
 
-const char *uvm_access_counter_type_string(uvm_access_counter_type_t access_counter_type)
-{
-    BUILD_BUG_ON(UVM_ACCESS_COUNTER_TYPE_MAX != 2);
-
-    switch (access_counter_type) {
-        UVM_ENUM_STRING_CASE(UVM_ACCESS_COUNTER_TYPE_MIMC);
-        UVM_ENUM_STRING_CASE(UVM_ACCESS_COUNTER_TYPE_MOMC);
-        UVM_ENUM_STRING_DEFAULT();
-    }
-}
-
 void uvm_hal_print_access_counter_buffer_entry(const uvm_access_counter_buffer_entry_t *entry)
 {
-    if (!entry->address.is_virtual) {
-        UVM_DBG_PRINT("physical address: {0x%llx:%s}\n", entry->address.address,
-                                                         uvm_aperture_string(entry->address.aperture));
-    }
-    else {
-        UVM_DBG_PRINT("virtual address: 0x%llx\n", entry->address.address);
-        UVM_DBG_PRINT("    instance_ptr    {0x%llx:%s}\n", entry->virtual_info.instance_ptr.address,
-                                                    uvm_aperture_string(entry->virtual_info.instance_ptr.aperture));
-        UVM_DBG_PRINT("    mmu_engine_type %s\n", uvm_mmu_engine_type_string(entry->virtual_info.mmu_engine_type));
-        UVM_DBG_PRINT("    mmu_engine_id   %u\n", entry->virtual_info.mmu_engine_id);
-        UVM_DBG_PRINT("    ve_id           %u\n", entry->virtual_info.ve_id);
-    }
-
-    UVM_DBG_PRINT("    is_virtual      %u\n", entry->address.is_virtual);
-    UVM_DBG_PRINT("    counter_type    %s\n", uvm_access_counter_type_string(entry->counter_type));
+    UVM_DBG_PRINT("virtual address: 0x%llx\n", entry->address);
+    UVM_DBG_PRINT("    instance_ptr    {0x%llx:%s}\n",
+                  entry->instance_ptr.address,
+                  uvm_aperture_string(entry->instance_ptr.aperture));
+    UVM_DBG_PRINT("    mmu_engine_type %s\n", uvm_mmu_engine_type_string(entry->mmu_engine_type));
+    UVM_DBG_PRINT("    mmu_engine_id   %u\n", entry->mmu_engine_id);
+    UVM_DBG_PRINT("    ve_id           %u\n", entry->ve_id);
     UVM_DBG_PRINT("    counter_value   %u\n", entry->counter_value);
     UVM_DBG_PRINT("    subgranularity  0x%08x\n", entry->sub_granularity);
     UVM_DBG_PRINT("    bank            %u\n", entry->bank);
@@ -969,16 +1060,6 @@ bool uvm_hal_method_is_valid_stub(uvm_push_t *push, NvU32 method_address, NvU32
     return true;
 }
 
-bool uvm_hal_ce_memcopy_is_valid_stub(uvm_push_t *push, uvm_gpu_address_t dst, uvm_gpu_address_t src)
-{
-    return true;
-}
-
 void uvm_hal_ce_memcopy_patch_src_stub(uvm_push_t *push, uvm_gpu_address_t *src)
 {
 }
-
-bool uvm_hal_ce_memset_is_valid_stub(uvm_push_t *push, uvm_gpu_address_t dst, size_t num_elements, size_t element_size)
-{
-    return true;
-}
