@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2013-2019 NVidia Corporation
+    Copyright (c) 2013-2024 NVidia Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -320,7 +320,7 @@ typedef struct
 
 typedef struct
 {
-    NvProcessorUuid gpuUuidArray[UVM_MAX_GPUS];                    // IN
+    NvProcessorUuid gpuUuidArray[UVM_MAX_GPUS_V1];                 // IN
     NvU32           numGpus;                                       // IN
     NvU64           serverId                    NV_ALIGN_BYTES(8); // OUT
     NV_STATUS       rmStatus;                                      // OUT
@@ -344,9 +344,9 @@ typedef struct
 
 typedef struct
 {
-    NvProcessorUuid gpuUuidArray[UVM_MAX_GPUS]; // OUT
-    NvU32           validCount;                 // OUT
-    NV_STATUS       rmStatus;                   // OUT
+    NvProcessorUuid gpuUuidArray[UVM_MAX_GPUS_V1]; // OUT
+    NvU32           validCount;                    // OUT
+    NV_STATUS       rmStatus;                      // OUT
 } UVM_GET_GPU_UUID_TABLE_PARAMS;
 
 #if defined(WIN32) || defined(WIN64)
@@ -552,7 +552,7 @@ typedef struct
 
 typedef struct
 {
-    NvProcessorUuid gpu_uuid;    // IN
+    NvProcessorUuid gpu_uuid;    // IN/OUT
     NvBool          numaEnabled; // OUT
     NvS32           numaNodeId;  // OUT
     NvS32           rmCtrlFd;    // IN
@@ -633,6 +633,7 @@ typedef struct
     NvU64           requestedBase      NV_ALIGN_BYTES(8); // IN
     NvU64           length             NV_ALIGN_BYTES(8); // IN
     NvProcessorUuid preferredLocation;                    // IN
+    NvS32           preferredCpuNumaNode;                 // IN
     NV_STATUS       rmStatus;                             // OUT
 } UVM_SET_PREFERRED_LOCATION_PARAMS;
 
@@ -766,8 +767,19 @@ typedef struct
 #define UVM_MIGRATE_FLAGS_ALL                   (UVM_MIGRATE_FLAG_ASYNC | \
                                                  UVM_MIGRATE_FLAGS_TEST_ALL)
 
-// For pageable migrations, cpuNumaNode is used as the destination NUMA node if
-// destinationUuid is the CPU.
+// If NV_ERR_INVALID_ARGUMENT is returned it is because cpuMemoryNode is not
+// valid and the destination processor is the CPU. cpuMemoryNode is considered
+// invalid if:
+//      * it is less than -1,
+//      * it is equal to or larger than the maximum number of nodes, or
+//      * it corresponds to a registered GPU.
+//      * it is not in the node_possible_map set of nodes,
+//      * it does not have onlined memory
+//
+// For pageable migrations:
+//
+// In addition to the above, in the case of pageable memory, the
+// cpuMemoryNode is considered invalid if it's -1.
 //
 // If NV_WARN_NOTHING_TO_DO is returned, user-space is responsible for
 // completing the migration of the VA range described by userSpaceStart and
@@ -775,6 +787,7 @@ typedef struct
 //
 // If NV_ERR_MORE_PROCESSING_REQUIRED is returned, user-space is responsible
 // for re-trying with a different cpuNumaNode, starting at userSpaceStart.
+//
 #define UVM_MIGRATE                                                   UVM_IOCTL_BASE(51)
 typedef struct
 {
@@ -784,7 +797,7 @@ typedef struct
     NvU32           flags;                                // IN
     NvU64           semaphoreAddress   NV_ALIGN_BYTES(8); // IN
     NvU32           semaphorePayload;                     // IN
-    NvU32           cpuNumaNode;                          // IN
+    NvS32           cpuNumaNode;                          // IN
     NvU64           userSpaceStart     NV_ALIGN_BYTES(8); // OUT
     NvU64           userSpaceLength    NV_ALIGN_BYTES(8); // OUT
     NV_STATUS       rmStatus;                             // OUT
@@ -822,7 +835,8 @@ typedef struct
 
 //
 // Initialize any tracker object such as a queue or counter
-// UvmToolsCreateEventQueue, UvmToolsCreateProcessAggregateCounters, UvmToolsCreateProcessorCounters
+// UvmToolsCreateEventQueue, UvmToolsCreateProcessAggregateCounters,
+// UvmToolsCreateProcessorCounters.
 //
 #define UVM_TOOLS_INIT_EVENT_TRACKER                                  UVM_IOCTL_BASE(56)
 typedef struct
@@ -919,11 +933,9 @@ typedef struct
 typedef struct
 {
     NvU64     tablePtr                 NV_ALIGN_BYTES(8); // IN
-    NvU32     count;                                      // IN/OUT
     NV_STATUS rmStatus;                                   // OUT
 } UVM_TOOLS_GET_PROCESSOR_UUID_TABLE_PARAMS;
 
-
 //
 // UvmMapDynamicParallelismRegion
 //
@@ -1048,6 +1060,71 @@ typedef struct
     NV_STATUS               rmStatus;                                          // OUT
 } UVM_MAP_EXTERNAL_SPARSE_PARAMS;
 
+//
+// Used to initialise a secondary UVM file-descriptor which holds a
+// reference on the memory map to prevent it being torn down without
+// first notifying UVM. This is achieved by preventing mmap() calls on
+// the secondary file-descriptor so that on process exit
+// uvm_mm_release() will be called while the memory map is present
+// such that UVM can cleanly shutdown the GPU by handling faults
+// instead of cancelling them.
+//
+// This ioctl must be called after the primary file-descriptor has
+// been initialised with the UVM_INITIALIZE ioctl. The primary FD
+// should be passed in the uvmFd field and the UVM_MM_INITIALIZE ioctl
+// will hold a reference on the primary FD. Therefore uvm_release() is
+// guaranteed to be called after uvm_mm_release().
+//
+// Once this file-descriptor has been closed the UVM context is
+// effectively dead and subsequent operations requiring a memory map
+// will fail. Calling UVM_MM_INITIALIZE on a context that has already
+// been initialized via any FD will return NV_ERR_INVALID_STATE.
+//
+// Calling this with a non-UVM file-descriptor in uvmFd will return
+// NV_ERR_INVALID_ARGUMENT. Calling this on the same file-descriptor
+// as UVM_INITIALIZE or more than once on the same FD will return
+// NV_ERR_IN_USE.
+//
+// Not all platforms require this secondary file-descriptor. On those
+// platforms NV_WARN_NOTHING_TO_DO will be returned and users may
+// close the file-descriptor at anytime.
+#define UVM_MM_INITIALIZE                                             UVM_IOCTL_BASE(75)
+typedef struct
+{
+    NvS32                   uvmFd;    // IN
+    NV_STATUS               rmStatus; // OUT
+} UVM_MM_INITIALIZE_PARAMS;
+
+#define UVM_TOOLS_INIT_EVENT_TRACKER_V2                               UVM_IOCTL_BASE(76)
+typedef UVM_TOOLS_INIT_EVENT_TRACKER_PARAMS UVM_TOOLS_INIT_EVENT_TRACKER_V2_PARAMS;
+
+#define UVM_TOOLS_GET_PROCESSOR_UUID_TABLE_V2                         UVM_IOCTL_BASE(77)
+typedef UVM_TOOLS_GET_PROCESSOR_UUID_TABLE_PARAMS UVM_TOOLS_GET_PROCESSOR_UUID_TABLE_V2_PARAMS;
+
+//
+// UvmAllocDeviceP2P
+//
+#define UVM_ALLOC_DEVICE_P2P                                          UVM_IOCTL_BASE(78)
+typedef struct
+{
+    NvU64                   base                            NV_ALIGN_BYTES(8); // IN
+    NvU64                   length                          NV_ALIGN_BYTES(8); // IN
+    NvU64                   offset                          NV_ALIGN_BYTES(8); // IN
+    NvProcessorUuid         gpuUuid;                                           // IN
+    NvS32                   rmCtrlFd;                                          // IN
+    NvU32                   hClient;                                           // IN
+    NvU32                   hMemory;                                           // IN
+
+    NV_STATUS               rmStatus;                                          // OUT
+} UVM_ALLOC_DEVICE_P2P_PARAMS;
+
+#define UVM_CLEAR_ALL_ACCESS_COUNTERS                                 UVM_IOCTL_BASE(79)
+
+typedef struct
+{
+    NV_STATUS       rmStatus; // OUT
+} UVM_CLEAR_ALL_ACCESS_COUNTERS_PARAMS;
+
 //
 // Temporary ioctls which should be removed before UVM 8 release
 // Number backwards from 2047 - highest custom ioctl function number
