@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2013-2020 NVidia Corporation
+    Copyright (c) 2013-2024 NVidia Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -52,8 +52,18 @@ typedef enum
 
 typedef unsigned long long UvmStream;
 
-#define UVM_MAX_GPUS         NV_MAX_DEVICES
-#define UVM_MAX_PROCESSORS   (UVM_MAX_GPUS + 1)
+// The maximum number of sub-processors per parent GPU.
+#define UVM_PARENT_ID_MAX_SUB_PROCESSORS 8
+
+// The maximum number of GPUs changed when multiple MIG instances per
+// uvm_parent_gpu_t were added. The old version is kept as a convenience
+// for code that needs to maintain forward compatibility.
+#define UVM_MAX_GPUS_V1       NV_MAX_DEVICES
+#define UVM_MAX_PROCESSORS_V1 (UVM_MAX_GPUS_V1 + 1)
+#define UVM_MAX_GPUS          (NV_MAX_DEVICES * UVM_PARENT_ID_MAX_SUB_PROCESSORS)
+#define UVM_MAX_PROCESSORS    (UVM_MAX_GPUS + 1)
+
+#define UVM_PROCESSOR_MASK_SIZE ((UVM_MAX_PROCESSORS + (sizeof(NvU64) * 8) - 1) / (sizeof(NvU64) * 8))
 
 #define UVM_INIT_FLAGS_DISABLE_HMM                       ((NvU64)0x1)
 #define UVM_INIT_FLAGS_MULTI_PROCESS_SHARING_MODE        ((NvU64)0x2)
@@ -152,6 +162,8 @@ typedef enum {
 
 typedef struct
 {
+    // UUID of the physical GPU if the GPU is not SMC capable or SMC enabled,
+    // or the GPU instance UUID of the partition.
     NvProcessorUuid gpuUuid;
     NvU32           gpuMappingType;     // UvmGpuMappingType
     NvU32           gpuCachingType;     // UvmGpuCachingType
@@ -221,6 +233,19 @@ typedef union
     } rm_linux;
 } UvmAllocationPlatformParams;
 
+// Platform specific parameters for UvmAllocDeviceP2P
+typedef union
+{
+    struct {
+        // File descriptor for RM's control file
+        int ctrlFd;
+        // RM client handle
+        NvHandle hClient;
+        // RM allocation handle
+        NvHandle hMemory;
+    } rm_linux;
+} UvmDeviceP2PPlatformParams;
+
 //------------------------------------------------------------------------------
 //    Tools API types
 //------------------------------------------------------------------------------
@@ -404,7 +429,7 @@ typedef struct
     // or malign-double will have no effect on the field offsets.
     //
     NvU16 padding16Bits;
-    NvU32 padding32Bits;
+    NvU32 cpuId;              // CPU number of the faulting thread
     NvU64 address;            // faulting address
     NvU64 timeStamp;          // cpu time when the fault occurred
     NvU32 pid;                // process id causing the fault
@@ -466,6 +491,7 @@ typedef enum
     UvmFaultTypeInvalidPte                                                     = 2,
     UvmFaultTypeWrite                                                          = 3,
     UvmFaultTypeAtomic                                                         = 4,
+
     // The next fault types are fatal and cannot be serviced by the UVM driver
     UvmFaultTypeFatal                                                          = 5,
     UvmFaultTypeInvalidPdeSize                                                 = UvmFaultTypeFatal,
@@ -479,6 +505,8 @@ typedef enum
     UvmFaultTypeUnsupportedKind                                                = 13,
     UvmFaultTypeRegionViolation                                                = 14,
     UvmFaultTypePoison                                                         = 15,
+    UvmFaultTypeCcViolation                                                    = 16,
+
     // ---- Add new values above this line
     UvmEventNumFaultTypes
 } UvmEventFaultType;
@@ -493,7 +521,8 @@ typedef enum
     UvmEventFatalReasonInternalError      = 5,
 
     // This value is reported when a fault is triggered in an invalid context
-    // Example: CPU fault on a managed allocation while a kernel is running on a pre-Pascal GPU
+    // Example: CPU fault on a managed allocation while a kernel is running on a
+    // pre-Pascal GPU
     UvmEventFatalReasonInvalidOperation   = 6,
     // ---- Add new values above this line
     UvmEventNumFatalReasons
@@ -568,6 +597,48 @@ typedef struct
                                    // on the gpu
 } UvmEventMigrationInfo;
 
+typedef struct
+{
+    //
+    // eventType has to be the 1st argument of this structure. Setting eventType
+    // to UvmEventTypeMigration helps to identify event data in a queue.
+    //
+    NvU8 eventType;
+    //
+    // Cause that triggered the migration
+    //
+    NvU8 migrationCause;
+    //
+    // This structure is shared between UVM kernel and tools.
+    // Manually padding the structure so that compiler options like pragma pack
+    // or malign-double will have no effect on the field offsets
+    //
+    NvU16 padding16Bits;
+    //
+    // Indices are used for the source and destination of migration instead of
+    // using gpu uuid/cpu id. This reduces the size of each event. The index to
+    // gpuUuid relation can be obtained from UvmToolsGetProcessorUuidTable.
+    // Currently we do not distinguish between CPUs so they all use index 0.
+    //
+    NvU16 srcIndex;                // source CPU/GPU index
+    NvU16 dstIndex;                // destination CPU/GPU index
+    NvS32 srcNid;                  // source CPU NUMA node ID
+    NvS32 dstNid;                  // destination CPU NUMA node ID
+    NvU64 address;                 // base virtual addr used for migration
+    NvU64 migratedBytes;           // number of bytes migrated
+    NvU64 beginTimeStamp;          // cpu time stamp when the memory transfer
+                                   // was queued on the gpu
+    NvU64 endTimeStamp;            // cpu time stamp when the memory transfer
+                                   // finalization was communicated to the cpu
+                                   // For asynchronous operations this field
+                                   // will be zero
+    NvU64 rangeGroupId;            // range group tied with this migration
+    NvU64 beginTimeStampGpu;       // time stamp when the migration started
+                                   // on the gpu
+    NvU64 endTimeStampGpu;         // time stamp when the migration finished
+                                   // on the gpu
+} UvmEventMigrationInfo_V2;
+
 typedef enum
 {
     UvmEventFaultClientTypeInvalid            = 0,
@@ -634,6 +705,56 @@ typedef struct
     NvU16 padding16Bits;
 } UvmEventGpuFaultInfo;
 
+typedef struct
+{
+    //
+    // eventType has to be the 1st argument of this structure.
+    // Setting eventType = UvmEventTypeGpuFault helps to identify event data in
+    // a queue.
+    //
+    NvU8 eventType;
+    NvU8 faultType;       // type of gpu fault, refer UvmEventFaultType
+    NvU16 gpuIndex;       // GPU that experienced the fault
+    union
+    {
+        NvU16 gpcId;      // If this is a replayable fault, this field contains
+                          // the physical GPC index where the fault was
+                          // triggered
+
+        NvU16 channelId;  // If this is a non-replayable fault, this field
+                          // contains the id of the channel that launched the
+                          // operation that caused the fault.
+                          //
+                          // TODO: Bug 3283289: this field is ambiguous for
+                          // Ampere+ GPUs, but it is never consumed by clients.
+    };
+    NvU16 clientId;       // Id of the MMU client that triggered the fault. This
+                          // is the value provided by HW and is architecture-
+                          // specific. There are separate client ids for
+                          // different client types (See dev_fault.h).
+    NvU64 address;        // virtual address at which gpu faulted
+    NvU64 timeStamp;      // time stamp when the cpu started processing the
+                          // fault
+    NvU64 timeStampGpu;   // gpu time stamp when the fault entry was written
+                          // in the fault buffer
+    NvU32 batchId;        // Per-GPU unique id to identify the faults serviced
+                          // in batch before:
+                          // - Issuing a replay for replayable faults
+                          // - Re-scheduling the channel for non-replayable
+                          //   faults.
+    NvU8 clientType;      // Volta+ GPUs can fault on clients other than GR.
+                          // UvmEventFaultClientTypeGpc indicates replayable
+                          // fault, while UvmEventFaultClientTypeHub indicates
+                          // non-replayable fault.
+    NvU8 accessType;      // memory access type, refer UvmEventMemoryAccessType
+    //
+    // This structure is shared between UVM kernel and tools.
+    // Manually padding the structure so that compiler options like pragma pack
+    // or malign-double will have no effect on the field offsets
+    //
+    NvU16 padding16bits;
+} UvmEventGpuFaultInfo_V2;
+
 //------------------------------------------------------------------------------
 // This info is provided when a gpu fault is replayed (for replayable faults)
 // or when the channel that launched the operation that triggered the fault is
@@ -659,14 +780,32 @@ typedef struct
     // or malign-double will have no effect on the field offsets
     //
     NvU8 padding8bits;
-    NvU32 batchId;          // Per-GPU unique id to identify the faults that have
-                            // been serviced in batch
+    NvU32 batchId;          // Per-GPU unique id to identify the faults that
+                            // have been serviced in batch
     NvU64 timeStamp;        // cpu time when the replay of the faulting memory
                             // accesses is queued on the gpu
     NvU64 timeStampGpu;     // gpu time stamp when the replay operation finished
                             // executing on the gpu
 } UvmEventGpuFaultReplayInfo;
 
+typedef struct
+{
+    //
+    // eventType has to be the 1st argument of this structure.
+    // Setting eventType = UvmEventTypeGpuFaultReplay helps to identify event
+    // data in a queue.
+    //
+    NvU8 eventType;
+    NvU8 clientType;        // See clientType in UvmEventGpuFaultInfo
+    NvU16 gpuIndex;         // GPU that experienced the fault
+    NvU32 batchId;          // Per-GPU unique id to identify the faults that
+                            // have been serviced in batch
+    NvU64 timeStamp;        // cpu time when the replay of the faulting memory
+                            // accesses is queued on the gpu
+    NvU64 timeStampGpu;     // gpu time stamp when the replay operation finished
+                            // executing on the gpu
+} UvmEventGpuFaultReplayInfo_V2;
+
 //------------------------------------------------------------------------------
 // This info is provided per fatal fault
 //------------------------------------------------------------------------------
@@ -674,21 +813,41 @@ typedef struct
 {
     //
     // eventType has to be the 1st argument of this structure.
-    // Setting eventType = UvmEventTypeFatalFault helps to identify event data in
-    // a queue.
+    // Setting eventType = UvmEventTypeFatalFault helps to identify event data
+    // in a queue.
     //
     NvU8 eventType;
-    NvU8 faultType;       // type of gpu fault, refer UvmEventFaultType. Only valid
-                          // if processorIndex is a GPU
+    NvU8 faultType;       // type of gpu fault, refer UvmEventFaultType. Only
+                          // valid if processorIndex is a GPU
     NvU8 accessType;      // memory access type, refer UvmEventMemoryAccessType
     NvU8 processorIndex;  // processor that experienced the fault
-    NvU8 reason;          // reason why the fault is fatal, refer UvmEventFatalReason
+    NvU8 reason;          // reason why the fault is fatal, refer
+                          // UvmEventFatalReason
     NvU8 padding8bits;
     NvU16 padding16bits;
     NvU64 address;        // virtual address at which the processor faulted
     NvU64 timeStamp;      // CPU time when the fault is detected to be fatal
 } UvmEventFatalFaultInfo;
 
+typedef struct
+{
+    //
+    // eventType has to be the 1st argument of this structure.
+    // Setting eventType = UvmEventTypeFatalFault helps to identify event data
+    // in a queue.
+    //
+    NvU8 eventType;
+    NvU8 faultType;       // type of gpu fault, refer UvmEventFaultType. Only
+                          // valid if processorIndex is a GPU
+    NvU8 accessType;      // memory access type, refer UvmEventMemoryAccessType
+    NvU8 reason;          // reason why the fault is fatal, refer
+                          // UvmEventFatalReason
+    NvU16 processorIndex; // processor that experienced the fault
+    NvU16 padding16bits;
+    NvU64 address;        // virtual address at which the processor faulted
+    NvU64 timeStamp;      // CPU time when the fault is detected to be fatal
+} UvmEventFatalFaultInfo_V2;
+
 typedef struct
 {
     //
@@ -722,8 +881,8 @@ typedef struct
 {
     //
     // eventType has to be the 1st argument of this structure.
-    // Setting eventType = UvmEventTypeReadDuplicateInvalidate helps to
-    // identify event data in a queue.
+    // Setting eventType = UvmEventTypeReadDuplicate helps to identify event
+    // data in a queue.
     //
     NvU8 eventType;
     //
@@ -731,8 +890,39 @@ typedef struct
     // Manually padding the structure so that compiler options like pragma pack
     // or malign-double will have no effect on the field offsets
     //
+    NvU8  padding8bits;
+    NvU16 padding16bits;
+    NvU32 padding32bits;
+    NvU64 address;          // virtual address of the memory region that is
+                            // read-duplicated
+    NvU64 size;             // size in bytes of the memory region that is
+                            // read-duplicated
+    NvU64 timeStamp;        // cpu time stamp when the memory region becomes
+                            // read-duplicate. Since many processors can
+                            // participate in read-duplicate this is time stamp
+                            // when all the operations have been pushed to all
+                            // the processors.
+    NvU64 processors[UVM_PROCESSOR_MASK_SIZE];
+                            // mask that specifies in which processors this
+                            // memory region is read-duplicated. This is last
+                            // so UVM_PROCESSOR_MASK_SIZE can grow.
+} UvmEventReadDuplicateInfo_V2;
+
+typedef struct
+{
+    //
+    // eventType has to be the 1st argument of this structure.
+    // Setting eventType = UvmEventTypeReadDuplicateInvalidate helps to
+    // identify event data in a queue.
+    //
+    NvU8 eventType;
     NvU8 residentIndex;     // index of the cpu/gpu that now contains the only
                             // valid copy of the memory region
+    //
+    // This structure is shared between UVM kernel and tools.
+    // Manually padding the structure so that compiler options like pragma pack
+    // or malign-double will have no effect on the field offsets
+    //
     NvU16 padding16bits;
     NvU32 padding32bits;
     NvU64 address;          // virtual address of the memory region that is
@@ -746,6 +936,32 @@ typedef struct
                             // the processors.
 } UvmEventReadDuplicateInvalidateInfo;
 
+typedef struct
+{
+    //
+    // eventType has to be the 1st argument of this structure.
+    // Setting eventType = UvmEventTypeReadDuplicateInvalidate helps to
+    // identify event data in a queue.
+    //
+    NvU8 eventType;
+    NvU8 padding8bits;
+    NvU16 residentIndex;
+    //
+    // This structure is shared between UVM kernel and tools.
+    // Manually padding the structure so that compiler options like pragma pack
+    // or malign-double will have no effect on the field offsets
+    //
+    NvU32 padding32bits;
+    NvU64 address;          // virtual address of the memory region that is
+                            // read-duplicated
+    NvU64 size;             // size of the memory region that is
+                            // read-duplicated
+    NvU64 timeStamp;        // cpu time stamp when the memory region is no
+                            // longer read-duplicate. Since many processors can
+                            // participate in read-duplicate this is time stamp
+                            // when all the operations have been pushed to all
+                            // the processors.
+} UvmEventReadDuplicateInvalidateInfo_V2;
 
 typedef struct
 {
@@ -770,6 +986,29 @@ typedef struct
                             // queued on the gpu
 } UvmEventPageSizeChangeInfo;
 
+typedef struct
+{
+    //
+    // eventType has to be the 1st argument of this structure.
+    // Setting eventType = UvmEventTypePageSizeChange helps to identify event
+    // data in a queue.
+    //
+    NvU8 eventType;
+    //
+    // This structure is shared between UVM kernel and tools.
+    // Manually padding the structure so that compiler options like pragma pack
+    // or malign-double will have no effect on the field offsets
+    //
+    NvU8 padding8bits;
+    NvU16 processorIndex;   // cpu/gpu processor index for which the page size
+                            // changed
+    NvU32 size;             // new page size
+    NvU64 address;          // virtual address of the page whose size has
+                            // changed
+    NvU64 timeStamp;        // cpu time stamp when the new page size is
+                            // queued on the gpu
+} UvmEventPageSizeChangeInfo_V2;
+
 typedef struct
 {
     //
@@ -798,8 +1037,34 @@ typedef struct
 {
     //
     // eventType has to be the 1st argument of this structure.
-    // Setting eventType = UvmEventTypeThrottlingStart helps to identify event data
-    // in a queue.
+    // Setting eventType = UvmEventTypeThrashingDetected helps to identify event
+    // data in a queue.
+    //
+    NvU8 eventType;
+    //
+    // This structure is shared between UVM kernel and tools.
+    // Manually padding the structure so that compiler options like pragma pack
+    // or malign-double will have no effect on the field offsets
+    //
+    NvU8 padding8bits;
+    NvU16 padding16bits;
+    NvU32 padding32bits;
+    NvU64 address;          // virtual address of the memory region that is
+                            // thrashing
+    NvU64 size;             // size of the memory region that is thrashing
+    NvU64 timeStamp;        // cpu time stamp when thrashing is detected
+    NvU64 processors[UVM_PROCESSOR_MASK_SIZE];
+                            // mask that specifies which processors are
+                            // fighting for this memory region. This is last
+                            // so UVM_PROCESSOR_MASK_SIZE can grow.
+} UvmEventThrashingDetectedInfo_V2;
+
+typedef struct
+{
+    //
+    // eventType has to be the 1st argument of this structure.
+    // Setting eventType = UvmEventTypeThrottlingStart helps to identify event
+    // data in a queue.
     //
     NvU8 eventType;
     NvU8 processorIndex;    // index of the cpu/gpu that was throttled
@@ -819,8 +1084,29 @@ typedef struct
 {
     //
     // eventType has to be the 1st argument of this structure.
-    // Setting eventType = UvmEventTypeThrottlingEnd helps to identify event data
-    // in a queue.
+    // Setting eventType = UvmEventTypeThrottlingStart helps to identify event
+    // data in a queue.
+    //
+    NvU8 eventType;
+    //
+    // This structure is shared between UVM kernel and tools.
+    // Manually padding the structure so that compiler options like pragma pack
+    // or malign-double will have no effect on the field offsets
+    //
+    NvU8  padding8bits;
+    NvU16 processorIndex;   // index of the cpu/gpu that was throttled
+    NvU32 padding32bits;
+    NvU64 address;          // address of the page whose servicing is being
+                            // throttled
+    NvU64 timeStamp;        // cpu start time stamp for the throttling operation
+} UvmEventThrottlingStartInfo_V2;
+
+typedef struct
+{
+    //
+    // eventType has to be the 1st argument of this structure.
+    // Setting eventType = UvmEventTypeThrottlingEnd helps to identify event
+    // data in a queue.
     //
     NvU8 eventType;
     NvU8 processorIndex;    // index of the cpu/gpu that was throttled
@@ -836,6 +1122,27 @@ typedef struct
     NvU64 timeStamp;        // cpu end time stamp for the throttling operation
 } UvmEventThrottlingEndInfo;
 
+typedef struct
+{
+    //
+    // eventType has to be the 1st argument of this structure.
+    // Setting eventType = UvmEventTypeThrottlingEnd helps to identify event
+    // data in a queue.
+    //
+    NvU8 eventType;
+    //
+    // This structure is shared between UVM kernel and tools.
+    // Manually padding the structure so that compiler options like pragma pack
+    // or malign-double will have no effect on the field offsets
+    //
+    NvU8  padding8bits;
+    NvU16 processorIndex;   // index of the cpu/gpu that was throttled
+    NvU32 padding32bits;
+    NvU64 address;          // address of the page whose servicing is being
+                            // throttled
+    NvU64 timeStamp;        // cpu end time stamp for the throttling operation
+} UvmEventThrottlingEndInfo_V2;
+
 typedef enum
 {
     UvmEventMapRemoteCauseInvalid     = 0,
@@ -892,6 +1199,35 @@ typedef struct
                             // is a cpu, this field will be zero.
 } UvmEventMapRemoteInfo;
 
+typedef struct
+{
+    //
+    // eventType has to be the 1st argument of this structure.
+    // Setting eventType = UvmEventTypeMapRemote helps to identify event data
+    // in a queue.
+    //
+    NvU8 eventType;
+    NvU8 mapRemoteCause;    // field to type UvmEventMapRemoteCause that tells
+                            // the cause for the page to be mapped remotely
+    //
+    // This structure is shared between UVM kernel and tools.
+    // Manually padding the structure so that compiler options like pragma pack
+    // or malign-double will have no effect on the field offsets
+    //
+    NvU16 padding16bits;
+    NvU16 srcIndex;         // index of the cpu/gpu being remapped
+    NvU16 dstIndex;         // index of the cpu/gpu memory that contains the
+                            // memory region data
+    NvU64 address;          // virtual address of the memory region that is
+                            // thrashing
+    NvU64 size;             // size of the memory region that is thrashing
+    NvU64 timeStamp;        // cpu time stamp when all the required operations
+                            // have been pushed to the processor
+    NvU64 timeStampGpu;     // time stamp when the new mapping is effective in
+                            // the processor specified by srcIndex. If srcIndex
+                            // is a cpu, this field will be zero.
+} UvmEventMapRemoteInfo_V2;
+
 typedef struct
 {
     //
@@ -918,6 +1254,32 @@ typedef struct
     NvU64 timeStamp;        // cpu time stamp when eviction starts on the cpu
 } UvmEventEvictionInfo;
 
+typedef struct
+{
+    //
+    // eventType has to be the 1st argument of this structure.
+    // Setting eventType = UvmEventTypeEviction helps to identify event data
+    // in a queue.
+    //
+    NvU8 eventType;
+    //
+    // This structure is shared between UVM kernel and tools.
+    // Manually padding the structure so that compiler options like pragma pack
+    // or malign-double will have no effect on the field offsets
+    //
+    NvU8  padding8bits;
+    NvU16 padding16bits;
+    NvU16 srcIndex;         // index of the cpu/gpu from which data is being
+                            // evicted
+    NvU16 dstIndex;         // index of the cpu/gpu memory to which data is
+                            // going to be stored
+    NvU64 addressOut;       // virtual address of the memory region that is
+                            // being evicted
+    NvU64 addressIn;        // virtual address that caused the eviction
+    NvU64 size;             // size of the memory region that being evicted
+    NvU64 timeStamp;        // cpu time stamp when eviction starts on the cpu
+} UvmEventEvictionInfo_V2;
+
 // TODO: Bug 1870362: [uvm] Provide virtual address and processor index in
 // AccessCounter events
 //
@@ -946,8 +1308,8 @@ typedef struct
 {
     //
     // eventType has to be the 1st argument of this structure.
-    // Setting eventType = UvmEventTypeAccessCounter helps to identify event data
-    // in a queue.
+    // Setting eventType = UvmEventTypeAccessCounter helps to identify event
+    // data in a queue.
     //
     NvU8 eventType;
     NvU8 srcIndex;          // index of the gpu that received the access counter
@@ -961,12 +1323,11 @@ typedef struct
     NvU8 aperture;
     NvU8 instancePtrAperture;
 
-    NvU8 isVirtual;
-    NvU8 isFromCpu;
+    NvU8 padding8bits;
 
     NvU8 veId;
-    NvU8 onManaged;         // The access counter notification was triggered on
-                            // a managed memory region
+
+    NvU16 padding16bits;
 
     NvU32 value;
     NvU32 subGranularity;
@@ -976,6 +1337,38 @@ typedef struct
     NvU64 instancePtr;
 } UvmEventTestAccessCounterInfo;
 
+typedef struct
+{
+    //
+    // eventType has to be the 1st argument of this structure.
+    // Setting eventType = UvmEventTypeAccessCounter helps to identify event
+    // data in a queue.
+    //
+    NvU8 eventType;
+
+    // See uvm_access_counter_buffer_entry_t for details
+    NvU8 aperture;
+    NvU8 instancePtrAperture;
+    NvU8 veId;
+
+    //
+    // This structure is shared between UVM kernel and tools.
+    // Manually padding the structure so that compiler options like pragma pack
+    // or malign-double will have no effect on the field offsets
+    //
+    NvU16 padding16bits;
+
+    NvU16 srcIndex;         // index of the gpu that received the access counter
+                            // notification
+    NvU32 value;
+    NvU32 subGranularity;
+    NvU32 tag;
+    NvU32 bank;
+    NvU32 padding32bits;
+    NvU64 address;
+    NvU64 instancePtr;
+} UvmEventTestAccessCounterInfo_V2;
+
 typedef struct
 {
     NvU8 eventType;
@@ -1019,6 +1412,40 @@ typedef struct
     };
 } UvmEventEntry;
 
+typedef struct
+{
+    union
+    {
+        union
+        {
+            NvU8 eventType;
+            UvmEventMigrationInfo_Lite migration_Lite;
+
+            UvmEventCpuFaultInfo cpuFault;
+            UvmEventMigrationInfo_V2 migration;
+            UvmEventGpuFaultInfo_V2 gpuFault;
+            UvmEventGpuFaultReplayInfo_V2 gpuFaultReplay;
+            UvmEventFatalFaultInfo_V2 fatalFault;
+            UvmEventReadDuplicateInfo_V2 readDuplicate;
+            UvmEventReadDuplicateInvalidateInfo_V2 readDuplicateInvalidate;
+            UvmEventPageSizeChangeInfo_V2 pageSizeChange;
+            UvmEventThrashingDetectedInfo_V2 thrashing;
+            UvmEventThrottlingStartInfo_V2 throttlingStart;
+            UvmEventThrottlingEndInfo_V2 throttlingEnd;
+            UvmEventMapRemoteInfo_V2 mapRemote;
+            UvmEventEvictionInfo_V2 eviction;
+        } eventData;
+
+        union
+        {
+            NvU8 eventType;
+
+            UvmEventTestAccessCounterInfo_V2 accessCounter;
+            UvmEventTestSplitInvalidateInfo splitInvalidate;
+        } testEventData;
+    };
+} UvmEventEntry_V2;
+
 //------------------------------------------------------------------------------
 // Type of time stamp used in the event entry:
 //
@@ -1060,6 +1487,7 @@ typedef struct UvmEventControlData_tag {
     // entries between get_ahead and get_behind are currently being read
     volatile NvU32 get_ahead;
     volatile NvU32 get_behind;
+
     // entries between put_ahead and put_behind are currently being written
     volatile NvU32 put_ahead;
     volatile NvU32 put_behind;
