@@ -48,6 +48,14 @@
 #include "class/clc670.h"
 #include "class/clc67d.h"
 #include "class/clc770.h"
+#include "class/clc870.h"
+#include "class/clc87d.h"
+#include "class/clc970.h"
+#include "class/clc97d.h"
+#include "class/clca70.h"
+#include "class/clca7d.h"
+
+#include "hdmi_spec.h"
 
 // Class hierarchy structure
 typedef struct tagNVHDMIPKT_CLASS_HIERARCHY
@@ -85,7 +93,7 @@ typedef struct tagNVHDMIPKT_CLASS_HIERARCHY
  ************************************************************************************************/
 static const NVHDMIPKT_CLASS_HIERARCHY hierarchy[] =
 {
-    {// Index 0==NVHDMIPKT_0073_CLASS
+    [NVHDMIPKT_0073_CLASS] = {// Index 0==NVHDMIPKT_0073_CLASS
         NVHDMIPKT_0073_CLASS,             // classId
         NVHDMIPKT_0073_CLASS,             // parentClassId
         NV_TRUE,                          // isRootClass
@@ -95,7 +103,7 @@ static const NVHDMIPKT_CLASS_HIERARCHY hierarchy[] =
         0,                                // displayClass
         0                                 // coreDmaClass
     },
-    {// Index 1==NVHDMIPKT_9171_CLASS
+    [NVHDMIPKT_9171_CLASS] = {// Index 1==NVHDMIPKT_9171_CLASS
         NVHDMIPKT_9171_CLASS,             // classId
         NVHDMIPKT_9171_CLASS,             // parentClassId
         NV_TRUE,                          // isRootClass
@@ -105,7 +113,7 @@ static const NVHDMIPKT_CLASS_HIERARCHY hierarchy[] =
         NV9170_DISPLAY,                   // displayClass
         NV917D_CORE_CHANNEL_DMA           // coreDmaClass
     },
-    {// Index 2==NVHDMIPKT_9271_CLASS
+    [NVHDMIPKT_9271_CLASS] = {// Index 2==NVHDMIPKT_9271_CLASS
         NVHDMIPKT_9271_CLASS,             // classId
         NVHDMIPKT_9171_CLASS,             // parentClassId
         NV_FALSE,                         // isRootClass
@@ -115,7 +123,7 @@ static const NVHDMIPKT_CLASS_HIERARCHY hierarchy[] =
         NV9270_DISPLAY,                   // displayClass
         NV927D_CORE_CHANNEL_DMA           // coreDmaClass
     },
-    {// Index 3==NVHDMIPKT_9471_CLASS
+    [NVHDMIPKT_9471_CLASS] = {// Index 3==NVHDMIPKT_9471_CLASS
         NVHDMIPKT_9471_CLASS,             // classId
         NVHDMIPKT_9171_CLASS,             // parentClassId
         NV_FALSE,                         // isRootClass
@@ -125,7 +133,7 @@ static const NVHDMIPKT_CLASS_HIERARCHY hierarchy[] =
         NV9470_DISPLAY,                   // displayClass
         NV947D_CORE_CHANNEL_DMA           // coreDmaClass
     },
-    {// Index 4==NVHDMIPKT_9571_CLASS
+    [NVHDMIPKT_9571_CLASS] = {// Index 4==NVHDMIPKT_9571_CLASS
         NVHDMIPKT_9571_CLASS,             // classId
         NVHDMIPKT_9171_CLASS,             // parentClassId
         NV_FALSE,                         // isRootClass
@@ -135,7 +143,7 @@ static const NVHDMIPKT_CLASS_HIERARCHY hierarchy[] =
         NV9570_DISPLAY,                   // displayClass
         NV957D_CORE_CHANNEL_DMA           // coreDmaClass
     },
-    {// Index 5==NVHDMIPKT_C371_CLASS
+    [NVHDMIPKT_C371_CLASS] = {// Index 5==NVHDMIPKT_C371_CLASS
         NVHDMIPKT_C371_CLASS,             // classId
         NVHDMIPKT_9171_CLASS,             // parentClassId
         NV_FALSE,                         // isRootClass
@@ -145,7 +153,7 @@ static const NVHDMIPKT_CLASS_HIERARCHY hierarchy[] =
         NVC370_DISPLAY,                   // displayClass
         NVC37D_CORE_CHANNEL_DMA           // coreDmaClass
     },
-    {// Index 6==NVHDMIPKT_C571_CLASS
+    [NVHDMIPKT_C571_CLASS] = {// Index 6==NVHDMIPKT_C571_CLASS
      // Note that Turing (C57x) has a distinct displayClass and coreDmaClass,
      // but it inherits the _DISP_SF_USER class from Volta (C37x).  We call this
      // NVHDMIPKT_C571_CLASS, but reuse initInterface()/constructor()/destructor()
@@ -159,7 +167,7 @@ static const NVHDMIPKT_CLASS_HIERARCHY hierarchy[] =
         NVC570_DISPLAY,                   // displayClass
         NVC57D_CORE_CHANNEL_DMA           // coreDmaClass
     },
-    {// Index 7==NVHDMIPKT_C671_CLASS
+    [NVHDMIPKT_C671_CLASS] = {// Index 7==NVHDMIPKT_C671_CLASS
         NVHDMIPKT_C671_CLASS,             // classId
         NVHDMIPKT_9171_CLASS,             // parentClassId
         NV_FALSE,                         // isRootClass
@@ -169,7 +177,7 @@ static const NVHDMIPKT_CLASS_HIERARCHY hierarchy[] =
         NVC670_DISPLAY,                   // displayClass
         NVC67D_CORE_CHANNEL_DMA           // coreDmaClass
     },
-    {// Index 8==NVHDMIPKT_C771_CLASS
+    [NVHDMIPKT_C771_CLASS] = {// Index 8==NVHDMIPKT_C771_CLASS
         NVHDMIPKT_C771_CLASS,             // classId
         NVHDMIPKT_C671_CLASS,             // parentClassId
         NV_FALSE,                         // isRootClass
@@ -179,6 +187,36 @@ static const NVHDMIPKT_CLASS_HIERARCHY hierarchy[] =
         NVC770_DISPLAY,                   // displayClass
         NVC67D_CORE_CHANNEL_DMA           // coreDmaClass
     },
+    [NVHDMIPKT_C871_CLASS] = {// Index 9==NVHDMIPKT_C871_CLASS
+        NVHDMIPKT_C871_CLASS,             // classId
+        NVHDMIPKT_C671_CLASS,             // parentClassId
+        NV_FALSE,                         // isRootClass
+        initializeHdmiPktInterfaceC871,   // initInterface
+        hdmiConstructorC871,              // constructor
+        hdmiDestructorC871,               // destructor
+        NVC870_DISPLAY,                   // displayClass
+        NVC87D_CORE_CHANNEL_DMA           // coreDmaClass
+    },
+    [NVHDMIPKT_C971_CLASS] = {// Index 10==NVHDMIPKT_C971_CLASS
+        NVHDMIPKT_C971_CLASS,             // classId
+        NVHDMIPKT_C871_CLASS,             // parentClassId
+        NV_FALSE,                         // isRootClass
+        initializeHdmiPktInterfaceC971,   // initInterface
+        hdmiConstructorC971,              // constructor
+        hdmiDestructorC971,               // destructor
+        NVC970_DISPLAY,                   // displayClass
+        NVC97D_CORE_CHANNEL_DMA           // coreDmaClass
+    },
+    [NVHDMIPKT_CA71_CLASS] = {// Index 11==NVHDMIPKT_CA71_CLASS
+        NVHDMIPKT_CA71_CLASS,             // classId
+        NVHDMIPKT_C971_CLASS,             // parentClassId
+        NV_FALSE,                         // isRootClass
+        initializeHdmiPktInterfaceCA71,   // initInterface
+        hdmiConstructorCA71,              // constructor
+        hdmiDestructorCA71,               // destructor
+        NVCA70_DISPLAY,                   // displayClass
+        NVCA7D_CORE_CHANNEL_DMA           // coreDmaClass
+    },
 };
 
 /********************************** HDMI Library interfaces *************************************/
@@ -233,6 +271,23 @@ NvHdmiPkt_PacketWrite(NvHdmiPkt_Handle  libHandle,
         return NVHDMIPKT_INVALID_ARG;
     }
 
+    HDMI_PACKET_TYPE infoframeType = pPacket[0]; // header byte 0 is packet type
+    // Lower bound check. Since actual infoframe size varies depending on the infoframe packet being sent, 
+    // check all supported infoframe types and their expected sizes. This is not a strict == check becuase they may/may not need
+    // additional checksum byte (library clients take care of adding checksum byte if needed)
+    if (((infoframeType == hdmi_pktType_GeneralControl)                 && (packetLen < 6))                                             ||
+        ((infoframeType == hdmi_pktType_GamutMetadata)                  && (packetLen < sizeof(NVT_GAMUT_METADATA)))                    ||
+        ((infoframeType == hdmi_pktType_ExtendedMetadata)               && (packetLen < sizeof(NVT_EXTENDED_METADATA_PACKET_INFOFRAME)))||
+        ((infoframeType == hdmi_pktType_VendorSpecInfoFrame)            && (packetLen < 8))                                             ||
+        ((infoframeType == hdmi_pktType_AviInfoFrame)                   && (packetLen < 13))                                            ||
+        ((infoframeType == hdmi_pktType_SrcProdDescInfoFrame)           && (packetLen < sizeof(NVT_SPD_INFOFRAME)))                     ||
+        ((infoframeType == hdmi_pktType_DynamicRangeMasteringInfoFrame) && (packetLen < sizeof(NVT_HDR_INFOFRAME))))
+        //  Unused: hdmi_pktType_AudioClkRegeneration
+        //  Unused: hdmi_pktType_MpegSrcInfoFrame
+    {
+        NvHdmiPkt_Print(pClass, "WARNING - packet length too small for infoframe type %d check payload ", infoframeType);
+    }
+
     return pClass->hdmiPacketWrite(pClass,
                                    subDevice,
                                    displayId,
@@ -243,6 +298,61 @@ NvHdmiPkt_PacketWrite(NvHdmiPkt_Handle  libHandle,
                                    pPacket);
 }
 
+/*
+ * NvHdmiPkt_PacketRead
+ */
+NVHDMIPKT_RESULT
+NvHdmiPkt_PacketRead(NvHdmiPkt_Handle    libHandle,
+                     NvU32               subDevice,
+                     NvU32               head,
+                     NVHDMIPKT_TYPE      packetReg,
+                     NvU32               bufferLen,
+                     NvU8 *const         pOutPktBuffer)
+{
+    if (libHandle == NVHDMIPKT_INVALID_HANDLE)
+    {
+        return NVHDMIPKT_LIBRARY_INIT_FAIL;
+    }
+
+    NVHDMIPKT_CLASS* pClass = fromHdmiPktHandle(libHandle);
+
+    if ((pOutPktBuffer == NULL) || (bufferLen == 0))
+    {
+        return NVHDMIPKT_INVALID_ARG;
+    }
+
+    return pClass->hdmiPacketRead(pClass,
+                                  subDevice,
+                                  head,
+                                  packetReg,
+                                  bufferLen,
+                                  pOutPktBuffer);
+}
+
+/*
+ * NvHdmiPkt_SetupAdvancedInfoframe
+ */
+NVHDMIPKT_RESULT
+NvHdmiPkt_SetupAdvancedInfoframe(NvHdmiPkt_Handle          libHandle,
+                                 NvU32                     subDevice,
+                                 NvU32                     head,
+                                 NVHDMIPKT_TYPE            packetReg,
+                                 ADVANCED_INFOFRAME const *pInfoframe)
+{
+    if (libHandle == NVHDMIPKT_INVALID_HANDLE)
+    {
+        return NVHDMIPKT_LIBRARY_INIT_FAIL;
+    }
+
+    NVHDMIPKT_CLASS* pClass = fromHdmiPktHandle(libHandle);
+
+    return pClass->programAdvancedInfoframe(pClass,
+                                            subDevice,
+                                            head,
+                                            packetReg,
+                                            pInfoframe);
+}
+
 NVHDMIPKT_RESULT
 NvHdmi_AssessLinkCapabilities(NvHdmiPkt_Handle             libHandle,
                               NvU32                        subDevice,
