@@ -569,8 +569,28 @@ typedef struct
 
     // Ganged Link table
     NvU64 *ganged_link_table;
+
+    //
+    // Mask of links on the LR10 device connected to a disabled
+    // remote link
+    //
+    NvU64 disabledRemoteEndLinkMask;
+
+    //
+    // Bool indicating if disabledRemoteEndLinkMask
+    // has been cached previously
+    //
+    NvBool bDisabledRemoteEndLinkMaskCached;
 } lr10_device;
 
+#define NVSWITCH_NUM_DEVICES_PER_DELTA_LR10 6
+
+typedef struct {
+    NvU32 switchPhysicalId;
+    NvU64 accessLinkMask;
+    NvU64 trunkLinkMask;
+} lr10_links_connected_to_disabled_remote_end;
+
 #define NVSWITCH_GET_CHIP_DEVICE_LR10(_device)                  \
     (                                                           \
         ((_device)->chip_id == NV_PSMC_BOOT_42_CHIP_ID_LR10) ?  \
@@ -632,6 +652,7 @@ void      nvswitch_setup_link_loopback_mode_lr10(nvswitch_device *device, NvU32
 void nvswitch_reset_persistent_link_hw_state_lr10(nvswitch_device *device, NvU32 linkNumber);
 void nvswitch_store_topology_information_lr10(nvswitch_device *device, nvlink_link *link);
 void nvswitch_init_lpwr_regs_lr10(nvlink_link *link);
+void nvswitch_program_l1_scratch_reg_lr10(nvswitch_device *device, NvU32 linkNumber);
 NvlStatus nvswitch_set_training_mode_lr10(nvswitch_device *device);
 NvBool nvswitch_i2c_is_device_access_allowed_lr10(nvswitch_device *device, NvU32 port, NvU8 addr, NvBool bIsRead);
 NvU32     nvswitch_get_sublink_width_lr10(nvswitch_device *device,NvU32 linkNumber);
@@ -643,8 +664,8 @@ NvlStatus nvswitch_ctrl_get_nvlink_lp_counters_lr10(nvswitch_device *device, NVS
 NvlStatus nvswitch_service_nvldl_fatal_link_lr10(nvswitch_device *device, NvU32 nvliptInstance, NvU32 link);
 NvlStatus nvswitch_ctrl_inband_send_data_lr10(nvswitch_device *device, NVSWITCH_INBAND_SEND_DATA_PARAMS *p);
 NvlStatus nvswitch_ctrl_inband_read_data_lr10(nvswitch_device *device, NVSWITCH_INBAND_READ_DATA_PARAMS *p);
-void nvswitch_send_inband_nack_lr10(nvswitch_device *device, NvU32 *msghdr, NvU32  linkId);
-NvU32 nvswitch_get_max_persistent_message_count_lr10(nvswitch_device *device);
+void      nvswitch_send_inband_nack_lr10(nvswitch_device *device, NvU32 *msghdr, NvU32  linkId);
+NvU32     nvswitch_get_max_persistent_message_count_lr10(nvswitch_device *device);
 NvlStatus nvswitch_launch_ALI_link_training_lr10(nvswitch_device *device, nvlink_link *link, NvBool bSync);
 NvlStatus nvswitch_service_minion_link_lr10(nvswitch_device *device, NvU32 nvliptInstance);
 void      nvswitch_apply_recal_settings_lr10(nvswitch_device *device, nvlink_link *link);
@@ -653,6 +674,14 @@ void      nvswitch_setup_link_system_registers_lr10(nvswitch_device *device, nvl
 void      nvswitch_load_link_disable_settings_lr10(nvswitch_device *device, nvlink_link *link);
 NvBool    nvswitch_is_smbpbi_supported_lr10(nvswitch_device *device);
 NvlStatus nvswitch_ctrl_get_board_part_number_lr10(nvswitch_device *device, NVSWITCH_GET_BOARD_PART_NUMBER_VECTOR *p);
+NvlStatus nvswitch_ctrl_get_link_l1_capability_lr10(nvswitch_device *device, NvU32 linkId, NvBool *isL1Capable);
+NvlStatus nvswitch_ctrl_get_link_l1_threshold_lr10(nvswitch_device *device, NvU32 linkNum, NvU32 *lpThreshold);
+NvlStatus nvswitch_ctrl_set_link_l1_threshold_lr10(nvlink_link *link, NvU32 lpEntryThreshold);
+NvlStatus nvswitch_get_board_id_lr10(nvswitch_device *device, NvU16 *boardId);
+
+NvlStatus nvswitch_ctrl_get_soe_heartbeat_lr10(nvswitch_device *device, NVSWITCH_GET_SOE_HEARTBEAT_PARAMS *p);
+void      nvswitch_update_link_state_led_lr10(nvswitch_device *device);
+void      nvswitch_led_shutdown_lr10(nvswitch_device *device);
 
 NvlStatus nvswitch_ctrl_set_mc_rid_table_lr10(nvswitch_device *device, NVSWITCH_SET_MC_RID_TABLE_PARAMS *p);
 NvlStatus nvswitch_ctrl_get_mc_rid_table_lr10(nvswitch_device *device, NVSWITCH_GET_MC_RID_TABLE_PARAMS *p);
@@ -660,5 +689,26 @@ NvlStatus nvswitch_launch_ALI_lr10(nvswitch_device *device);
 NvlStatus nvswitch_reset_and_train_link_lr10(nvswitch_device *device, nvlink_link *link);
 
 NvlStatus nvswitch_ctrl_get_bios_info_lr10(nvswitch_device *device, NVSWITCH_GET_BIOS_INFO_PARAMS *p);
-
+NvBool    nvswitch_does_link_need_termination_enabled_lr10(nvswitch_device *device, nvlink_link *link);
+NvlStatus nvswitch_link_termination_setup_lr10(nvswitch_device *device, nvlink_link* link);
+void      nvswitch_fsp_update_cmdq_head_tail_lr10(nvswitch_device  *device, NvU32 queueHead, NvU32 queueTail);
+void      nvswitch_fsp_get_cmdq_head_tail_lr10(nvswitch_device  *device, NvU32 *pQueueHead, NvU32 *pQueueTail);
+void      nvswitch_fsp_update_msgq_head_tail_lr10(nvswitch_device *device, NvU32 msgqHead, NvU32 msgqTail);
+void      nvswitch_fsp_get_msgq_head_tail_lr10(nvswitch_device *device, NvU32 *pMsgqHead, NvU32 *pMsgqTail);
+NvU32     nvswitch_fsp_get_channel_size_lr10(nvswitch_device *device);
+NvU8      nvswitch_fsp_nvdm_to_seid_lr10(nvswitch_device *device, NvU8 nvdmType);
+NvU32     nvswitch_fsp_create_mctp_header_lr10(nvswitch_device *device, NvU8 som, NvU8 eom, NvU8 seid, NvU8 seq);
+NvU32     nvswitch_fsp_create_nvdm_header_lr10(nvswitch_device *device, NvU32 nvdmType);
+NvlStatus nvswitch_fsp_get_packet_info_lr10(nvswitch_device *device, NvU8 *pBuffer, NvU32 size, NvU8 *pPacketState, NvU8 *pTag);
+NvlStatus nvswitch_fsp_validate_mctp_payload_header_lr10(nvswitch_device *device, NvU8 *pBuffer, NvU32 size);
+NvlStatus nvswitch_fsp_process_nvdm_msg_lr10(nvswitch_device  *device, NvU8 *pBuffer, NvU32 size);
+NvlStatus nvswitch_fsp_process_cmd_response_lr10(nvswitch_device *device, NvU8 *pBuffer, NvU32 size);
+NvlStatus nvswitch_fsp_config_ememc_lr10(nvswitch_device *device, NvU32 offset, NvBool bAincw, NvBool bAincr);
+NvlStatus nvswitch_fsp_write_to_emem_lr10(nvswitch_device *device, NvU8 *pBuffer, NvU32 size);
+NvlStatus nvswitch_fsp_read_from_emem_lr10(nvswitch_device *device, NvU8 *pBuffer, NvU32 size);
+NvlStatus nvswitch_fsp_error_code_to_nvlstatus_map_lr10(nvswitch_device *device, NvU32 errorCode);
+NvlStatus nvswitch_tnvl_get_attestation_certificate_chain_lr10(nvswitch_device *device, NVSWITCH_GET_ATTESTATION_CERTIFICATE_CHAIN_PARAMS *params);
+NvlStatus nvswitch_tnvl_get_attestation_report_lr10(nvswitch_device *device, NVSWITCH_GET_ATTESTATION_REPORT_PARAMS *params);
+NvlStatus nvswitch_tnvl_get_status_lr10(nvswitch_device *device, NVSWITCH_GET_TNVL_STATUS_PARAMS *params);
+void      nvswitch_tnvl_disable_interrupts_lr10(nvswitch_device *device);
 #endif //_LR10_H_
