@@ -159,7 +159,7 @@ typedef struct _NVHsChannelStatisticsOneEyeRec {
 
 typedef struct _NVHsChannelFlipQueueEntry {
     NVListRec flipQueueEntry;
-    NVFlipChannelEvoHwState hwState;
+    NVHsLayerRequestedFlipState hwState;
 } NVHsChannelFlipQueueEntry;
 
 typedef struct _NVHsChannelEvoRec {
@@ -180,15 +180,15 @@ typedef struct _NVHsChannelEvoRec {
     } nv3d;
 
     struct {
-        NvU32 handle;
+        NvU32 handle[1];
     } nv2d;
 
     /*
-     * NvKmsFlipParams is too large to declare on the stack.  We preallocate it
-     * here so that we don't have to allocate and free it on every headSurface
-     * flip.
+     * Flip request parameters are too large to declare on the stack.  We
+     * preallocate them here so that we don't have to allocate and free them on
+     * every headSurface flip.
      */
-    struct NvKmsFlipParams scratchParams;
+    struct NvKmsFlipRequestOneHead scratchParams;
 
     /*
      * The index into NVDevEvoRec::apiHeadSurfaceAllDisps[apiHead]::surface[] to use
@@ -246,7 +246,7 @@ typedef struct _NVHsChannelEvoRec {
      */
 
     struct {
-        NVFlipChannelEvoHwState current;
+        NVHsLayerRequestedFlipState current;
         NVListRec queue;
     } flipQueue[NVKMS_MAX_LAYERS_PER_HEAD];
 
@@ -272,10 +272,10 @@ typedef struct _NVHsChannelEvoRec {
     NvBool usingRgIntrForSwapGroups;
 
     /*
-     * Handle to the RG line interrupt callback object. This is needed to
+     * Pointer to the RG line interrupt callback object. This is needed to
      * enabled and disable the RG interrupt callback.
      */
-    NvU32 rgIntrCallbackObjectHandle;
+    NVRgLine1CallbackPtr pRgIntrCallback;
 
 #if NVKMS_PROCFS_ENABLE
 
@@ -495,7 +495,7 @@ static inline void HsChangeSurfaceFlipRefCount(
 }
 
 /*!
- * Get the last NVFlipChannelEvoHwState entry in the pHsChannel's flip queue for
+ * Get the last NVHsLayerRequestedFlipState entry in the pHsChannel's flip queue for
  * the specified layer.
  *
  * If the flip queue is empty, return the 'current' entry.  Otherwise, return
@@ -503,7 +503,7 @@ static inline void HsChangeSurfaceFlipRefCount(
  *
  * This function cannot fail.
  */
-static inline const NVFlipChannelEvoHwState *HsGetLastFlipQueueEntry(
+static inline const NVHsLayerRequestedFlipState *HsGetLastFlipQueueEntry(
     const NVHsChannelEvoRec *pHsChannel,
     const NvU8 layer)
 {
