@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -39,7 +39,9 @@
  *   | VGA WORKSPACE            |
  *   ---------------------------- <- vbiosReservedOffset  (64K? aligned)
  *   | (potential align. gap)   |
- *   ---------------------------- <- gspFwWprEnd (128K aligned)
+ *   ---------------------------- <- gspFwWprEnd + frtsSize + pmuReservedSize
+ *   | PMU mem reservation      |
+ *   ---------------------------- <- gspFwWprEnd (128K aligned) + frtsSize
  *   | FRTS data                |    (frtsSize is 0 on GA100)
  *   | ------------------------ | <- frtsOffset
  *   | BOOT BIN (e.g. SK + BL)  |
@@ -80,8 +82,26 @@ typedef struct
     NvU64 bootloaderDataOffset;
     NvU64 bootloaderManifestOffset;
 
-    NvU64 sysmemAddrOfSignature;
-    NvU64 sizeOfSignature;
+    union
+    {
+        // Used only at initial boot
+        struct
+        {
+            NvU64 sysmemAddrOfSignature;
+            NvU64 sizeOfSignature;
+        };
+
+        //
+        // Used at suspend/resume to read GspFwHeapFreeList
+        // Offset relative to GspFwWprMeta FBMEM PA (gspFwWprStart)
+        //
+        struct
+        {
+            NvU32 gspFwHeapFreeListWprOffset;
+            NvU32 unused0;
+            NvU64 unused1;
+        };
+    };
 
     // ---- Members describing FB layout --------------------------------
     NvU64 gspFwRsvdStart;
@@ -119,36 +139,69 @@ typedef struct
     // Boot count.  Used to determine whether to load the firmware image.
     NvU64 bootCount;
 
-    // TODO: the partitionRpc* fields below do not really belong in this
-    //       structure. The values are patched in by the partition bootstrapper
-    //       when GSP-RM is booted in a partition, and this structure was a
-    //       convenient place for the bootstrapper to access them. These should
-    //       be moved to a different comm. mechanism between the bootstrapper
-    //       and the GSP-RM tasks.
+    // This union is organized the way it is to start at an 8-byte boundary and achieve natural
+    // packing of the internal struct fields.
+    union
+    {
+        struct
+        {
+            // TODO: the partitionRpc* fields below do not really belong in this
+            //       structure. The values are patched in by the partition bootstrapper
+            //       when GSP-RM is booted in a partition, and this structure was a
+            //       convenient place for the bootstrapper to access them. These should
+            //       be moved to a different comm. mechanism between the bootstrapper
+            //       and the GSP-RM tasks.
 
-    // Shared partition RPC memory (physical address)
-    NvU64 partitionRpcAddr;
+            // Shared partition RPC memory (physical address)
+            NvU64 partitionRpcAddr;
 
-    // Offsets relative to partitionRpcAddr
-    NvU16 partitionRpcRequestOffset;
-    NvU16 partitionRpcReplyOffset;
+            // Offsets relative to partitionRpcAddr
+            NvU16 partitionRpcRequestOffset;
+            NvU16 partitionRpcReplyOffset;
 
-    // Code section and dataSection offset and size.
-    NvU32 elfCodeOffset;
-    NvU32 elfDataOffset;
-    NvU32 elfCodeSize;
-    NvU32 elfDataSize;
+            // Code section and dataSection offset and size.
+            NvU32 elfCodeOffset;
+            NvU32 elfDataOffset;
+            NvU32 elfCodeSize;
+            NvU32 elfDataSize;
 
-    // Used during GSP-RM resume to check for revocation
-    NvU32 lsUcodeVersion;
+            // Used during GSP-RM resume to check for revocation
+            NvU32 lsUcodeVersion;
+        };
+
+        struct
+        {
+            // Pad for the partitionRpc* fields, plus 4 bytes
+            NvU32 partitionRpcPadding[4];
+
+            // CrashCat (contiguous) buffer size/location - occupies same bytes as the
+            // elf(Code|Data)(Offset|Size) fields above.
+            // TODO: move to GSP_FMC_INIT_PARAMS
+            NvU64 sysmemAddrOfCrashReportQueue;
+            NvU32 sizeOfCrashReportQueue;
+
+            // Pad for the lsUcodeVersion field
+            NvU32 lsUcodeVersionPadding[1];
+        };
+    };
 
     // Number of VF partitions allocating sub-heaps from the WPR heap
     // Used during boot to ensure the heap is adequately sized
     NvU8 gspFwHeapVfPartitionCount;
 
+    // Flags to help decide GSP-FW flow.
+    NvU8 flags;
+
     // Pad structure to exactly 256 bytes.  Can replace padding with additional
     // fields without incrementing revision.  Padding initialized to 0.
-    NvU8 padding[7];
+    NvU8 padding[2];
+
+    //
+    // Starts at gspFwWprEnd+frtsSize b/c FRTS is positioned
+    // to end where this allocation starts (when RM requests FSP to create
+    // FRTS).
+    //
+    NvU32 pmuReservedSize;
 
     // BL to use for verification (i.e. Booter says OK to boot)
     NvU64 verified;  // 0x0 -> unverified, 0xa0a0a0a0a0a0a0a0 -> verified
@@ -158,4 +211,30 @@ typedef struct
 #define GSP_FW_WPR_META_REVISION  1
 #define GSP_FW_WPR_META_MAGIC     0xdc3aae21371a60b3ULL
 
+#define GSP_FW_WPR_HEAP_FREE_REGION_COUNT 128
+
+typedef struct
+{
+    //
+    // offset relative to GspFwWprMeta FBMEM PA
+    // describes a region at [offs, offs + length)
+    //
+    NvU32 offs; // start, inclusive
+    NvU32 length;
+} GspFwHeapFreeRegion;
+
+typedef struct
+{
+    NvU64 magic;
+    NvU32 nregions;
+    GspFwHeapFreeRegion regions[GSP_FW_WPR_HEAP_FREE_REGION_COUNT];
+} GspFwHeapFreeList;
+
+#define GSP_FW_HEAP_FREE_LIST_MAGIC 0x4845415046524545ULL
+
+#define GSP_FW_FLAGS                            8:0
+#define GSP_FW_FLAGS_CLOCK_BOOST                NVBIT(0)
+#define GSP_FW_FLAGS_RECOVERY_MARGIN_PRESENT    NVBIT(1)
+#define GSP_FW_FLAGS_PPCIE_ENABLED              NVBIT(2)
+
 #endif // GSP_FW_WPR_META_H_
