@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2020-2020 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2020-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -25,6 +25,7 @@
 
 #include "rmapi/rmapi.h"
 
+
 //
 // Alloc a client, device and subdevice handle for a gpu
 //
@@ -58,6 +59,10 @@ NvBool rmapiutilIsExternalClassIdInternalOnly(NvU32 externalClassId);
 //
 // Return the flags and access right associated with this RM control command
 //
-NV_STATUS rmapiutilGetControlInfo(NvU32 cmd, NvU32 *pFlags, NvU32 *pAccessRight);
+
+NV_STATUS rmapiutilGetControlInfo(NvU32 cmd, NvU32 *pFlags,
+                                  NvU32 *pAccessRight, NvU32 *pParamsSize);
+
+NvBool rmapiutilSkipErrorMessageForUnsupportedVgpuGuestControl(OBJGPU *pGpu, NvU32 cmd);
 
 #endif /* RMAPI_UTILS_H */
