@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -23,6 +23,7 @@
 
 #include "core/core.h"
 #include "gpu/gpu.h"
+#include "gpu/device/device.h"
 #include "mem_mgr/vaspace.h"
 #include "mem_mgr/io_vaspace.h"
 #include "mem_mgr/gpu_vaspace.h"
@@ -33,13 +34,15 @@
 #include "kernel/gpu/nvbitmask.h"
 #include "platform/chipset/chipset.h"
 #include "rmapi/client.h"
+#include "platform/sli/sli.h"
 #include "nvdevid.h"
+#include "containers/eheap_old.h"
+#include "gpu/bus/p2p_api.h"
 
-#include "gpu/subdevice/subdevice.h"
 #include "gpu/gsp/gsp_static_config.h"
 #include "vgpu/rpc.h"
 
-#include "nvRmReg.h"
+#include "nvrm_registry.h"
 
 static NV_STATUS kbusInitRegistryOverrides(OBJGPU *pGpu, KernelBus *pKernelBus);
 
@@ -181,7 +184,32 @@ kbusInitRegistryOverrides(OBJGPU *pGpu, KernelBus *pKernelBus)
             break;
     }
 
-    if (RMCFG_FEATURE_PLATFORM_WINDOWS_LDDM && !pGpu->getProperty(pGpu, PDB_PROP_GPU_IN_TCC_MODE))
+    if ((osReadRegistryDword(pGpu, NV_REG_STR_RM_FORCE_STATIC_BAR1, &data32) == NV_OK) &&
+        data32 < NV_REG_STR_RM_FORCE_STATIC_BAR1_MAX)
+    {
+        pKernelBus->staticBar1ForceType = data32;
+        NV_PRINTF(LEVEL_WARNING, "Forcing static BAR1 to type %d.\n", data32);
+    }
+    else
+    {
+        pKernelBus->staticBar1ForceType = NV_REG_STR_RM_FORCE_STATIC_BAR1_DEFAULT;
+    }
+
+    //
+    // NV_REG_STR_RM_GPUDIRECT_RDMA_FORCE_SPA regkey
+    // is only used on coherent systems with BAR1 enabled.
+    //
+    pKernelBus->bGrdmaForceSpa = NV_REG_STR_RM_GPUDIRECT_RDMA_FORCE_SPA_DEFAULT;
+
+    if (osReadRegistryDword(pGpu, NV_REG_STR_RM_GPUDIRECT_RDMA_FORCE_SPA, &data32) == NV_OK)
+    {
+        if (data32 == NV_REG_STR_RM_GPUDIRECT_RDMA_FORCE_SPA_YES)
+        {
+            pKernelBus->bGrdmaForceSpa = NV_TRUE;
+        }
+    }
+
+    if (RMCFG_FEATURE_PLATFORM_WINDOWS && !pGpu->getProperty(pGpu, PDB_PROP_GPU_IN_TCC_MODE))
     {
         //
         // Aligns to unlinked SLI: Volta and up
@@ -196,31 +224,46 @@ kbusInitRegistryOverrides(OBJGPU *pGpu, KernelBus *pKernelBus)
         pKernelBus->bP2pMailboxClientAllocated = !!data32;
     }
 
+    if (osReadRegistryDword(pGpu, NV_REG_STR_RESTORE_BAR1_SIZE_BUG_3249028_WAR, &data32) == NV_OK)
+    {
+        pKernelBus->setProperty(pKernelBus, PDB_PROP_KBUS_RESTORE_BAR1_SIZE_BUG_3249028_WAR, !!data32);
+    }
+
+    if (osReadRegistryDword(pGpu, NV_REG_STR_RM_FORCE_BAR_ACCESS_ON_HCC, &data32) == NV_OK &&
+        data32 == NV_REG_STR_RM_FORCE_BAR_ACCESS_ON_HCC_YES  && gpuIsCCDevToolsModeEnabled(pGpu))
+    {
+        pKernelBus->bForceBarAccessOnHcc = NV_TRUE;
+    }
+
     return NV_OK;
 }
 
 /**
- * @brief  Gets the BAR1 VA range for a client
+ * @brief  Gets the BAR1 VA range for a device
  *
  * @param[in] pGpu
  * @param[in] pKernelBus
- * @param[in] hClient               Client handle
+ * @param[in] pDevice               Device pointer
  * @param[out] pBar1VARange         BAR1 VA range
  */
 
 NV_STATUS
-kbusGetBar1VARangeForClient_IMPL(OBJGPU *pGpu, KernelBus *pKernelBus, NvHandle hClient, NV_RANGE *pBar1VARange)
+kbusGetBar1VARangeForDevice_IMPL(OBJGPU *pGpu, KernelBus *pKernelBus, Device *pDevice, NV_RANGE *pBar1VARange)
 {
     KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
     OBJVASPACE       *pBar1VAS          = kbusGetBar1VASpace_HAL(pGpu, pKernelBus);
+    RmClient         *pRmClient;
 
     NV_ASSERT_OR_RETURN(pBar1VAS != NULL, NV_ERR_INVALID_STATE);
 
    *pBar1VARange = rangeMake(vaspaceGetVaStart(pBar1VAS), vaspaceGetVaLimit(pBar1VAS));
 
+    pRmClient = dynamicCast(RES_GET_CLIENT(pDevice), RmClient);
+    NV_ASSERT_OR_RETURN(pRmClient != NULL, NV_ERR_INVALID_CLIENT);
+
     if ((pKernelMIGManager != NULL) && kmigmgrIsMIGMemPartitioningEnabled(pGpu, pKernelMIGManager) &&
-        !rmclientIsCapableByHandle(hClient, NV_RM_CAP_SYS_SMC_MONITOR) &&
-        !kmigmgrIsClientUsingDeviceProfiling(pGpu, pKernelMIGManager, hClient))
+        !rmclientIsCapable(pRmClient, NV_RM_CAP_SYS_SMC_MONITOR) &&
+        !kmigmgrIsDeviceUsingDeviceProfiling(pGpu, pKernelMIGManager, pDevice))
     {
         MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
         KernelMemorySystem *pKernelMemorySystem = GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu);
@@ -228,8 +271,8 @@ kbusGetBar1VARangeForClient_IMPL(OBJGPU *pGpu, KernelBus *pKernelBus, NvHandle h
 
        *pBar1VARange = memmgrGetMIGPartitionableBAR1Range(pGpu, pMemoryManager);
 
-        NV_ASSERT_OK_OR_RETURN(kmigmgrGetInstanceRefFromClient(pGpu, pKernelMIGManager,
-                                   hClient, &ref));
+        NV_ASSERT_OK_OR_RETURN(kmigmgrGetInstanceRefFromDevice(pGpu, pKernelMIGManager,
+                                   pDevice, &ref));
         NV_ASSERT_OK_OR_RETURN(kmemsysSwizzIdToMIGMemRange(pGpu, pKernelMemorySystem, ref.pKernelMIGGpuInstance->swizzId,
                                    *pBar1VARange, pBar1VARange));
     }
@@ -307,12 +350,31 @@ kbusDestruct_IMPL(KernelBus *pKernelBus)
 {
     OBJGPU *pGpu = ENG_GET_GPU(pKernelBus);
 
+    memdescFree(pKernelBus->bar1[GPU_GFID_PF].pInstBlkMemDesc);
+    memdescDestroy(pKernelBus->bar1[GPU_GFID_PF].pInstBlkMemDesc);
+    pKernelBus->bar1[GPU_GFID_PF].pInstBlkMemDesc = NULL;
+    memdescFree(pKernelBus->bar2[GPU_GFID_PF].pInstBlkMemDesc);
+    memdescDestroy(pKernelBus->bar2[GPU_GFID_PF].pInstBlkMemDesc);
+    pKernelBus->bar2[GPU_GFID_PF].pInstBlkMemDesc = NULL;
+
     //
     // We need to clean-up the memory resources for BAR2 as late as possible,
     // and after all memory descriptors have been reclaimed.
     //
     kbusDestructVirtualBar2_HAL(pGpu, pKernelBus, NV_TRUE, GPU_GFID_PF);
 
+    // Unmap BAR2 mapping that was retained for RPC
+    if (IS_VGPU_GSP_PLUGIN_OFFLOAD_ENABLED(pGpu) &&
+        IS_VIRTUAL_WITH_SRIOV(pGpu) &&
+        pKernelBus->virtualBar2[GPU_GFID_PF].pCpuMapping != NULL)
+    {
+        // vgpuDestructObject should have been run
+        OBJVGPU *pVGpu = GPU_GET_VGPU(pGpu);
+        NV_ASSERT(pVGpu == NULL);
+
+        osUnmapPciMemoryKernelOld(pGpu, pKernelBus->virtualBar2[GPU_GFID_PF].pCpuMapping);
+    }
+
     return;
 }
 
@@ -332,13 +394,6 @@ kbusSendSysmembar_IMPL
         return NV_OK;
     }
 
-    if (kbusIsFbFlushDisabled(pKernelBus))
-    {
-        // Eliminate FB flushes, but keep mmu invalidates
-        NV_PRINTF(LEVEL_INFO, "disable_fb_flush flag, skipping flush.\n");
-        return NV_OK;
-    }
-
     // Wait for the flush to flow through
     SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY | SLI_LOOP_FLAGS_IGNORE_REENTRANCY);
         pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
@@ -393,7 +448,9 @@ kbusCommitBar2_KERNEL
     NvU32      flags
 )
 {
-    if (!KBUS_BAR0_PRAMIN_DISABLED(pGpu))
+    if (!KBUS_BAR0_PRAMIN_DISABLED(pGpu) &&
+        !kbusIsBarAccessBlocked(pKernelBus) &&
+        !(flags & GPU_STATE_FLAGS_GC6_TRANSITION))
     {
         // we will initialize bar2 to the default big page size of the system
         NV_ASSERT_OK_OR_RETURN(kbusInitVirtualBar2_HAL(pGpu, pKernelBus));
@@ -613,7 +670,7 @@ kbusCpuOffsetInBar2WindowGet_IMPL
  * @return VA limit of BAR2
  */
 NvU64
-kbusGetVaLimitForBar2_KERNEL
+kbusGetVaLimitForBar2_FWCLIENT
 (
     OBJGPU    *pGpu,
     KernelBus *pKernelBus
@@ -632,6 +689,56 @@ kbusGetVaLimitForBar2_KERNEL
     return limit;
 }
 
+/*!
+ * @brief Calculates the BAR2 VA limit (in Byte units)
+ * Can be safely called only after kbusSetBarsApertureSize_HAL is executed.
+ */
+NvU64 kbusGetVaLimitForBar2_IMPL
+(
+    OBJGPU    *pGpu,
+    KernelBus *pKernelBus
+)
+{
+    NvU64 limit = 0;
+    NvU32 gfid;
+
+    NV_ASSERT_OR_RETURN(vgpuGetCallingContextGfid(pGpu, &gfid) == NV_OK, 0);
+
+    //
+    // Return zero from the guest in the paravirtualization case or
+    // if guest is running in SRIOV heavy mode.
+    //
+    if (IS_VIRTUAL_WITHOUT_SRIOV(pGpu) ||
+        (IS_VIRTUAL(pGpu) && gpuIsWarBug200577889SriovHeavyEnabled(pGpu)))
+    {
+        return 0;
+    }
+
+    kbusCalcCpuInvisibleBar2Range_HAL(pGpu, pKernelBus, gfid);
+
+    //
+    // we are just accounting here for possibility that
+    // we are on pre_PASCAL and so we set the va limit
+    // to account only for the cpuVisible Aperture
+    //
+    if (pKernelBus->bar2[gfid].cpuInvisibleLimit > pKernelBus->bar2[gfid].cpuInvisibleBase)
+    {
+        limit = pKernelBus->bar2[gfid].cpuInvisibleLimit;
+    }
+    else
+    {
+        limit = pKernelBus->bar2[gfid].cpuVisibleLimit;
+    }
+    NV_PRINTF(LEVEL_INFO, "va limit: 0x%llx\n", limit);
+    //
+    // pKernelBus->bar2.vaLimit is set by this function.
+    // Assert to ensure that this value doesn't get changed.
+    //
+    NV_ASSERT(pKernelBus->bar2[gfid].vaLimit == 0 || pKernelBus->bar2[gfid].vaLimit == limit);
+
+    return limit;
+}
+
 /*!
  * Patch CPU-RM's SW cache of BAR1 PDB to GSP-RM's BAR1 PDB so that CPU-RM can
  * do TLB invalidation to correct VA space.
@@ -655,7 +762,7 @@ kbusPatchBar1Pdb_GSPCLIENT
     MEMORY_DESCRIPTOR   *pMemDesc  = NULL;
     GVAS_GPU_STATE      *pGpuState = gvaspaceGetGpuState(pGVAS, pGpu);
     const MMU_FMT_LEVEL *pRootFmt  = pGpuState->pFmt->pRoot;
-    NvU32                rootSize  = pRootFmt->entrySize;
+    NvU32                rootSize  = mmuFmtLevelSize(pRootFmt);
     MMU_WALK_USER_CTX    userCtx   = {0};
     GspStaticConfigInfo *pGSCI     = GPU_GET_GSP_STATIC_INFO(pGpu);
 
@@ -666,7 +773,9 @@ kbusPatchBar1Pdb_GSPCLIENT
     memdescDescribe(pMemDesc, ADDR_FBMEM, pGSCI->bar1PdeBase, rootSize);
     memdescSetPageSize(pMemDesc, VAS_ADDRESS_TRANSLATION(pKernelBus->bar1[GPU_GFID_PF].pVAS), RM_PAGE_SIZE);
 
-    gvaspaceWalkUserCtxAcquire(pGVAS, pGpu, NULL, &userCtx);
+    NV_ASSERT_OK_OR_GOTO(status,
+        gvaspaceWalkUserCtxAcquire(pGVAS, pGpu, NULL, &userCtx),
+        done);
 
     //
     // Modify the CPU-RM's walker state with the new backing memory.
@@ -676,18 +785,22 @@ kbusPatchBar1Pdb_GSPCLIENT
                                         pRootFmt,
                                         vaspaceGetVaStart(pKernelBus->bar1[GPU_GFID_PF].pVAS),
                                         (MMU_WALK_MEMDESC*)pMemDesc,
-                                        mmuFmtLevelSize(pRootFmt),
+                                        rootSize,
                                         NV_TRUE,
                                         NV_TRUE,
                                         NV_FALSE);
     gvaspaceWalkUserCtxRelease(pGVAS, &userCtx);
-    if (NV_OK != status)
+
+done:
+    if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "Failed to modify CPU-RM's BAR1 PDB to GSP-RM's BAR1 PDB.\n");
-        return status;
+        memdescDestroy(pMemDesc);
+    }
+    else
+    {
+        gvaspaceInvalidateTlb(pGVAS, pGpu, PTE_DOWNGRADE);
     }
-
-    gvaspaceInvalidateTlb(pGVAS, pGpu, PTE_DOWNGRADE);
 
     return status;
 }
@@ -717,10 +830,14 @@ kbusPatchBar2Pdb_GSPCLIENT
     KernelBus   *pKernelBus
 )
 {
-    NV_STATUS            status = NV_OK;
+    NV_STATUS            status   = NV_OK;
     PMEMORY_DESCRIPTOR   pMemDesc;
-    GspStaticConfigInfo *pGSCI  = GPU_GET_GSP_STATIC_INFO(pGpu);
+    GspStaticConfigInfo *pGSCI    = GPU_GET_GSP_STATIC_INFO(pGpu);
+    const MMU_FMT_LEVEL *pRootFmt = pKernelBus->bar2[GPU_GFID_PF].pFmt->pRoot;
     NvU64                entryValue;
+    MEMORY_DESCRIPTOR   *pOldPdb;
+
+    pOldPdb = pKernelBus->virtualBar2[GPU_GFID_PF].pPDB;
 
     NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
         memdescCreate(&pMemDesc, pGpu, pKernelBus->bar2[GPU_GFID_PF].pageDirSize, RM_PAGE_SIZE, NV_TRUE,
@@ -733,20 +850,46 @@ kbusPatchBar2Pdb_GSPCLIENT
 
     //
     // BAR2 page table is not yet working at this point, so retrieving the
-    // PDE3[0] of BAR2 page table via BAR0_WINDOW
+    // PDE3[0] of BAR2 page table via BAR0_WINDOW or GSP-DMA (in case BARs
+    // are blocked)
     //
-    entryValue = GPU_REG_RD32(pGpu, (NvU32)pKernelBus->bar2[GPU_GFID_PF].bar2OffsetInBar0Window) |
+    if (kbusIsBarAccessBlocked(pKernelBus))
+    {
+        MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
+        TRANSFER_SURFACE surf = {0};
+
+        surf.pMemDesc = pOldPdb;
+        surf.offset = 0;
+
+        NV_ASSERT_OK_OR_RETURN(
+            memmgrMemRead(pMemoryManager, &surf, &entryValue,
+                          pRootFmt->entrySize, TRANSFER_FLAGS_NONE));
+    }
+    else
+    {
+        entryValue = GPU_REG_RD32(pGpu, (NvU32)pKernelBus->bar2[GPU_GFID_PF].bar2OffsetInBar0Window) |
                  ((NvU64)GPU_REG_RD32(pGpu, (NvU32)pKernelBus->bar2[GPU_GFID_PF].bar2OffsetInBar0Window + 4) << 32);
+    }
 
     //
     // Provide the PDE3[0] value to GSP-RM so that GSP-RM can merge CPU-RM's
     // page table to GSP-RM's page table
     //
-    NV_RM_RPC_UPDATE_BAR_PDE(pGpu, NV_RPC_UPDATE_PDE_BAR_2, entryValue, pKernelBus->bar2[GPU_GFID_PF].pFmt->pRoot->virtAddrBitLo, status);
+    NV_RM_RPC_UPDATE_BAR_PDE(pGpu, NV_RPC_UPDATE_PDE_BAR_2, entryValue, pRootFmt->virtAddrBitLo, status);
 
     return NV_OK;
 }
 
+NvBool
+kbusCheckEngine_KERNEL
+(
+    OBJGPU        *pGpu,
+    KernelBus     *pKernelBus,
+    ENGDESCRIPTOR  engDesc
+)
+{
+    return kbusCheckEngineWithOrderList_KERNEL(pGpu, pKernelBus, engDesc, NV_TRUE);
+}
 
 /*!
  * @brief Checks whether an engine is available or not.
@@ -756,23 +899,25 @@ kbusPatchBar2Pdb_GSPCLIENT
  * rely on timeouts after a read of a register in the reg space for engine.
  * Instead, it
  *  - Return TRUE for all engines which are must present in GPU.
- *  - Get information about CE, MSENC, NVJPG and OFA engines from plugin.
- *  - Rest engines are determined from HAL creation data.
+ *  - Get information about CE, MSENC, NVJPG and OFA engines from plugin or GSP-RM.
+ *  - If bCheckEngineOrder is true, the remaining engines are searched for in gpuChildOrderList_HAL.
  *
  * @param[in] pGpu       OBJGPU pointer
  * @param[in] pKernelBus KernelBus pointer
  * @param[in] engDesc    ENGDESCRIPTOR pointer used to check Engine presence
+ * @param[in] bCheckEngineOrder If true, check gpuChildOrderList_HAL for engDesc as well
  *
  * @returns NV_TRUE if engine is available.
  *          NV_FALSE if engine is not available or floorswept.
  *
  */
 NvBool
-kbusCheckEngine_KERNEL
+kbusCheckEngineWithOrderList_KERNEL
 (
     OBJGPU        *pGpu,
     KernelBus     *pKernelBus,
-    ENGDESCRIPTOR  engDesc
+    ENGDESCRIPTOR  engDesc,
+    NvBool         bCheckEngineOrder
 )
 {
     NvU32     rmEngineCaps[NVGPU_ENGINE_CAPS_MASK_ARRAY_MAX] = {0};
@@ -780,6 +925,31 @@ kbusCheckEngine_KERNEL
     NvBool    bSupported;
     NV_STATUS status;
 
+    if (IS_VIRTUAL(pGpu))
+    {
+        NvU32             i     = 0;
+        NvU32             j     = 0;
+        VGPU_STATIC_INFO *pVSI  = GPU_GET_STATIC_INFO(pGpu);
+
+        if (pVSI == NULL)
+        {
+            return NV_FALSE;
+        }
+
+        //
+        // vGPU Enginelist can accommodate maximum of NVGPU_VGPU_ENGINE_LIST_LAST engine mask.
+        // Currently, vGPU plugin advertises the same in NvU64 bitmask.
+        //
+        ct_assert(RM_ENGINE_TYPE_LAST <= NVGPU_VGPU_ENGINE_LIST_LAST);
+        ct_assert(NVGPU_ENGINE_CAPS_MASK_ARRAY_MAX <= (NVGPU_VGPU_ENGINE_LIST_MASK_ARRAY_MAX * 2));
+
+        for (i = 0; i < (NVGPU_ENGINE_CAPS_MASK_ARRAY_MAX/2); i++)
+        {
+            nv2080EngineCaps[j++] = NvU64_LO32(pVSI->engineList[i]);
+            nv2080EngineCaps[j++] = NvU64_HI32(pVSI->engineList[i]);
+        }
+    }
+    else
     {
         NvU32 i;
         GspStaticConfigInfo *pGSCI = GPU_GET_GSP_STATIC_INFO(pGpu);
@@ -800,93 +970,67 @@ kbusCheckEngine_KERNEL
                                   rmEngineCaps),
         return NV_FALSE);
 
-    switch (engDesc)
+    switch (ENGDESC_FIELD(engDesc, _CLASS))
     {
-        case ENG_LSFM:
-        case ENG_PMU:
-        case ENG_CLK:
-        case ENG_ACR:
-        case ENG_DISP:
+        case ENG_CLASS_LSFM:
+        case ENG_CLASS_PMU:
+        case ENG_CLASS_CLK:
+        case ENG_CLASS_ACR:
+        case ENG_CLASS_DISP:
             return NV_FALSE;
         //
         // This function is used in two environments:
         // (a) vGPU where display is not yet supported.
         // (b) RM offload (Kernel RM) where display is supported.
         //
-        case ENG_KERNEL_DISPLAY:
+        case ENG_CLASS_KERNEL_DISPLAY:
             return IS_GSP_CLIENT(pGpu);
 
-        case ENG_BIF:
-        case ENG_KERNEL_BIF:
-        case ENG_MC:
-        case ENG_KERNEL_MC:
-        case ENG_PRIV_RING:
-        case ENG_SW_INTR:
-        case ENG_TMR:
-        case ENG_DMA:
-        case ENG_BUS:
-        case ENG_GR(0):
-        case ENG_CIPHER:
-        case ENG_INTR:
-        case ENG_GPULOG:
-        case ENG_GPUMON:
-        case ENG_FIFO:
+        case ENG_CLASS_BIF:
+        case ENG_CLASS_KERNEL_BIF:
+        case ENG_CLASS_MC:
+        case ENG_CLASS_KERNEL_MC:
+        case ENG_CLASS_PRIV_RING:
+        case ENG_CLASS_SW_INTR:
+        case ENG_CLASS_TMR:
+        case ENG_CLASS_DMA:
+        case ENG_CLASS_BUS:
+        case ENG_CLASS_CIPHER:
+        case ENG_CLASS_INTR:
+        case ENG_CLASS_GPULOG:
+        case ENG_CLASS_GPUMON:
+        case ENG_CLASS_FIFO:
             return NV_TRUE;
 
-        case ENG_CE(0):
-        case ENG_CE(1):
-        case ENG_CE(2):
-        case ENG_CE(3):
-        case ENG_CE(4):
-        case ENG_CE(5):
-        case ENG_CE(6):
-        case ENG_CE(7):
-        case ENG_CE(8):
-        case ENG_CE(9):
+        case ENG_CLASS_CE:
             return !!NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps,
                 RM_ENGINE_TYPE_COPY(GET_CE_IDX(engDesc)));
 
-        case ENG_MSENC(0):
-        case ENG_MSENC(1):
-        case ENG_MSENC(2):
+        case ENG_CLASS_NVENC:
             return !!NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps,
                 RM_ENGINE_TYPE_NVENC(GET_MSENC_IDX(engDesc)));
-        case ENG_SEC2:
+        case ENG_CLASS_SEC2:
             return !!NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps,
                                                 RM_ENGINE_TYPE_SEC2);
-        case ENG_NVDEC(0):
-        case ENG_NVDEC(1):
-        case ENG_NVDEC(2):
-        case ENG_NVDEC(3):
-        case ENG_NVDEC(4):
-        case ENG_NVDEC(5):
-        case ENG_NVDEC(6):
-        case ENG_NVDEC(7):
+        case ENG_CLASS_NVDEC:
             return !!NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps,
                 RM_ENGINE_TYPE_NVDEC(GET_NVDEC_IDX(engDesc)));
 
-        case ENG_OFA:
+        case ENG_CLASS_OFA:
             return !!NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps,
-                                                RM_ENGINE_TYPE_OFA);
-        case ENG_NVJPEG(0):
-        case ENG_NVJPEG(1):
-        case ENG_NVJPEG(2):
-        case ENG_NVJPEG(3):
-        case ENG_NVJPEG(4):
-        case ENG_NVJPEG(5):
-        case ENG_NVJPEG(6):
-        case ENG_NVJPEG(7):
+                RM_ENGINE_TYPE_OFA(GET_OFA_IDX(engDesc)));
+
+        case ENG_CLASS_NVJPEG:
             return !!NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps,
                 RM_ENGINE_TYPE_NVJPEG(GET_NVJPEG_IDX(engDesc)));
 
-        case ENG_GR(1):
-        case ENG_GR(2):
-        case ENG_GR(3):
-        case ENG_GR(4):
-        case ENG_GR(5):
-        case ENG_GR(6):
-        case ENG_GR(7):
+        case ENG_CLASS_GR:
         {
+            if (engDesc == ENG_GR(0))
+            {
+                return NV_TRUE;
+            }
+
             KernelFifo *pKernelFifo  = GPU_GET_KERNEL_FIFO(pGpu);
 
             NV_ASSERT_OR_RETURN(pKernelFifo != NULL, NV_FALSE);
@@ -897,7 +1041,7 @@ kbusCheckEngine_KERNEL
                     bSupported);
         }
 
-        case ENG_INVALID:
+        case ENG_CLASS_INVALID:
             NV_PRINTF(LEVEL_ERROR,
                       "Query for ENG_INVALID considered erroneous: %d\n",
                       engDesc);
@@ -908,15 +1052,23 @@ kbusCheckEngine_KERNEL
         // HAL lists of GPU. So Add ASSERT there.
         //
         default:
-            bSupported = gpuIsEngDescSupported(pGpu, engDesc);
-
-            if (!bSupported)
+        {
+            if (bCheckEngineOrder)
             {
-                NV_PRINTF(LEVEL_ERROR, "Unable to check engine ID: %d\n",
-                          engDesc);
-                NV_ASSERT(bSupported);
+                bSupported = gpuIsEngDescSupported(pGpu, engDesc);
+
+                if (!bSupported)
+                {
+                    NV_PRINTF(LEVEL_ERROR, "Unable to check engine ID: 0x%x\n",
+                              engDesc);
+                    NV_ASSERT(bSupported);
+                }
             }
+            else
+                bSupported = NV_FALSE;
+
             return bSupported;
+        }
     }
 }
 
@@ -951,7 +1103,7 @@ kbusGetDeviceCaps_IMPL
     bExplicitCacheFlushRequired = NVCPU_IS_ARM &&
                                   (RMCFG_FEATURE_PLATFORM_UNIX || RMCFG_FEATURE_PLATFORM_MODS_UNIX);
     if (bExplicitCacheFlushRequired ||
-        (!pCl->getProperty(pCL, PDB_PROP_CL_IS_CHIPSET_IO_COHERENT)))
+        (!pCl->getProperty(pCl, PDB_PROP_CL_IS_CHIPSET_IO_COHERENT)))
         RMCTRL_SET_CAP(tempCaps, NV0080_CTRL_HOST_CAPS, _EXPLICIT_CACHE_FLUSH_REQD);
 
     if ((pCl->FHBBusInfo.vendorID == PCI_VENDOR_ID_NVIDIA) &&
@@ -962,9 +1114,6 @@ kbusGetDeviceCaps_IMPL
         RMCTRL_SET_CAP(tempCaps, NV0080_CTRL_HOST_CAPS, _CPU_WRITE_WAR_BUG_420495);
     }
 
-    // the RM always supports GPU-coherent mappings
-    RMCTRL_SET_CAP(tempCaps, NV0080_CTRL_HOST_CAPS, _GPU_COHERENT_MAPPING_SUPPORTED);
-
     // If we don't have existing caps with which to reconcile, then just return
     if (!bCapsInitialized)
     {
@@ -972,163 +1121,55 @@ kbusGetDeviceCaps_IMPL
         return;
     }
 
-    // factor in this GPUs caps: all these are feature caps, so use AND
-    RMCTRL_AND_CAP(pHostCaps, tempCaps, temp,
-                   NV0080_CTRL_HOST_CAPS, _P2P_4_WAY);
-    RMCTRL_AND_CAP(pHostCaps, tempCaps, temp,
-                   NV0080_CTRL_HOST_CAPS, _P2P_8_WAY);
-    RMCTRL_AND_CAP(pHostCaps, tempCaps, temp,
-                   NV0080_CTRL_HOST_CAPS, _GPU_COHERENT_MAPPING_SUPPORTED);
-
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _SEMA_ACQUIRE_BUG_105665);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _SYS_SEMA_DEADLOCK_BUG_148216);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _SLOWSLI);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _SEMA_READ_ONLY_BUG);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _MEM2MEM_BUG_365782);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _LARGE_NONCOH_UPSTR_WRITE_BUG_114871);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _LARGE_UPSTREAM_WRITE_BUG_115115);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _SEP_VIDMEM_PB_NOTIFIERS_BUG_83923);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _P2P_DEADLOCK_BUG_203825);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _COMPRESSED_BL_P2P_BUG_257072);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _CROSS_BLITS_BUG_270260);
     RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
                   NV0080_CTRL_HOST_CAPS, _CPU_WRITE_WAR_BUG_420495);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _BAR1_READ_DEADLOCK_BUG_511418);
 
     return;
 }
 
 NV_STATUS
-kbusMapFbApertureByHandle_IMPL
+kbusMapFbApertureSingle_IMPL
 (
-    OBJGPU    *pGpu,
+    OBJGPU *pGpu,
     KernelBus *pKernelBus,
-    NvHandle   hClient,
-    NvHandle   hMemory,
-    NvU64      offset,
-    NvU64      size,
-    NvU64     *pBar1Va
+    MEMORY_DESCRIPTOR *pMemDesc,
+    NvU64 offset,
+    NvU64 *pAperOffset,
+    NvU64 *pLength,
+    NvU32 flags,
+    Device *pDevice
 )
 {
-    NV_STATUS status;
-    RsClient *pClient = NULL;
-    RsResourceRef *pSrcMemoryRef = NULL;
-    Memory *pSrcMemory = NULL;
-    MEMORY_DESCRIPTOR *pMemDesc = NULL;
-    NvU64 fbApertureOffset = 0;
-    NvU64 fbApertureLength = size;
-
-    NV_ASSERT_OK_OR_RETURN(serverGetClientUnderLock(&g_resServ, hClient, &pClient));
-
-    status = clientGetResourceRef(pClient, hMemory, &pSrcMemoryRef);
-    if (status != NV_OK)
-    {
-        return status;
-    }
-
-    pSrcMemory = dynamicCast(pSrcMemoryRef->pResource, Memory);
-    if (pSrcMemory == NULL)
-    {
-        return NV_ERR_INVALID_OBJECT;
-    }
-
-    pMemDesc = pSrcMemory->pMemDesc;
-
-    if (memdescGetAddressSpace(pMemDesc) != ADDR_FBMEM)
-    {
-        return NV_ERR_INVALID_ARGUMENT;
-    }
-
-    status = kbusMapFbAperture_HAL(pGpu, pKernelBus, pMemDesc, offset,
-                                   &fbApertureOffset, &fbApertureLength,
-                                   BUS_MAP_FB_FLAGS_MAP_UNICAST, hClient);
-    if (status != NV_OK)
-    {
-        return status;
-    }
-
-    NV_ASSERT_OR_GOTO(fbApertureLength >= size, failed);
-
-    if ((!NV_IS_ALIGNED64(fbApertureOffset, osGetPageSize())) ||
-        (!NV_IS_ALIGNED64(fbApertureLength, osGetPageSize())))
-    {
-        status = NV_ERR_NOT_SUPPORTED;
-        goto failed;
-    }
-
-    *pBar1Va = gpumgrGetGpuPhysFbAddr(pGpu) + fbApertureOffset;
-
-    if (!NV_IS_ALIGNED64(*pBar1Va, osGetPageSize()))
-    {
-        status = NV_ERR_INVALID_ADDRESS;
-        goto failed;
-    }
-
+    MemoryArea memArea;
+    MemoryRange memRange;
+    memArea.numRanges = 1;
+    memArea.pRanges = &memRange;
+    memRange.start = *pAperOffset;
+    NV_ASSERT_OK_OR_RETURN(kbusMapFbAperture_HAL(pGpu, pKernelBus, pMemDesc, mrangeMake(offset, *pLength), &memArea,
+        flags | BUS_MAP_FB_FLAGS_UNMANAGED_MEM_AREA, pDevice));
+    *pLength = memRange.size;
+    *pAperOffset = memRange.start;
     return NV_OK;
-
-failed:
-    // Note: fbApertureLength is not used by kbusUnmapFbAperture_HAL(), so it's passed as 0
-    kbusUnmapFbAperture_HAL(pGpu, pKernelBus, pMemDesc,
-                            fbApertureOffset, 0,
-                            BUS_MAP_FB_FLAGS_MAP_UNICAST);
-
-    return status;
 }
-
+    
 NV_STATUS
-kbusUnmapFbApertureByHandle_IMPL
+kbusUnmapFbApertureSingle_IMPL
 (
-    OBJGPU    *pGpu,
+    OBJGPU *pGpu,
     KernelBus *pKernelBus,
-    NvHandle   hClient,
-    NvHandle   hMemory,
-    NvU64      bar1Va
+    MEMORY_DESCRIPTOR *pMemDesc,
+    NvU64 aperOffset,
+    NvU64 length,
+    NvU32 flags
 )
 {
-    NV_STATUS status;
-    RsClient *pClient = NULL;
-    RsResourceRef *pSrcMemoryRef = NULL;
-    Memory *pSrcMemory = NULL;
-    MEMORY_DESCRIPTOR *pMemDesc = NULL;
+    MemoryArea memArea;
+    MemoryRange memRange = mrangeMake(aperOffset, length);
 
-    NV_ASSERT_OK_OR_RETURN(serverGetClientUnderLock(&g_resServ, hClient, &pClient));
+    memArea.numRanges = 1;
+    memArea.pRanges = &memRange;
 
-    status = clientGetResourceRef(pClient, hMemory, &pSrcMemoryRef);
-    if (status != NV_OK)
-    {
-        return status;
-    }
-
-    pSrcMemory = dynamicCast(pSrcMemoryRef->pResource, Memory);
-    if (pSrcMemory == NULL)
-    {
-        return NV_ERR_INVALID_OBJECT;
-    }
-
-    pMemDesc = pSrcMemory->pMemDesc;
-
-    // Note: fbApertureLength is not used by kbusUnmapFbAperture_HAL(), so it's passed as 0
-    status = kbusUnmapFbAperture_HAL(pGpu, pKernelBus, pMemDesc,
-                                     bar1Va - gpumgrGetGpuPhysFbAddr(pGpu),
-                                     0, BUS_MAP_FB_FLAGS_MAP_UNICAST);
-    if (status != NV_OK)
-    {
-        return status;
-    }
-
-    return NV_OK;
+    return kbusUnmapFbAperture_HAL(pGpu, pKernelBus, pMemDesc, memArea, flags | BUS_MAP_FB_FLAGS_UNMANAGED_MEM_AREA);
 }
 
 /*!
@@ -1150,20 +1191,33 @@ kbusSendBusInfo_IMPL
 )
 {
     NV_STATUS status = NV_OK;
-    NV2080_CTRL_BUS_GET_INFO_V2_PARAMS busGetInfoParams = {0};
 
-    busGetInfoParams.busInfoList[0] = *pBusInfo;
-    busGetInfoParams.busInfoListSize = 1;
+    if (IS_VIRTUAL(pGpu))
+    {
+        VGPU_STATIC_INFO *pVSI = GPU_GET_STATIC_INFO(pGpu);
 
-    NV_RM_RPC_CONTROL(pGpu,
-                      pGpu->hInternalClient,
-                      pGpu->hInternalSubdevice,
-                      NV2080_CTRL_CMD_BUS_GET_INFO_V2,
-                      &busGetInfoParams,
-                      sizeof(busGetInfoParams),
-                      status);
+        NV_ASSERT_OR_RETURN(pVSI != NULL, NV_ERR_INVALID_STATE);
+
+        pBusInfo->data = pVSI->busGetInfoV2.busInfoList[pBusInfo->index].data;
+    }
+    else
+    {
+        RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
+        NV2080_CTRL_BUS_GET_INFO_V2_PARAMS busGetInfoParams = {0};
+
+        busGetInfoParams.busInfoList[0] = *pBusInfo;
+        busGetInfoParams.busInfoListSize = 1;
+
+        status = pRmApi->Control(pRmApi,
+                                 pGpu->hInternalClient,
+                                 pGpu->hInternalSubdevice,
+                                 NV2080_CTRL_CMD_BUS_GET_INFO_V2,
+                                 &busGetInfoParams,
+                                 sizeof(busGetInfoParams));
+
+        pBusInfo->data = busGetInfoParams.busInfoList[0].data;
+    }
 
-    pBusInfo->data = busGetInfoParams.busInfoList[0].data;
     return status;
 }
 
@@ -1185,9 +1239,27 @@ kbusGetNvlinkP2PPeerId_VGPU
     KernelBus *pKernelBus0,
     OBJGPU    *pGpu1,
     KernelBus *pKernelBus1,
-    NvU32     *nvlinkPeer
+    NvU32     *nvlinkPeer,
+    NvU32      flags
 )
 {
+    NvBool bEgmPeer = FLD_TEST_DRF(_P2PAPI, _ATTRIBUTES, _REMOTE_EGM, _YES, flags);
+    NvU32 status = NV_OK;
+
+    if (bEgmPeer)
+    {
+        *nvlinkPeer = kbusGetEgmPeerId_HAL(pGpu0, pKernelBus0, pGpu1);
+    }
+    else
+    {
+        *nvlinkPeer = kbusGetPeerId_HAL(pGpu0, pKernelBus0, pGpu1);
+    }
+
+    if (*nvlinkPeer != BUS_INVALID_PEER)
+    {
+        return NV_OK;
+    }
+
     *nvlinkPeer = kbusGetUnusedPeerId_HAL(pGpu0, pKernelBus0);
 
     // If could not find a free peer ID, return error
@@ -1199,7 +1271,33 @@ kbusGetNvlinkP2PPeerId_VGPU
         return NV_ERR_GENERIC;
     }
     // Reserve the peer ID for NVLink use
-    return kbusReserveP2PPeerIds_HAL(pGpu0, pKernelBus0, NVBIT(*nvlinkPeer));
+    status = kbusReserveP2PPeerIds_HAL(pGpu0, pKernelBus0, NVBIT(*nvlinkPeer));
+
+    if (status == NV_OK)
+    {
+        pKernelBus0->p2p.bEgmPeer[*nvlinkPeer] = bEgmPeer;
+    }
+
+    return status;
+}
+
+/**
+ * @brief     Check if the static bar1 is enabled
+ *
+ * @param[in] pGpu
+ * @param[in] pKernelBus
+ */
+NvBool
+kbusIsStaticBar1Enabled_IMPL
+(
+    OBJGPU    *pGpu,
+    KernelBus *pKernelBus
+)
+{
+    NvU32 gfid;
+
+    return ((vgpuGetCallingContextGfid(pGpu, &gfid) == NV_OK) &&
+            pKernelBus->bar1[gfid].bStaticBar1Enabled);
 }
 
 /**
@@ -1218,3 +1316,106 @@ kbusIsGpuP2pAlive_IMPL
 {
     return (pKernelBus->totalP2pObjectsAliveRefCount > 0);
 }
+
+/**
+ * @brief     Gets BAR0 size in bytes for VF
+ *
+ * @param[in] pGpu
+ * @param[in] pKernelBus
+ */
+NvU64 kbusGetVfBar0SizeBytes_IMPL
+(
+    OBJGPU    *pGpu,
+    KernelBus *pKernelBus
+)
+{
+    // Bar 0 size for VF is always 16MB
+    return 16llu << 20;
+}
+
+/**
+ * @brief Update BAR1 size and availability in RUSD
+ *
+ * @param[in] pGpu
+ */
+NV_STATUS
+kbusUpdateRusdStatistics_IMPL
+(
+    OBJGPU *pGpu
+)
+{
+    KernelBus *pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
+    OBJVASPACE *pBar1VAS;
+    OBJEHEAP *pVASHeap;
+    RUSD_BAR1_MEMORY_INFO *pSharedData;
+    NvU64 bar1Size = 0;
+    NvU64 bar1AvailSize = 0;
+    NV_RANGE bar1VARange = NV_RANGE_EMPTY;
+    NvBool bZeroRusd = kbusIsBar1Disabled(pKernelBus);
+    
+    if (kbusIsStaticBar1Enabled(pGpu, pKernelBus))
+    {
+        // PMA owns BAR1 info, so don't  update.
+        return NV_OK;
+    }
+
+    bZeroRusd = bZeroRusd || IS_MIG_ENABLED(pGpu);
+
+    if (!bZeroRusd)
+    {
+        pBar1VAS = kbusGetBar1VASpace_HAL(pGpu, pKernelBus);
+        NV_ASSERT_OR_RETURN(pBar1VAS != NULL, NV_ERR_INVALID_STATE);
+        pVASHeap = vaspaceGetHeap(pBar1VAS);
+        bar1VARange = rangeMake(vaspaceGetVaStart(pBar1VAS), vaspaceGetVaLimit(pBar1VAS));
+
+        bar1Size = (NvU32)(rangeLength(bar1VARange) / 1024);
+
+        if (pVASHeap != NULL)
+        {
+            NvU64 freeSize = 0;
+
+            pVASHeap->eheapInfoForRange(pVASHeap, bar1VARange, NULL, NULL, NULL, &freeSize);
+            bar1AvailSize = (NvU32)(freeSize / 1024);
+        }
+    }
+
+    // Minimize critical section, write data once we have it.
+    pSharedData = gpushareddataWriteStart(pGpu, bar1MemoryInfo);
+    MEM_WR32(&pSharedData->bar1Size, bar1Size);
+    MEM_WR32(&pSharedData->bar1AvailSize, bar1AvailSize);
+    gpushareddataWriteFinish(pGpu, bar1MemoryInfo);
+
+    return NV_OK;
+}
+
+NV_STATUS
+kbusGetGpuFbPhysAddressForRdma_IMPL
+(
+    OBJGPU    *pGpu,
+    KernelBus *pKernelBus,
+    NvBool     bForcePcie,
+    NvU64     *pPhysAddr
+)
+{
+    if((bForcePcie) &&
+       (!pGpu->getProperty(pGpu, PDB_PROP_GPU_COHERENT_CPU_MAPPING)))
+    {
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    //
+    // For forced PCIe mappings on coherent systems, use SPA by default instead of GPA
+    // if the RmGpuDirectRdmaForceSPA regkey is set.
+    // This is a stop-gap measure until hypervisor ensures GPA==SPA.
+    //
+    if (bForcePcie && pKernelBus->bGrdmaForceSpa)
+    {
+        *pPhysAddr = pKernelBus->grdmaBar1Spa;
+    }
+    else
+    {
+        *pPhysAddr = gpumgrGetGpuPhysFbAddr(pGpu);
+    }
+
+    return NV_OK;
+}
