@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2017-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2017-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -26,7 +26,6 @@
  */
 
 #include "gpu/falcon/kernel_falcon.h"
-#include "gpu/mc/kernel_mc.h"
 #include "os/os.h"
 
 #include "published/turing/tu102/dev_riscv_pri.h"
@@ -150,89 +149,79 @@ kflcnIsRiscvActive_TU102
 }
 
 /*!
- *  Perform a reset of the Falcon.
+ * Returns true if the RISC-V core is selected.
  */
-void
+NvBool
+kflcnIsRiscvSelected_TU102
+(
+    OBJGPU *pGpu,
+    KernelFalcon *pKernelFlcn
+)
+{
+    //
+    // There is no explicit core select on TU10x/GA100. Use the "is the RISC-V core active" check,
+    // even though:
+    //  (a) this can theoretically race with the ucode switching into a Falcon mode
+    //  (b) this returns a false negative if the RISC-V core has halted
+    // since there is no need to support either of these cases on TU10x/GA100.
+    //
+    return kflcnIsRiscvActive_HAL(pGpu, pKernelFlcn);
+}
+
+/*!
+ * Reset falcon using secure reset.
+ * This leaves the falcon in falcon mode after reset.
+ */
+NV_STATUS
 kflcnReset_TU102
 (
     OBJGPU *pGpu,
     KernelFalcon *pKernelFlcn
 )
 {
-    kflcnEnable_HAL(pGpu, pKernelFlcn, NV_FALSE);
-    kflcnEnable_HAL(pGpu, pKernelFlcn, NV_TRUE);
+    NV_ASSERT_OK_OR_RETURN(kflcnPreResetWait_HAL(pGpu, pKernelFlcn));
+    NV_ASSERT_OK(kflcnResetHw(pGpu, pKernelFlcn));
+    NV_ASSERT_OK_OR_RETURN(kflcnWaitForResetToFinish_HAL(pGpu, pKernelFlcn));
+    kflcnSwitchToFalcon_HAL(pGpu, pKernelFlcn);
+    kflcnRegWrite_HAL(pGpu, pKernelFlcn, NV_PFALCON_FALCON_RM,
+                      pGpu->chipId0);
+    return NV_OK;
 }
 
 /*!
- * Does a reset of the Falcon using secure reset and switches to Falcon mode.
+ * Reset falcon using secure reset, ready to run riscv.
  */
-void
-kflcnSecureReset_TU102
+NV_STATUS
+kflcnResetIntoRiscv_TU102
 (
     OBJGPU *pGpu,
     KernelFalcon *pKernelFlcn
 )
 {
-    NV_ASSERT_OR_RETURN_VOID(kflcnPreResetWait_HAL(pGpu, pKernelFlcn) == NV_OK);
-    NV_ASSERT_OK(kflcnResetHw(pGpu, pKernelFlcn));
-
-    kflcnWaitForResetToFinish_HAL(pGpu, pKernelFlcn);
-
-    kflcnSwitchToFalcon_HAL(pGpu, pKernelFlcn);
+    //
+    // Turing and GA100 do not have an explicit core switch,
+    // the core will be ready to run riscv after reset.
+    //
+    NV_ASSERT_OK_OR_RETURN(kflcnReset_TU102(pGpu, pKernelFlcn));
+    kflcnSetRiscvMode(pKernelFlcn, NV_TRUE);
+    return NV_OK;
 }
 
 /*!
- * Enable or disable the Falcon to FALCON mode.
+ * Switch the core to FALCON mode.
  */
 void
-kflcnEnable_TU102
+kflcnSwitchToFalcon_TU102
 (
     OBJGPU *pGpu,
-    KernelFalcon *pKernelFlcn,
-    NvBool bEnable
+    KernelFalcon *pKernelFlcn
 )
 {
-    KernelMc *pKernelMc = GPU_GET_KERNEL_MC(pGpu);
-
-    if (!bEnable)
-    {
-        // Disable in PMC if engine is present in PMC
-        if (pKernelFlcn->pmcEnableMask > 0)
-        {
-            kmcWritePmcEnableReg_HAL(pGpu, pKernelMc, pKernelFlcn->pmcEnableMask,
-                                     NV_FALSE, pKernelFlcn->bIsPmcDeviceEngine);
-            // Read back to create enough of a delay
-            kmcReadPmcEnableReg_HAL(pGpu, pKernelMc, pKernelFlcn->bIsPmcDeviceEngine);
-            kmcReadPmcEnableReg_HAL(pGpu, pKernelMc, pKernelFlcn->bIsPmcDeviceEngine);
-        }
-        else
-        {
-            kflcnSecureReset_HAL(pGpu, pKernelFlcn);
-        }
-    }
-    else
-    {
-        // Enable in PMC if engine is present in PMC
-        if (pKernelFlcn->pmcEnableMask > 0)
-        {
-            kmcWritePmcEnableReg_HAL(pGpu, pKernelMc, pKernelFlcn->pmcEnableMask,
-                                     NV_TRUE, pKernelFlcn->bIsPmcDeviceEngine);
-            // Read back to create enough of a delay
-            kmcReadPmcEnableReg_HAL(pGpu, pKernelMc, pKernelFlcn->bIsPmcDeviceEngine);
-            kmcReadPmcEnableReg_HAL(pGpu, pKernelMc, pKernelFlcn->bIsPmcDeviceEngine);
-        }
-        else
-        {
-            kflcnSecureReset_HAL(pGpu, pKernelFlcn);
-        }
-
-        kflcnSwitchToFalcon_HAL(pGpu, pKernelFlcn);
-
-        kflcnWaitForResetToFinish_HAL(pGpu, pKernelFlcn);
-
-        kflcnRegWrite_HAL(pGpu, pKernelFlcn, NV_PFALCON_FALCON_RM,
-                          pGpu->chipId0);
-    }
+    //
+    // Turing and GA100 do not have an explicit core switch,
+    // so simply update the software state tracking the expected mode.
+    //
+    kflcnSetRiscvMode(pKernelFlcn, NV_FALSE);
 }
 
 /*!
@@ -365,7 +354,7 @@ kflcnWaitForHalt_TU102
 }
 
 /*!
- * Read the IRQ status of the RISCV Falcon.
+ * Read the IRQ status of the Falcon.
  *
  * @return IRQ status mask
  */
@@ -376,12 +365,26 @@ kflcnReadIntrStatus_TU102
     KernelFalcon *pKernelFlcn
 )
 {
-    return ((kflcnRegRead_HAL(pGpu, pKernelFlcn, NV_PFALCON_FALCON_IRQSTAT) &
-             kflcnRegRead_HAL(pGpu, pKernelFlcn, NV_PFALCON_FALCON_IRQMASK) &
-             kflcnRegRead_HAL(pGpu, pKernelFlcn, NV_PFALCON_FALCON_IRQDEST)) |
-            (kflcnRegRead_HAL(pGpu, pKernelFlcn, NV_PFALCON_FALCON_IRQSTAT) &
-             kflcnRiscvRegRead_HAL(pGpu, pKernelFlcn, NV_PRISCV_RISCV_IRQMASK) &
-             kflcnRiscvRegRead_HAL(pGpu, pKernelFlcn, NV_PRISCV_RISCV_IRQDEST)));
+    return (kflcnRegRead_HAL(pGpu, pKernelFlcn, NV_PFALCON_FALCON_IRQSTAT) &
+            kflcnRegRead_HAL(pGpu, pKernelFlcn, NV_PFALCON_FALCON_IRQMASK) &
+            kflcnRegRead_HAL(pGpu, pKernelFlcn, NV_PFALCON_FALCON_IRQDEST));
+}
+
+/*!
+ * Read the IRQ status of the Falcon in RISC-V mode.
+ *
+ * @return IRQ status mask
+ */
+NvU32
+kflcnRiscvReadIntrStatus_TU102
+(
+    OBJGPU *pGpu,
+    KernelFalcon *pKernelFlcn
+)
+{
+    return (kflcnRegRead_HAL(pGpu, pKernelFlcn, NV_PFALCON_FALCON_IRQSTAT) &
+            kflcnRiscvRegRead_HAL(pGpu, pKernelFlcn, NV_PRISCV_RISCV_IRQMASK) &
+            kflcnRiscvRegRead_HAL(pGpu, pKernelFlcn, NV_PRISCV_RISCV_IRQDEST));
 }
 
 
