@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -28,16 +28,22 @@
 #include "kernel/gpu/mem_mgr/mem_mgr.h"
 #include "kernel/gpu/gr/kernel_graphics.h"
 #include "kernel/gpu/falcon/kernel_falcon.h"
+#include "kernel/gpu/rc/kernel_rc.h"
+#include "platform/sli/sli.h"
+
+#include "kernel/gpu/conf_compute/conf_compute.h"
 
 #include "class/cl0090.h" // KERNEL_GRAPHICS_CONTEXT
 #include "class/cl9067.h" // FERMI_CONTEXT_SHARE_A
 
 #include "libraries/utils/nvprintf.h"
 #include "gpu/gpu.h"
+#include "gpu/device/device.h"
 #include "kernel/gpu/mig_mgr/kernel_mig_manager.h"
 #include "gpu/mem_mgr/vaspace_api.h"
 #include "vgpu/rpc.h"
 #include "rmapi/rs_utils.h"
+#include "containers/eheap_old.h"
 
 NV_STATUS
 kchangrpapiConstruct_IMPL
@@ -47,29 +53,30 @@ kchangrpapiConstruct_IMPL
     RS_RES_ALLOC_PARAMS_INTERNAL *pParams
 )
 {
-    NvBool            bTsgAllocated     = NV_FALSE;
-    RsResourceRef    *pResourceRef      = pCallContext->pResourceRef;
-    NV_STATUS         rmStatus;
-    OBJVASPACE       *pVAS              = NULL;
-    OBJGPU           *pGpu              = GPU_RES_GET_GPU(pKernelChannelGroupApi);
-    KernelMIGManager *pKernelMIGManager = NULL;
-    KernelFifo       *pKernelFifo       = GPU_GET_KERNEL_FIFO(pGpu);
-    NvHandle          hVASpace          = NV01_NULL_OBJECT;
-    Device           *pDevice           = NULL;
-    NvU32             gfid              = GPU_GFID_PF;
-    RsShared         *pShared           = NULL;
-    RsClient         *pClient;
-    NvBool            bLockAcquired           = NV_FALSE;
-    Heap             *pHeap                   = GPU_GET_HEAP(pGpu);
-    NvBool            bMIGInUse               = NV_FALSE;
-    CTX_BUF_INFO     *bufInfoList             = NULL;
-    NvU32             bufCount                = 0;
-    NvBool            bReserveMem             = NV_FALSE;
-    MIG_INSTANCE_REF  ref;
-    RM_API           *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
-    KernelChannelGroup *pKernelChannelGroup = NULL;
-    NV_CHANNEL_GROUP_ALLOCATION_PARAMETERS *pAllocParams = NULL;
-    RM_ENGINE_TYPE    rmEngineType;
+    NvBool                                  bTsgAllocated       = NV_FALSE;
+    RsResourceRef                          *pResourceRef        = pCallContext->pResourceRef;
+    NV_STATUS                               rmStatus;
+    OBJVASPACE                             *pVAS                = NULL;
+    OBJGPU                                 *pGpu                = GPU_RES_GET_GPU(pKernelChannelGroupApi);
+    KernelMIGManager                       *pKernelMIGManager   = NULL;
+    KernelFifo                             *pKernelFifo         = GPU_GET_KERNEL_FIFO(pGpu);
+    NvHandle                                hVASpace            = NV01_NULL_OBJECT;
+    Device                                 *pDevice             = NULL;
+    NvU32                                   gfid                = GPU_GFID_PF;
+    RsShared                               *pShared             = NULL;
+    RsClient                               *pClient;
+    NvBool                                  bLockAcquired       = NV_FALSE;
+    Heap                                   *pHeap               = GPU_GET_HEAP(pGpu);
+    NvBool                                  bMIGInUse           = NV_FALSE;
+    CTX_BUF_INFO                           *bufInfoList         = NULL;
+    NvU32                                   bufCount            = 0;
+    NvBool                                  bReserveMem         = NV_FALSE;
+    MIG_INSTANCE_REF                        ref;
+    RM_API                                 *pRmApi              = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+    KernelChannelGroup                     *pKernelChannelGroup = NULL;
+    NV_CHANNEL_GROUP_ALLOCATION_PARAMETERS *pAllocParams        = NULL;
+    RM_ENGINE_TYPE                          rmEngineType;
+
 
     NV_PRINTF(LEVEL_INFO,
               "hClient: 0x%x, hParent: 0x%x, hObject:0x%x, hClass: 0x%x\n",
@@ -96,7 +103,7 @@ kchangrpapiConstruct_IMPL
     if (rmDeviceGpuLockIsOwner(pGpu->gpuInstance))
     {
         NV_PRINTF(LEVEL_ERROR, "TSG alloc should be called without acquiring GPU lock\n");
-        LOCK_ASSERT_AND_RETURN(0);
+        NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_LOCK_STATE);
     }
 
     bufInfoList = portMemAllocNonPaged(NV_ENUM_SIZE(GR_CTX_BUFFER) * sizeof(*bufInfoList));
@@ -228,7 +235,9 @@ kchangrpapiConstruct_IMPL
 
     // vGpu plugin context flag should only be set on host if context is plugin
     if (gpuIsSriovEnabled(pGpu))
+    {
         pKernelChannelGroup->bIsCallingContextVgpuPlugin = pAllocParams->bIsCallingContextVgpuPlugin;
+    }
 
     if (pKernelChannelGroup->bIsCallingContextVgpuPlugin)
         gfid = GPU_GFID_PF;
@@ -239,13 +248,26 @@ kchangrpapiConstruct_IMPL
 
     if (!RMCFG_FEATURE_PLATFORM_GSP)
     {
-        NV_ASSERT_OK_OR_GOTO(rmStatus,
-            ctxBufPoolInit(pGpu, pHeap, &pKernelChannelGroup->pCtxBufPool),
-            failed);
+        RmClient *pRmClient = dynamicCast(pClient, RmClient);
+        //
+        // WAR for 4217716 - Force allocations made on behalf of internal clients to
+        // RM reserved heap. This avoids a constant memory allocation from appearing
+        // due to the ctxBufPool reservation out of PMA.
+        //
+        if ((pRmClient == NULL) || !(pRmClient->Flags & RMAPI_CLIENT_FLAG_RM_INTERNAL_CLIENT))
+        {
+            NV_ASSERT_OK_OR_GOTO(rmStatus,
+                ctxBufPoolInit(pGpu, pHeap, &pKernelChannelGroup->pCtxBufPool),
+                failed);
 
-        NV_ASSERT_OK_OR_GOTO(rmStatus,
-            ctxBufPoolInit(pGpu, pHeap, &pKernelChannelGroup->pChannelBufPool),
-            failed);
+            NV_ASSERT_OK_OR_GOTO(rmStatus,
+                ctxBufPoolInit(pGpu, pHeap, &pKernelChannelGroup->pChannelBufPool),
+                failed);
+        }
+        else
+        {
+            NV_PRINTF(LEVEL_INFO, "Skipping ctxBufPoolInit for RC watchdog\n");
+        }
     }
 
     NV_ASSERT_OK_OR_GOTO(rmStatus,
@@ -275,6 +297,28 @@ kchangrpapiConstruct_IMPL
                                    NVA06C_CTRL_INTERLEAVE_LEVEL_MEDIUM),
         failed);
 
+    ConfidentialCompute *pConfCompute = GPU_GET_CONF_COMPUTE(pGpu);
+    MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
+    if ((pConfCompute != NULL) &&
+        (pConfCompute->getProperty(pCC, PDB_PROP_CONFCOMPUTE_CC_FEATURE_ENABLED)))
+    {
+        // TODO: jira CONFCOMP-1621: replace this with actual flag for TSG alloc that skips scrub
+        if ((pMemoryManager->bScrubChannelSetupInProgress) &&
+            (pKernelChannelGroup->pChannelBufPool != NULL) &&
+            (pKernelChannelGroup->pCtxBufPool != NULL))
+        {
+            if (pCallContext->secInfo.privLevel < RS_PRIV_LEVEL_KERNEL)
+            {
+                rmStatus = NV_ERR_INVALID_ARGUMENT;
+                NV_PRINTF(LEVEL_ERROR, "Only kernel priv clients can skip scrubber\n");
+                goto failed;
+            }
+            ctxBufPoolSetScrubSkip(pKernelChannelGroup->pChannelBufPool, NV_TRUE);
+            ctxBufPoolSetScrubSkip(pKernelChannelGroup->pCtxBufPool, NV_TRUE);
+            NV_PRINTF(LEVEL_INFO, "Skipping scrubber for all allocations on this context\n");
+        }
+    }
+
     //
     // If ctx buf pools are enabled, filter out partitionable engines
     // that aren't part of our instance.
@@ -507,11 +551,18 @@ done:
     if (bLockAcquired)
         rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
 
-    if (bReserveMem)
+    if ((rmStatus == NV_OK) && bReserveMem)
     {
         // GPU lock should not be held when reserving memory for ctxBufPool
-        NV_ASSERT_OK_OR_CAPTURE_FIRST_ERROR(rmStatus,
+        NV_CHECK_OK(rmStatus, LEVEL_ERROR,
             ctxBufPoolReserve(pGpu, pKernelChannelGroup->pCtxBufPool, bufInfoList, bufCount));
+        if (rmStatus != NV_OK)
+        {
+            // Acquire the lock again for the cleanup path
+            NV_ASSERT_OK_OR_RETURN(rmGpuLocksAcquire(GPUS_LOCK_FLAGS_NONE, RM_LOCK_MODULES_FIFO));
+            bLockAcquired = NV_TRUE;
+            goto failed;
+        }
     }
 
     portMemFree(bufInfoList);
@@ -863,6 +914,8 @@ kchangrpapiSetLegacyMode_IMPL
 
     pKernelChannelGroup->pSubctxIdHeap->eheapDestruct(
         pKernelChannelGroup->pSubctxIdHeap);
+    pKernelChannelGroup->pVaSpaceIdHeap->eheapDestruct(
+        pKernelChannelGroup->pVaSpaceIdHeap);
     //
     // There should only be 1 (SYNC) or 2 legacy kctxshares (SYNC + ASYNC),
     // depending on chip
@@ -875,6 +928,8 @@ kchangrpapiSetLegacyMode_IMPL
 
     constructObjEHeap(pKernelChannelGroup->pSubctxIdHeap,
                       0, maxSubctx, sizeof(KernelCtxShare *), 0);
+    constructObjEHeap(pKernelChannelGroup->pVaSpaceIdHeap,
+                      0, maxSubctx, sizeof(KernelCtxShare *), 0);
 
     pKernelChannelGroup->bLegacyMode = NV_TRUE;
 
@@ -1330,7 +1385,7 @@ kchangrpapiCtrlCmdSetInterleaveLevel_IMPL
     RsResourceRef   *pResourceRef = RES_GET_REF(pKernelChannelGroupApi);
     KernelChannelGroup *pKernelChannelGroup =
         pKernelChannelGroupApi->pKernelChannelGroup;
-    PCLASSDESCRIPTOR pClass       = NULL;
+    CLASSDESCRIPTOR *pClass       = NULL;
     NV_STATUS        status       = NV_OK;
 
     if (gpuGetClassByClassId(pGpu, pResourceRef->externalClassId, &pClass) != NV_OK)
@@ -1382,19 +1437,3 @@ kchangrpapiCtrlCmdGetInterleaveLevel_IMPL
     return NV_OK;
 }
 
-/*!
- * @brief Handler for NVA06C_CTRL_CMD_INTERNAL_PROMOTE_FAULT_METHOD_BUFFERS
- *
- * This is currently un-implemented as split change for bug 200691429
- */
-NV_STATUS
-kchangrpapiCtrlCmdInternalPromoteFaultMethodBuffers_IMPL
-(
-    KernelChannelGroupApi *pKernelChannelGroupApi,
-    NVA06C_CTRL_INTERNAL_PROMOTE_FAULT_METHOD_BUFFERS_PARAMS *pParams
-)
-{
-    NV_PRINTF(LEVEL_INFO,
-        "bug 200691429: kchangrpapiCtrlCmdInternalPromoteFaultMethodBuffers_IMPL received\n");
-    return NV_OK;
-}
