@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2019-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2019-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -51,6 +51,8 @@
 #include "msgq/msgq_priv.h"
 #include "gpu/gsp/kernel_gsp.h"
 #include "nvrm_registry.h"
+#include "gpu/conf_compute/ccsl.h"
+#include "gpu/conf_compute/conf_compute.h"
 
 ct_assert(GSP_MSG_QUEUE_HEADER_SIZE > sizeof(msgqTxHeader) + sizeof(msgqRxHeader));
 
@@ -63,10 +65,8 @@ _getMsgQueueParams
     MESSAGE_QUEUE_COLLECTION *pMQCollection
 )
 {
-    KernelGsp *pKernelGsp = GPU_GET_KERNEL_GSP(pGpu);
     NvLength queueSize;
     MESSAGE_QUEUE_INFO *pRmQueueInfo = &pMQCollection->rpcQueues[RPC_TASK_RM_QUEUE_IDX];
-    MESSAGE_QUEUE_INFO *pTaskIsrQueueInfo = &pMQCollection->rpcQueues[RPC_TASK_ISR_QUEUE_IDX];
     NvU32 numPtes;
     const NvLength defaultCommandQueueSize = 0x40000; // 256 KB
     const NvLength defaultStatusQueueSize  = 0x40000; // 256 KB
@@ -99,24 +99,11 @@ _getMsgQueueParams
         pRmQueueInfo->statusQueueSize = defaultStatusQueueSize;
     }
 
-    // TaskIsrQueue sizes
-    if (pKernelGsp->bIsTaskIsrQueueRequired)
-    {
-        pTaskIsrQueueInfo->commandQueueSize = defaultCommandQueueSize;
-        pTaskIsrQueueInfo->statusQueueSize = defaultStatusQueueSize;
-    }
-    else
-    {
-        pTaskIsrQueueInfo->commandQueueSize = 0;
-        pTaskIsrQueueInfo->statusQueueSize = 0;
-    }
-
     //
     // Calculate the number of entries required to map both queues in addition
     // to the page table itself.
     //
-    queueSize = pRmQueueInfo->commandQueueSize      + pRmQueueInfo->statusQueueSize +
-                pTaskIsrQueueInfo->commandQueueSize + pTaskIsrQueueInfo->statusQueueSize;
+    queueSize = pRmQueueInfo->commandQueueSize + pRmQueueInfo->statusQueueSize;
     NV_ASSERT((queueSize & RM_PAGE_MASK) == 0);
     numPtes = (queueSize >> RM_PAGE_SHIFT);
 
@@ -202,10 +189,8 @@ GspMsgQueuesInit
     MESSAGE_QUEUE_COLLECTION **ppMQCollection
 )
 {
-    KernelGsp *pKernelGsp = GPU_GET_KERNEL_GSP(pGpu);
     MESSAGE_QUEUE_COLLECTION *pMQCollection = NULL;
     MESSAGE_QUEUE_INFO  *pRmQueueInfo = NULL;
-    MESSAGE_QUEUE_INFO  *pTaskIsrQueueInfo = NULL;
     RmPhysAddr  *pPageTbl;
     NvP64        pVaKernel;
     NvP64        pPrivKernel;
@@ -233,13 +218,12 @@ GspMsgQueuesInit
     _getMsgQueueParams(pGpu, pMQCollection);
 
     pRmQueueInfo      = &pMQCollection->rpcQueues[RPC_TASK_RM_QUEUE_IDX];
-    pTaskIsrQueueInfo = &pMQCollection->rpcQueues[RPC_TASK_ISR_QUEUE_IDX];
 
     sharedBufSize = pMQCollection->pageTableSize +
                     pRmQueueInfo->commandQueueSize +
-                    pRmQueueInfo->statusQueueSize +
-                    pTaskIsrQueueInfo->commandQueueSize +
-                    pTaskIsrQueueInfo->statusQueueSize;
+                    pRmQueueInfo->statusQueueSize;
+
+    flags |= MEMDESC_FLAGS_ALLOC_IN_UNPROTECTED_MEMORY;
 
     //
     // For now, put all shared queue memory in one block.
@@ -252,9 +236,9 @@ GspMsgQueuesInit
 
     memdescSetFlag(pMQCollection->pSharedMemDesc, MEMDESC_FLAGS_KERNEL_MODE, NV_TRUE);
 
-    NV_ASSERT_OK_OR_GOTO(nvStatus,
-        memdescAlloc(pMQCollection->pSharedMemDesc),
-        error_ret);
+    memdescTagAlloc(nvStatus, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_58,
+                    pMQCollection->pSharedMemDesc);
+    NV_ASSERT_OK_OR_GOTO(nvStatus, nvStatus, error_ret);
 
     // Create kernel mapping for command queue.
     NV_ASSERT_OK_OR_GOTO(nvStatus,
@@ -285,10 +269,6 @@ GspMsgQueuesInit
     //   RM Command queue entries
     //   RM Status queue header
     //   RM Status queue entries
-    //   TASKISR Command queue header
-    //   TASKISR Command queue entries
-    //   TASKISR Status queue header
-    //   TASKISR Status queue entries
     memdescGetPhysAddrs(pMQCollection->pSharedMemDesc,
                     AT_GPU,                     // addressTranslation
                     0,                          // offset
@@ -305,18 +285,6 @@ GspMsgQueuesInit
     lastQueueVa   = NV_PTR_TO_NvP64(pRmQueueInfo->pStatusQueue);
     lastQueueSize = pRmQueueInfo->statusQueueSize;
 
-    if (pKernelGsp->bIsTaskIsrQueueRequired)
-    {
-        pTaskIsrQueueInfo->pCommandQueue = NvP64_VALUE(
-            NvP64_PLUS_OFFSET(NV_PTR_TO_NvP64(pRmQueueInfo->pStatusQueue), pRmQueueInfo->statusQueueSize));
-
-        pTaskIsrQueueInfo->pStatusQueue  = NvP64_VALUE(
-            NvP64_PLUS_OFFSET(NV_PTR_TO_NvP64(pTaskIsrQueueInfo->pCommandQueue), pTaskIsrQueueInfo->commandQueueSize));
-
-        lastQueueVa   = NV_PTR_TO_NvP64(pTaskIsrQueueInfo->pStatusQueue);
-        lastQueueSize = pTaskIsrQueueInfo->statusQueueSize;
-    }
-
     // Assert that the last queue offset + size fits into the shared memory.
     NV_ASSERT(NvP64_PLUS_OFFSET(pVaKernel, sharedBufSize) ==
               NvP64_PLUS_OFFSET(lastQueueVa, lastQueueSize));
@@ -324,12 +292,6 @@ GspMsgQueuesInit
     NV_ASSERT_OK_OR_GOTO(nvStatus, _gspMsgQueueInit(pRmQueueInfo), error_ret);
     pRmQueueInfo->queueIdx = RPC_TASK_RM_QUEUE_IDX;
 
-    if (pKernelGsp->bIsTaskIsrQueueRequired)
-    {
-        NV_ASSERT_OK_OR_GOTO(nvStatus, _gspMsgQueueInit(pTaskIsrQueueInfo), error_ret);
-        pTaskIsrQueueInfo->queueIdx = RPC_TASK_ISR_QUEUE_IDX;
-    }
-
     *ppMQCollection             = pMQCollection;
     pMQCollection->sharedMemPA  = pPageTbl[0];
 
@@ -400,7 +362,12 @@ NV_STATUS GspStatusQueueInit(OBJGPU *pGpu, MESSAGE_QUEUE_INFO **ppMQI)
         if (nvStatus != NV_OK)
             break;
 
-        kgspDumpGspLogs(pGpu, pKernelGsp, NV_FALSE);
+        kgspDumpGspLogs(pKernelGsp, NV_FALSE);
+        if (!kgspHealthCheck_HAL(pGpu, pKernelGsp))
+        {
+            nvStatus = NV_ERR_RESET_REQUIRED;
+            break;
+        }
     }
 
     if (nRet < 0)
@@ -433,17 +400,14 @@ void GspMsgQueuesCleanup(MESSAGE_QUEUE_COLLECTION **ppMQCollection)
 {
     MESSAGE_QUEUE_COLLECTION *pMQCollection = NULL;
     MESSAGE_QUEUE_INFO       *pRmQueueInfo  = NULL;
-    MESSAGE_QUEUE_INFO       *pTaskIsrQueueInfo = NULL;
 
     if ((ppMQCollection == NULL) || (*ppMQCollection == NULL))
         return;
 
     pMQCollection     = *ppMQCollection;
     pRmQueueInfo      = &pMQCollection->rpcQueues[RPC_TASK_RM_QUEUE_IDX];
-    pTaskIsrQueueInfo = &pMQCollection->rpcQueues[RPC_TASK_ISR_QUEUE_IDX];
 
     _gspMsgQueueCleanup(pRmQueueInfo);
-    _gspMsgQueueCleanup(pTaskIsrQueueInfo);
 
     if (pMQCollection->pSharedMemDesc != NULL)
     {
@@ -467,24 +431,6 @@ void GspMsgQueuesCleanup(MESSAGE_QUEUE_COLLECTION **ppMQCollection)
     *ppMQCollection = NULL;
 }
 
-/*!
- * Calculate 32-bit checksum
- *
- * This routine assumes that the data is padded out with zeros to the next
- * 8-byte alignment, and it is OK to read past the end to the 8-byte alignment.
- */
-static NV_INLINE NvU32 _checkSum32(void *pData, NvU32 uLen)
-{
-    NvU64 *p        = (NvU64 *)pData;
-    NvU64 *pEnd     = (NvU64 *)((NvUPtr)pData + uLen);
-    NvU64  checkSum = 0;
-
-    while (p < pEnd)
-        checkSum ^= *p++;
-
-    return NvU64_HI32(checkSum) ^ NvU64_LO32(checkSum);
-}
-
 /*!
  * GspMsgQueueSendCommand
  *
@@ -502,40 +448,78 @@ NV_STATUS GspMsgQueueSendCommand(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
     NvU8      *pSrc             = (NvU8 *)pCQE;
     NvU8      *pNextElement     = NULL;
     int        nRet;
-    int        i;
-    int        nRetries;
-    int        nElements;
+    NvU32      i;
     RMTIMEOUT  timeout;
     NV_STATUS  nvStatus         = NV_OK;
-    NvU32      uElementSize     = GSP_MSG_QUEUE_ELEMENT_HDR_SIZE +
+    NvU32      msgLen           = GSP_MSG_QUEUE_ELEMENT_HDR_SIZE +
                                   pMQI->pCmdQueueElement->rpc.length;
 
-    if ((uElementSize < sizeof(GSP_MSG_QUEUE_ELEMENT)) ||
-        (uElementSize > GSP_MSG_QUEUE_ELEMENT_SIZE_MAX))
+    if ((msgLen < sizeof(GSP_MSG_QUEUE_ELEMENT)) ||
+        (msgLen > GSP_MSG_QUEUE_ELEMENT_SIZE_MAX))
     {
-        NV_PRINTF(LEVEL_ERROR, "Incorrect length %u\n",
+        NV_PRINTF(LEVEL_ERROR, "Incorrect message length %u\n",
             pMQI->pCmdQueueElement->rpc.length);
         nvStatus = NV_ERR_INVALID_PARAM_STRUCT;
         goto done;
     }
 
     // Make sure the queue element in our working space is zero padded for checksum.
-    if ((uElementSize & 7) != 0)
-        portMemSet(pSrc + uElementSize, 0, 8 - (uElementSize & 7));
+    if ((msgLen & 7) != 0)
+        portMemSet(pSrc + msgLen, 0, 8 - (msgLen & 7));
 
-    pCQE->seqNum   = pMQI->txSeqNum;
-    pCQE->checkSum = 0;
-    pCQE->checkSum = _checkSum32(pSrc, uElementSize);
+    pCQE->seqNum    = pMQI->txSeqNum;
+    pCQE->elemCount = GSP_MSG_QUEUE_BYTES_TO_ELEMENTS(msgLen);
+    pCQE->checkSum  = 0; // The checkSum field is included in the checksum calculation, so zero it.
 
-    nElements = GSP_MSG_QUEUE_BYTES_TO_ELEMENTS(uElementSize);
-
-    for (i = 0; i < nElements; i++)
+    if (gpuIsCCFeatureEnabled(pGpu))
     {
-        // Set a timeout of 1 sec
-        gpuSetTimeout(pGpu, 1000000, &timeout, 0);
+        ConfidentialCompute *pCC = GPU_GET_CONF_COMPUTE(pGpu);
 
-        // Wait for space to put the next element. Retry for up to 10 ms.
-        for (nRetries = 0; ; nRetries++)
+        // Use sequence number as AAD.
+        portMemCopy((NvU8*)pCQE->aadBuffer, sizeof(pCQE->aadBuffer), (NvU8 *)&pCQE->seqNum, sizeof(pCQE->seqNum));
+
+        // We need to encrypt the full queue elements to obscure the data.
+        nvStatus = ccslEncryptWithRotationChecks(pCC->pRpcCcslCtx,
+                               (pCQE->elemCount * GSP_MSG_QUEUE_ELEMENT_SIZE_MIN) - GSP_MSG_QUEUE_ELEMENT_HDR_SIZE,
+                               pSrc + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE,
+                               (NvU8*)pCQE->aadBuffer,
+                               sizeof(pCQE->aadBuffer),
+                               pSrc + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE,
+                               pCQE->authTagBuffer);
+
+        if (nvStatus != NV_OK)
+        {
+            // Do not re-try if encryption fails.
+            NV_PRINTF(LEVEL_ERROR, "Encryption failed with status = 0x%x.\n", nvStatus);
+            if (nvStatus == NV_ERR_INSUFFICIENT_RESOURCES)
+            {
+                // We hit potential IV overflow, this is fatal.
+                NV_PRINTF(LEVEL_ERROR, "Fatal error detected in RPC encrypt: IV overflow!\n");
+                confComputeSetErrorState(pGpu, pCC);
+            }
+            return nvStatus;
+        }
+
+        // Now that encryption covers elements completely, include them in checksum.
+        pCQE->checkSum = _checkSum32(pSrc, pCQE->elemCount * GSP_MSG_QUEUE_ELEMENT_SIZE_MIN);
+    }
+    else
+    {
+        pCQE->checkSum = _checkSum32(pSrc, msgLen);
+    }
+
+    for (i = 0; i < pCQE->elemCount; i++)
+    {
+        NvU32 timeoutFlags = 0;
+
+        if (pMQI->txBufferFull)
+            timeoutFlags |= GPU_TIMEOUT_FLAGS_BYPASS_JOURNAL_LOG;
+
+        // Set a timeout of 1 sec
+        gpuSetTimeout(pGpu, 1000000, &timeout, timeoutFlags);
+
+        // Wait for space to put the next element.
+        while (NV_TRUE)
         {
             // Must get the buffers one at a time, since they could wrap.
             pNextElement = (NvU8 *)msgqTxGetWriteBuffer(pMQI->hQueue, i);
@@ -553,10 +537,17 @@ NV_STATUS GspMsgQueueSendCommand(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
 
         if (pNextElement == NULL)
         {
-            NV_PRINTF(LEVEL_ERROR, "buffer is full\n");
+            pMQI->txBufferFull++;
+            NV_PRINTF_COND(pMQI->txBufferFull == 1, LEVEL_ERROR, LEVEL_INFO,
+                           "buffer is full (waiting for %d free elements, got %d)\n",
+                           pCQE->elemCount, i);
             nvStatus = NV_ERR_BUSY_RETRY;
             goto done;
         }
+        else
+        {
+            pMQI->txBufferFull = 0;
+        }
 
         portMemCopy(pNextElement, GSP_MSG_QUEUE_ELEMENT_SIZE_MIN,
                     pSrc,         GSP_MSG_QUEUE_ELEMENT_SIZE_MIN);
@@ -573,7 +564,7 @@ NV_STATUS GspMsgQueueSendCommand(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
     //
     portAtomicMemoryFenceStore();
 
-    nRet = msgqTxSubmitBuffers(pMQI->hQueue, nElements);
+    nRet = msgqTxSubmitBuffers(pMQI->hQueue, pCQE->elemCount);
 
     if (nRet != 0)
     {
@@ -603,16 +594,17 @@ done:
  *  NV_ERR_NOT_READY            - Partial read.
  *  NV_ERR_INVALID_STATE        - Something really bad happenned.
  */
-NV_STATUS GspMsgQueueReceiveStatus(MESSAGE_QUEUE_INFO *pMQI)
+NV_STATUS GspMsgQueueReceiveStatus(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
 {
     const NvU8 *pNextElement = NULL;
     NvU8       *pTgt         = (NvU8 *)pMQI->pCmdQueueElement;
     int         nRet;
-    int         i;
-    int         nRetries;
-    int         nMaxRetries  = 3;
-    int         nElements    = 1;  // Assume record fits in one queue element for now.
-    NvU32       uElementSize = 0;
+    NvU32       i;
+    NvU32       nRetries;
+    NvU32       nMaxRetries  = 3;
+    NvU32       nElements    = 1;  // Assume record fits in one queue element for now.
+    NvU32       msgLen;
+    NvU32       checkSum;
     NvU32       seqMismatchDiff = NV_U32_MAX;
     NV_STATUS   nvStatus     = NV_OK;
 
@@ -646,29 +638,14 @@ NV_STATUS GspMsgQueueReceiveStatus(MESSAGE_QUEUE_INFO *pMQI)
                         pNextElement, GSP_MSG_QUEUE_ELEMENT_SIZE_MIN);
             pTgt += GSP_MSG_QUEUE_ELEMENT_SIZE_MIN;
 
-            if (i != 0)
-                continue;
-
-            //
-            // Special processing for first element of the record.
-            // Pull out the length and make sure it is valid.
-            //
-            uElementSize = GSP_MSG_QUEUE_ELEMENT_HDR_SIZE +
-                pMQI->pCmdQueueElement->rpc.length;
-
-            if ((uElementSize < sizeof(GSP_MSG_QUEUE_ELEMENT)) ||
-                (uElementSize > GSP_MSG_QUEUE_ELEMENT_SIZE_MAX))
+            if (i == 0)
             {
-                // The length is not valid.  If we are running without a fence,
-                // this could mean that the data is still in flight from the CPU.
-                NV_PRINTF(LEVEL_ERROR, "Incorrect length %u\n",
-                    pMQI->pCmdQueueElement->rpc.length);
-                nvStatus = NV_ERR_INVALID_PARAM_STRUCT;
-                break;
+                //
+                // Special processing for first element of the record.
+                // Pull out the element count. This adjusts the loop condition.
+                //
+                nElements = pMQI->pCmdQueueElement->elemCount;
             }
-
-            // This adjusts the loop condition.
-            nElements = GSP_MSG_QUEUE_BYTES_TO_ELEMENTS(uElementSize);
         }
 
         // Retry if there was an error.
@@ -676,7 +653,25 @@ NV_STATUS GspMsgQueueReceiveStatus(MESSAGE_QUEUE_INFO *pMQI)
             continue;
 
         // Retry if checksum fails.
-        if (_checkSum32(pMQI->pCmdQueueElement, uElementSize) != 0)
+        if (gpuIsCCFeatureEnabled(pGpu))
+        {
+            //
+            // In the Confidential Compute scenario, the actual message length
+            // is inside the encrypted payload, and we can't access it before
+            // decryption, therefore the checksum encompasses the whole element
+            // range. This makes checksum verification significantly slower
+            // because messages are typically much smaller than element size.
+            //
+            checkSum = _checkSum32(pMQI->pCmdQueueElement,
+                                   (nElements * GSP_MSG_QUEUE_ELEMENT_SIZE_MIN));
+        } else
+        {
+            checkSum = _checkSum32(pMQI->pCmdQueueElement,
+                                   (GSP_MSG_QUEUE_ELEMENT_HDR_SIZE +
+                                    pMQI->pCmdQueueElement->rpc.length));
+        }
+
+        if (checkSum != 0)
         {
             NV_PRINTF(LEVEL_ERROR, "Bad checksum.\n");
             nvStatus = NV_ERR_INVALID_DATA;
@@ -716,18 +711,6 @@ NV_STATUS GspMsgQueueReceiveStatus(MESSAGE_QUEUE_INFO *pMQI)
         break;
     }
 
-    if (nvStatus == NV_OK)
-    {
-        pMQI->rxSeqNum++;
-
-        nRet = msgqRxMarkConsumed(pMQI->hQueue, nElements);
-        if (nRet < 0)
-        {
-            NV_PRINTF(LEVEL_ERROR, "msgqRxMarkConsumed failed: %d\n", nRet);
-            nvStatus = NV_ERR_GENERIC;
-        }
-    }
-
     if (nRetries > 0)
     {
         if (nvStatus == NV_OK)
@@ -737,9 +720,54 @@ NV_STATUS GspMsgQueueReceiveStatus(MESSAGE_QUEUE_INFO *pMQI)
         else
         {
             NV_PRINTF(LEVEL_ERROR, "Read failed after %d retries.\n", nRetries);
+            goto exit;
         }
     }
 
+    if (gpuIsCCFeatureEnabled(pGpu))
+    {
+        ConfidentialCompute *pCC = GPU_GET_CONF_COMPUTE(pGpu);
+        nvStatus = ccslDecryptWithRotationChecks(pCC->pRpcCcslCtx,
+                               (nElements * GSP_MSG_QUEUE_ELEMENT_SIZE_MIN) - GSP_MSG_QUEUE_ELEMENT_HDR_SIZE,
+                               ((NvU8*)pMQI->pCmdQueueElement) + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE,
+                               NULL,
+                               (NvU8*)pMQI->pCmdQueueElement->aadBuffer,
+                               sizeof(pMQI->pCmdQueueElement->aadBuffer),
+                               ((NvU8*)pMQI->pCmdQueueElement) + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE,
+                               ((NvU8*)pMQI->pCmdQueueElement->authTagBuffer));
+
+        if (nvStatus != NV_OK)
+        {
+            // Do not re-try if decryption failed. Decryption failure is considered fatal.
+            NV_PRINTF(LEVEL_ERROR, "Fatal error detected in RPC decrypt: 0x%x!\n", nvStatus);
+            confComputeSetErrorState(pGpu, pCC);
+            return nvStatus;
+        }
+    }
+
+    // Sanity check for the given RPC length
+    msgLen = GSP_MSG_QUEUE_ELEMENT_HDR_SIZE + pMQI->pCmdQueueElement->rpc.length;
+
+    if ((msgLen < sizeof(GSP_MSG_QUEUE_ELEMENT)) ||
+        (msgLen > GSP_MSG_QUEUE_ELEMENT_SIZE_MAX))
+    {
+        // The length is not valid.  If we are running without a fence,
+        // this could mean that the data is still in flight from the CPU.
+        NV_PRINTF(LEVEL_ERROR, "Incorrect message length %u\n",
+            pMQI->pCmdQueueElement->rpc.length);
+        nvStatus = NV_ERR_INVALID_PARAM_STRUCT;
+    }
+
+exit:
+    pMQI->rxSeqNum++;
+
+    nRet = msgqRxMarkConsumed(pMQI->hQueue, nElements);
+    if (nRet < 0)
+    {
+        NV_PRINTF(LEVEL_ERROR, "msgqRxMarkConsumed failed: %d\n", nRet);
+        nvStatus = NV_ERR_GENERIC;
+    }
+
     return nvStatus;
 }
 
