@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -23,15 +23,23 @@
 
 #include "core/core.h"
 #include "gpu/gpu.h"
+#include "nvtypes.h"
 #include "os/os.h"
 #include "kernel/gpu/mem_sys/kern_mem_sys.h"
 #include "gpu/mem_mgr/mem_desc.h"
 #include "gpu/bus/kern_bus.h"
+#include "kernel/gpu/intr/intr.h"
+#include "nverror.h"
 
 #include "published/hopper/gh100/dev_fb.h"
+#include "published/hopper/gh100/dev_fbpa.h"
 #include "published/hopper/gh100/dev_vm.h"
 #include "published/hopper/gh100/pri_nv_xal_ep.h"
 #include "published/hopper/gh100/dev_nv_xal_addendum.h"
+#include "published/hopper/gh100/dev_nv_xpl.h"
+#include "published/hopper/gh100/dev_xtl_ep_pri.h"
+#include "published/hopper/gh100/hwproject.h"
+#include "published/ampere/ga100/dev_fb.h"
 
 NV_STATUS
 kmemsysDoCacheOp_GH100
@@ -61,6 +69,20 @@ kmemsysDoCacheOp_GH100
         return rmStatus;
     }
 
+    if (IS_VIRTUAL(pGpu))
+    {
+        switch (reg)
+        {
+            case NV_VIRTUAL_FUNCTION_PRIV_FUNC_L2_SYSMEM_INVALIDATE:
+            case NV_VIRTUAL_FUNCTION_PRIV_FUNC_L2_PEERMEM_INVALIDATE:
+                break;
+            case NV_XAL_EP_UFLUSH_L2_FLUSH_DIRTY:
+                return NV_OK;
+            default:
+                return NV_ERR_NOT_SUPPORTED;
+        }
+    }
+
     switch (reg)
     {
         case NV_XAL_EP_UFLUSH_L2_FLUSH_DIRTY:
@@ -186,9 +208,7 @@ kmemsysDoCacheOp_GH100
 #ifdef DEBUG
     if (cnt > 1)
     {
-        NvU32 intr0 = 0;
-        intr0 = GPU_REG_RD32(pGpu, NV_XAL_EP_INTR_0);
-        NV_ASSERT(DRF_VAL(_XAL_EP, _INTR_0, _FB_ACK_TIMEOUT, intr0) != NV_XAL_EP_INTR_0_FB_ACK_TIMEOUT_PENDING);
+        NV_ASSERT(kmemsysAssertFbAckTimeoutPending_HAL(pGpu, pKernelMemorySystem) == NV_FALSE);
     }
 #endif // DEBUG
 
@@ -272,6 +292,10 @@ kmemsysCacheOp_GH100
             break;
 
         case FB_CACHE_VIDEO_MEMORY:
+            // Fbmem cache ops are not supported from VF -- force NV_OK for MODS
+            if (IS_VIRTUAL(pGpu))
+                return NV_OK;
+
             if (cacheOp == FB_CACHE_EVICT)
             {
                 NvU32 flags = NV2080_CTRL_INTERNAL_MEMSYS_L2_INVALIDATE_EVICT_FLAGS_ALL |
@@ -400,6 +424,163 @@ kmemsysAssertSysmemFlushBufferValid_GH100
               (GPU_REG_RD_DRF(pGpu, _PFB, _FBHUB_PCIE_FLUSH_SYSMEM_ADDR_HI, _ADR) != 0));
 }
 
+/*!
+ * @brief Add GPU memory as a NUMA node.
+ *
+ * Add GPU memory as a NUMA node to the OS kernel in platforms where
+ * GPU is coherently connected to the CPU.
+ *
+ * @param[in]  pGPU                OBJGPU pointer
+ * @param[in]  pKernelMemorySystem KernelMemorySystem pointer
+ * @param[in]  swizzId             swizzId of the MIG GPU instance, 0 for full GPU instance/non-MIG.
+ * @param[in]  offset              start offset of the GPU instance within FB
+ * @param[in]  size                size of the GPU instance
+ * @param[out] numaNodeId          NUMA node id corresponding to the added @swizzId partition memory
+ *                                 when NV_OK is returned.
+ *
+ * @returns NV_OK if all is okay.  Otherwise an error-specific value.
+ *
+ */
+NV_STATUS
+kmemsysNumaAddMemory_GH100
+(
+    OBJGPU             *pGpu,
+    KernelMemorySystem *pKernelMemorySystem,
+    NvU32               swizzId,
+    NvU64               offset,
+    NvU64               size,
+    NvS32              *numaNodeId
+)
+{
+    NV_STATUS status;
+    NvU64     memblockSize   = 0;
+    NvU32     lNumaNodeId;
+
+    NV_ASSERT_OK_OR_RETURN(osNumaMemblockSize(&memblockSize));
+    NV_ASSERT_OR_RETURN(NV_IS_ALIGNED(size, memblockSize), NV_ERR_INVALID_STATE);
+
+    if (pKernelMemorySystem->memPartitionNumaInfo[swizzId].bInUse)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Memory partition: %u is already in use!\n", swizzId);
+        return NV_ERR_IN_USE;
+    }
+
+    status = osNumaAddGpuMemory(pGpu->pOsGpuInfo, offset, size, &lNumaNodeId);
+    if (status == NV_OK)
+    {
+        pKernelMemorySystem->memPartitionNumaInfo[swizzId].bInUse = NV_TRUE;
+        pKernelMemorySystem->memPartitionNumaInfo[swizzId].offset = offset;
+        pKernelMemorySystem->memPartitionNumaInfo[swizzId].size = size;
+        pKernelMemorySystem->memPartitionNumaInfo[swizzId].numaNodeId = lNumaNodeId;
+        *numaNodeId = lNumaNodeId;
+
+        pKernelMemorySystem->bNumaNodesAdded = NV_TRUE;
+
+        NV_PRINTF(LEVEL_INFO, "Memory partition: %u added successfully!"
+                  " numa id: %u offset: 0x%llx size: 0x%llx\n",
+                  swizzId, lNumaNodeId, offset, size);
+    }
+
+    return status;
+}
+
+/*!
+ * @brief Remove a particular MIG GPU instance GPU memory from OS kernel.
+ *
+ * Remove GPU memory from the OS kernel that is earlier added as a NUMA node
+ * to the kernel in platforms where GPU is coherently connected to the CPU.
+ *
+ * @param[in]  pGPU                OBJGPU pointer
+ * @param[in]  pKernelMemorySystem KernelMemorySystem pointer
+ * @param[in]  swizzId             swizzId of the MIG GPU instance, 0 for full partition/non-MIG.
+ */
+void
+kmemsysNumaRemoveMemory_GH100
+(
+    OBJGPU             *pGpu,
+    KernelMemorySystem *pKernelMemorySystem,
+    NvU32               swizzId
+)
+{
+    if (pKernelMemorySystem->memPartitionNumaInfo[swizzId].bInUse == NV_FALSE)
+    {
+        return;
+    }
+
+    osNumaRemoveGpuMemory(pGpu->pOsGpuInfo,
+                          pKernelMemorySystem->memPartitionNumaInfo[swizzId].offset,
+                          pKernelMemorySystem->memPartitionNumaInfo[swizzId].size,
+                          pKernelMemorySystem->memPartitionNumaInfo[swizzId].numaNodeId);
+    pKernelMemorySystem->memPartitionNumaInfo[swizzId].bInUse = NV_FALSE;
+    pKernelMemorySystem->memPartitionNumaInfo[swizzId].offset = 0;
+    pKernelMemorySystem->memPartitionNumaInfo[swizzId].size = 0;
+    pKernelMemorySystem->memPartitionNumaInfo[swizzId].numaNodeId = NV_U32_MAX;
+
+    NV_PRINTF(LEVEL_INFO, "memory partition: %u removed successfully!\n",
+              swizzId);
+    return;
+}
+
+/*!
+ * @brief Remove all GPU memory from OS kernel.
+ *
+ * Remove all MIG GPU instances GPU memory from the OS kernel that is earlier added
+ * as a NUMA node  to the kernel in platforms where GPU is coherently
+ * connected to the CPU.
+ *
+ * @param[in]  pGPU                OBJGPU pointer
+ * @param[in]  pKernelMemorySystem KernelMemorySystem pointer
+ *
+ */
+void
+kmemsysNumaRemoveAllMemory_GH100
+(
+    OBJGPU             *pGpu,
+    KernelMemorySystem *pKernelMemorySystem
+)
+{
+    NvU32 swizzId;
+
+    for (swizzId = 0; swizzId < KMIGMGR_MAX_GPU_SWIZZID; swizzId++)
+    {
+        kmemsysNumaRemoveMemory_HAL(pGpu, pKernelMemorySystem, swizzId);
+    }
+
+    return;
+}
+
+/*!
+ * @brief Return if a given swizzId is rejected by HW
+ */
+NvBool
+kmemsysIsSwizzIdRejectedByHW_GH100
+(
+    OBJGPU *pGpu,
+    KernelMemorySystem *pKernelMemorySystem,
+    NvU32 swizzId
+)
+{
+    KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
+    const KERNEL_MIG_MANAGER_STATIC_INFO *pStaticInfo = kmigmgrGetStaticInfo(pGpu, pKernelMIGManager);
+
+    NV_ASSERT_OR_RETURN(swizzId < KMIGMGR_MAX_GPU_SWIZZID, NV_TRUE);
+    NV_ASSERT_OR_RETURN(pStaticInfo != NULL, NV_TRUE);
+    NV_ASSERT_OR_RETURN(pStaticInfo->pSwizzIdFbMemPageRanges != NULL, NV_TRUE);
+
+    // empty range returned by AMAP means swizzID is rejected by HW
+    if (rangeIsEmpty(rangeMake(pStaticInfo->pSwizzIdFbMemPageRanges->fbMemPageRanges[swizzId].lo,
+                               pStaticInfo->pSwizzIdFbMemPageRanges->fbMemPageRanges[swizzId].hi)))
+    {
+        NV_PRINTF(LEVEL_INFO,
+            "GPU Instance Mem Config for swizzId = 0x%x is rejected by HW\n",
+            swizzId);
+
+        return NV_TRUE;
+    }
+
+    return NV_FALSE;
+}
+
 /*
  * @brief   Function to map swizzId to VMMU Segments
  */
@@ -422,13 +603,93 @@ kmemsysSwizzIdToVmmuSegmentsRange_GH100
     NV_ASSERT_OR_RETURN(pStaticInfo != NULL, NV_ERR_INVALID_STATE);
     NV_ASSERT_OR_RETURN(pStaticInfo->pSwizzIdFbMemPageRanges != NULL, NV_ERR_INVALID_STATE);
 
-    startingVmmuSegment = pStaticInfo->pSwizzIdFbMemPageRanges->fbMemPageRanges[swizzId].lo;
-    memSizeInVmmuSegment = pStaticInfo->pSwizzIdFbMemPageRanges->fbMemPageRanges[swizzId].hi -
-                           pStaticInfo->pSwizzIdFbMemPageRanges->fbMemPageRanges[swizzId].lo;
-    NV_ASSERT_OR_RETURN((memSizeInVmmuSegment <= totalVmmuSegments), NV_ERR_INVALID_STATE);
+    if (kmemsysIsSwizzIdRejectedByHW_HAL(pGpu, pKernelMemorySystem, swizzId))
+    {
+        startingVmmuSegment = 0;
+        memSizeInVmmuSegment = 0;
+    }
+    else
+    {
+        startingVmmuSegment = pStaticInfo->pSwizzIdFbMemPageRanges->fbMemPageRanges[swizzId].lo;
+        memSizeInVmmuSegment = (pStaticInfo->pSwizzIdFbMemPageRanges->fbMemPageRanges[swizzId].hi -
+                                pStaticInfo->pSwizzIdFbMemPageRanges->fbMemPageRanges[swizzId].lo + 1);
+    }
+
+    if (memSizeInVmmuSegment > totalVmmuSegments)
+    {
+        //
+        // SwizzID-0 should cover only partitionable range, however for AMAP,
+        // there is no difference between swizzID-0 and no MIG which can result in
+        // AMAP returning an additional vmmuSegment for swizzID-0
+        //
+        NV_ASSERT_OR_RETURN((swizzId == 0), NV_ERR_INVALID_STATE);
+    }
 
     NV_ASSERT_OK_OR_RETURN(
         kmemsysInitMIGGPUInstanceMemConfigForSwizzId(pGpu, pKernelMemorySystem, swizzId, startingVmmuSegment, memSizeInVmmuSegment));
 
     return NV_OK;
 }
+
+/*
+ * @brief   Function to check if an FB ACK timeout occured. Used only for Debug.
+ */
+NvBool
+kmemsysAssertFbAckTimeoutPending_GH100
+(
+    OBJGPU *pGpu,
+    KernelMemorySystem *pKernelMemorySystem
+)
+{
+#ifdef DEBUG
+    NvU32 intr0 = 0;
+    intr0 = GPU_REG_RD32(pGpu, NV_XAL_EP_INTR_0);
+    return DRF_VAL(_XAL_EP, _INTR_0, _FB_ACK_TIMEOUT, intr0) == NV_XAL_EP_INTR_0_FB_ACK_TIMEOUT_PENDING;
+#else
+    return NV_FALSE;
+#endif
+}
+
+NvU32
+kmemsysGetEccDedCountSize_GH100
+(
+    OBJGPU             *pGpu,
+    KernelMemorySystem *pKernelMemorySystem
+)
+{
+    return NV_PFB_FBPA_0_ECC_DED_COUNT__SIZE_1;
+}
+
+NvU32
+kmemsysGetEccDedCountRegAddr_GH100
+(
+    OBJGPU             *pGpu,
+    KernelMemorySystem *pKernelMemorySystem,
+    NvU32               fbpa,
+    NvU32               subp
+)
+{
+    return NV_PFB_FBPA_0_ECC_DED_COUNT(fbpa) + (subp * NV_FBPA_PRI_STRIDE);
+}
+
+/*!
+ * @brief Check if the GPU memory partition is in use, which
+ *        indicates whether memory has been onlined or not.
+ *
+ * @param[in] pGpu                OBJGPU pointer
+ * @param[in] pKernelMemorySystem KernelMemorySystem pointer
+ * @param[in] swizzId             swizzId of the MIG GPU instance,
+ *                                0 for full GPU instance/non-MIG.
+ *
+ * @returns NvBool - NV_TRUE if memory is onlined, NV_FALSE otherwise
+ */
+NvBool
+kmemsysIsNumaPartitionInUse_GH100
+(
+    OBJGPU             *pGpu,
+    KernelMemorySystem *pKernelMemorySystem,
+    NvU32               swizzId
+)
+{
+    return pKernelMemorySystem->memPartitionNumaInfo[swizzId].bInUse;
+}
