@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2018-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2018-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -28,6 +28,8 @@
  *
  *****************************************************************************/
 
+#define NVOC_GPU_INSTANCE_SUBSCRIPTION_H_PRIVATE_ACCESS_ALLOWED
+
 #include "core/core.h"
 #include "core/system.h"
 #include "gpu/gpu.h"
@@ -55,12 +57,14 @@ _gisubscriptionClientSharesVASCrossPartition
     NvU32 targetedSwizzId
 )
 {
-    NV_STATUS status = NV_OK;
     OBJGPU *pGpu = GPU_RES_GET_GPU(pGPUInstanceSubscription);
     KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
+    RsClient *pRsClientShare;
     RsResourceRef *pDeviceRef;
     Device *pDevice;
     MIG_INSTANCE_REF shareRef;
+    RS_ITERATOR it;
+    NvBool bClientShareHasMatchingInstance = NV_FALSE;
 
     NV_ASSERT_OR_RETURN(pGPUInstanceSubscription != NULL, NV_TRUE);
 
@@ -86,12 +90,30 @@ _gisubscriptionClientSharesVASCrossPartition
     // is subscribed to a different GPU instance than the subscription request, or
     // if the sharing client isn't subscribed to any GPU instance.
     //
-    status = kmigmgrGetInstanceRefFromClient(pGpu, pKernelMIGManager,
-                                             pDevice->hClientShare,
-                                             &shareRef);
+    NV_ASSERT_OK_OR_RETURN(
+        serverGetClientUnderLock(&g_resServ, pDevice->hClientShare, &pRsClientShare));
 
+    it = clientRefIter(pRsClientShare, NULL, classId(Device), RS_ITERATE_CHILDREN, NV_TRUE);
 
-    return (status != NV_OK) || (shareRef.pKernelMIGGpuInstance->swizzId != targetedSwizzId);
+    while (clientRefIterNext(pRsClientShare, &it))
+    {
+        pDevice = dynamicCast(it.pResourceRef->pResource, Device);
+
+        if ((pGpu != GPU_RES_GET_GPU(pDevice)) ||
+            (kmigmgrGetInstanceRefFromDevice(pGpu, pKernelMIGManager, pDevice,
+                                             &shareRef) != NV_OK))
+        {
+            continue;
+        }
+
+        if (shareRef.pKernelMIGGpuInstance->swizzId == targetedSwizzId)
+        {
+            bClientShareHasMatchingInstance = NV_TRUE;
+            break;
+        }
+    }
+
+    return !bClientShareHasMatchingInstance;
 }
 
 NV_STATUS
@@ -104,11 +126,13 @@ gisubscriptionConstruct_IMPL
 {
     NVC637_ALLOCATION_PARAMETERS *pUserParams = pRmAllocParams->pAllocParams;
     RsClient *pRsClient = pCallContext->pClient;
+    RmClient *pRmClient = dynamicCast(pRsClient, RmClient);
     OBJGPU *pGpu;
     KernelMIGManager *pKernelMIGManager;
     NvU32 swizzId;
     NV_STATUS status;
 
+    NV_ASSERT_OR_RETURN(pRmClient != NULL, NV_ERR_INVALID_CLIENT);
     pGpu = GPU_RES_GET_GPU(pGPUInstanceSubscription);
 
     osRmCapInitDescriptor(&pGPUInstanceSubscription->dupedCapDescriptor);
@@ -139,7 +163,7 @@ gisubscriptionConstruct_IMPL
     {
         // Check if this is a root-client or un-privileged device profiling is allowed
         if (gpuIsRmProfilingPrivileged(pGpu) &&
-            !rmclientIsAdminByHandle(pRmAllocParams->hClient, pCallContext->secInfo.privLevel))
+            !rmclientIsAdmin(pRmClient, pCallContext->secInfo.privLevel))
         {
             return NV_ERR_INSUFFICIENT_PERMISSIONS;
         }
@@ -366,16 +390,19 @@ gisubscriptionCtrlCmdExecPartitionsCreate_IMPL
     KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
     KERNEL_MIG_GPU_INSTANCE *pKernelMIGGpuInstance = pGPUInstanceSubscription->pKernelMIGGpuInstance;
 
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
 
 {
     CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
 
     NV_ASSERT_OR_RETURN(pCallContext != NULL, NV_ERR_INVALID_STATE);
 
-    if (!rmclientIsCapableOrAdminByHandle(RES_GET_CLIENT_HANDLE(pGPUInstanceSubscription),
-                                          NV_RM_CAP_SYS_SMC_CONFIG,
-                                          pCallContext->secInfo.privLevel))
+    RmClient *pRmClient = dynamicCast(RES_GET_CLIENT(pGPUInstanceSubscription), RmClient);
+    NV_ASSERT_OR_RETURN(NULL != pRmClient, NV_ERR_INVALID_CLIENT);
+
+    if (!rmclientIsCapableOrAdmin(pRmClient,
+                                  NV_RM_CAP_SYS_SMC_CONFIG,
+                                  pCallContext->secInfo.privLevel))
     {
         NV_PRINTF(LEVEL_ERROR, "Non-privileged context issued privileged cmd\n");
         return NV_ERR_INSUFFICIENT_PERMISSIONS;
@@ -415,23 +442,70 @@ gisubscriptionCtrlCmdExecPartitionsCreate_IMPL
             .inst.request.requestFlags = pParams->flags
         };
 
-        if (!hypervisorIsVgxHyper())
+        if (!gpuIsSriovEnabled(pGpu))
         {
             request.inst.request.requestFlags = FLD_SET_DRF(C637_CTRL, _DMA_EXEC_PARTITIONS_CREATE_REQUEST, _WITH_PART_ID, _FALSE, request.inst.request.requestFlags);
         }
 
         if (IS_VIRTUAL(pGpu))
         {
-            status = kmigmgrCreateComputeInstances_HAL(pGpu, pKernelMIGManager, pKernelMIGGpuInstance,
-                                                       pParams->bQuery,
-                                                       request,
-                                                       pParams->execPartId,
-                                                       NV_TRUE /* create MIG compute instance capabilities */);
+            if (pGpu->getProperty(pGpu, PDB_PROP_GPU_MIG_MIRROR_HOST_CI_ON_GUEST))
+            {
+                NvU32 i;
+                NVC637_CTRL_EXEC_PARTITIONS_IMPORT_EXPORT_PARAMS export;
+                GPUMGR_SAVE_COMPUTE_INSTANCE save;
+
+                KMIGMGR_CREATE_COMPUTE_INSTANCE_PARAMS restore =
+                {
+                    .type = KMIGMGR_CREATE_COMPUTE_INSTANCE_PARAMS_TYPE_RESTORE,
+                    .inst.restore.pComputeInstanceSave = &save,
+                };
+
+                for (i = 0; i < pParams->execPartCount; i++)
+                {
+                    portMemSet(&export, 0, sizeof(export));
+                    export.id = pParams->execPartId[i];
+
+                    NV_RM_RPC_CONTROL(pGpu, pKernelMIGGpuInstance->instanceHandles.hClient,
+                        pKernelMIGGpuInstance->instanceHandles.hSubscription,
+                        NVC637_CTRL_CMD_EXEC_PARTITIONS_EXPORT,
+                        &export,
+                        sizeof(export), status);
+
+                    NV_ASSERT_OK_OR_RETURN(status);
+
+                    portMemSet(&save, 0, sizeof(save));
+                    save.bValid = NV_TRUE;
+                    save.id = pParams->execPartId[i];
+                    save.ciInfo = export.info;
+
+                    status = kmigmgrCreateComputeInstances_HAL(pGpu, pKernelMIGManager, pKernelMIGGpuInstance,
+                                                    pParams->bQuery, restore, &pParams->execPartId[i], NV_TRUE);
+                }
+            }
+            else
+            {
+                status = kmigmgrCreateComputeInstances_HAL(pGpu, pKernelMIGManager, pKernelMIGGpuInstance,
+                                                            pParams->bQuery,
+                                                            request,
+                                                            pParams->execPartId,
+                                                            NV_TRUE /* create MIG compute instance capabilities */);
+            }
         }
         else
         {
             return NV_ERR_NOT_SUPPORTED;
         }
+
+        {
+            NvU32 i;
+
+            for (i = 0; i < pParams->execPartCount; i++)
+            {
+                gpumgrCacheCreateComputeInstance(pGpu, pKernelMIGGpuInstance->swizzId,
+                                                 pParams->execPartId[i]);
+            }
+        }
     }
     else
     {
@@ -467,6 +541,9 @@ gisubscriptionCtrlCmdExecPartitionsCreate_IMPL
             NV_ASSERT_OK_OR_RETURN(
                 kmigmgrCreateComputeInstances_HAL(pGpu, pKernelMIGManager, pKernelMIGGpuInstance,
                                                   NV_FALSE, restore, &pParams->execPartId[i], NV_TRUE));
+
+            gpumgrCacheCreateComputeInstance(pGpu, pKernelMIGGpuInstance->swizzId,
+                                             pParams->execPartId[i]);
         }
     }
 
@@ -501,16 +578,20 @@ gisubscriptionCtrlCmdExecPartitionsDelete_IMPL
     KERNEL_MIG_GPU_INSTANCE *pKernelMIGGpuInstance = pGPUInstanceSubscription->pKernelMIGGpuInstance;
     NvU32 execPartIdx;
 
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
 
 {
     CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
+    RmClient *pRmClient;
 
     NV_ASSERT_OR_RETURN(pCallContext != NULL, NV_ERR_INVALID_STATE);
 
-    if (!rmclientIsCapableOrAdminByHandle(RES_GET_CLIENT_HANDLE(pGPUInstanceSubscription),
-                                          NV_RM_CAP_SYS_SMC_CONFIG,
-                                          pCallContext->secInfo.privLevel))
+    pRmClient = dynamicCast(RES_GET_CLIENT(pGPUInstanceSubscription), RmClient);
+    NV_ASSERT_OR_RETURN(NULL != pRmClient, NV_ERR_INVALID_CLIENT);
+
+    if (!rmclientIsCapableOrAdmin(pRmClient,
+                                  NV_RM_CAP_SYS_SMC_CONFIG,
+                                  pCallContext->secInfo.privLevel))
     {
         NV_PRINTF(LEVEL_ERROR, "Non-privileged context issued privileged cmd\n");
         return NV_ERR_INSUFFICIENT_PERMISSIONS;
@@ -554,6 +635,8 @@ gisubscriptionCtrlCmdExecPartitionsDelete_IMPL
         {
             return NV_ERR_NOT_SUPPORTED;
         }
+        gpumgrCacheDestroyComputeInstance(pGpu, pKernelMIGGpuInstance->swizzId,
+                                          pParams->execPartId[execPartIdx]);
     }
 
     //
@@ -596,7 +679,9 @@ gisubscriptionCtrlCmdExecPartitionsGet_IMPL
     ComputeInstanceSubscription *pComputeInstanceSubscription = NULL;
     KERNEL_MIG_GPU_INSTANCE *pKernelMIGGpuInstance = pGPUInstanceSubscription->pKernelMIGGpuInstance;
     NvU32 ciIdx;
-    NvHandle hClient = RES_GET_CLIENT_HANDLE(pGPUInstanceSubscription);
+    RmClient *pRmClient = dynamicCast(RES_GET_CLIENT(pGPUInstanceSubscription), RmClient);
+    NV_ASSERT_OR_RETURN(NULL != pRmClient, NV_ERR_INVALID_CLIENT);
+
     CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
     NvBool bEnumerateAll = NV_FALSE;
 
@@ -605,14 +690,14 @@ gisubscriptionCtrlCmdExecPartitionsGet_IMPL
     // Capability checks shouldn't be done on
     if (!RMCFG_FEATURE_PLATFORM_GSP)
     {
-        bEnumerateAll = rmclientIsCapableOrAdminByHandle(hClient,
-                                                         NV_RM_CAP_SYS_SMC_CONFIG,
-                                                         pCallContext->secInfo.privLevel);
+        bEnumerateAll = rmclientIsCapableOrAdmin(pRmClient,
+                                                 NV_RM_CAP_SYS_SMC_CONFIG,
+                                                 pCallContext->secInfo.privLevel);
     }
 
     MIG_COMPUTE_INSTANCE *pTargetComputeInstanceInfo = NULL;
 
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
 
     NV_ASSERT_OR_RETURN(pGpu->getProperty(pGpu, PDB_PROP_GPU_MIG_SUPPORTED),
                         NV_ERR_NOT_SUPPORTED);
@@ -622,7 +707,7 @@ gisubscriptionCtrlCmdExecPartitionsGet_IMPL
     (void)cisubscriptionGetComputeInstanceSubscription(RES_GET_CLIENT(pGPUInstanceSubscription), RES_GET_HANDLE(pGPUInstanceSubscription), &pComputeInstanceSubscription);
     if (pComputeInstanceSubscription != NULL)
     {
-        pTargetComputeInstanceInfo = pComputeInstanceSubscription->pMIGComputeInstance;
+        pTargetComputeInstanceInfo = cisubscriptionGetMIGComputeInstance(pComputeInstanceSubscription);
     }
     else if (!bEnumerateAll)
     {
@@ -649,9 +734,26 @@ gisubscriptionCtrlCmdExecPartitionsGet_IMPL
         ++pParams->execPartCount;
 
         pOutInfo->gpcCount = pMIGComputeInstance->resourceAllocation.gpcCount;
+        pOutInfo->gfxGpcCount = pMIGComputeInstance->resourceAllocation.gfxGpcCount;
         pOutInfo->veidCount = pMIGComputeInstance->resourceAllocation.veidCount;
-        pOutInfo->ceCount = kmigmgrCountEnginesOfType(&pMIGComputeInstance->resourceAllocation.engines,
-                                                      RM_ENGINE_TYPE_COPY(0));
+
+        if (pGpu->getProperty(pGpu, PDB_PROP_GPU_MIG_SUPPORTS_SPLIT_CE_RANGES))
+        {
+            KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
+            ENGTYPE_BIT_VECTOR globalEngines;
+            NV_ASSERT_OK_OR_RETURN(
+                kmigmgrEngBitVectorXlate(&pKernelMIGGpuInstance->resourceAllocation.localEngines,
+                                         &pMIGComputeInstance->resourceAllocation.engines,
+                                         &pKernelMIGGpuInstance->resourceAllocation.engines, &globalEngines));
+            pOutInfo->ceCount = kmigmgrCountEnginesInRange(&globalEngines,
+                                                           kmigmgrGetAsyncCERange_HAL(pGpu, pKernelMIGManager));
+        }
+        else
+        {
+            pOutInfo->ceCount = kmigmgrCountEnginesOfType(&pMIGComputeInstance->resourceAllocation.engines,
+                                                          RM_ENGINE_TYPE_COPY(0));
+        }
+
         pOutInfo->nvEncCount = kmigmgrCountEnginesOfType(&pMIGComputeInstance->resourceAllocation.engines,
                                                       RM_ENGINE_TYPE_NVENC(0));
         pOutInfo->nvDecCount = kmigmgrCountEnginesOfType(&pMIGComputeInstance->resourceAllocation.engines,
@@ -659,7 +761,7 @@ gisubscriptionCtrlCmdExecPartitionsGet_IMPL
         pOutInfo->nvJpgCount = kmigmgrCountEnginesOfType(&pMIGComputeInstance->resourceAllocation.engines,
                                                       RM_ENGINE_TYPE_NVJPG);
         pOutInfo->ofaCount = kmigmgrCountEnginesOfType(&pMIGComputeInstance->resourceAllocation.engines,
-                                                      RM_ENGINE_TYPE_OFA);
+                                                      RM_ENGINE_TYPE_OFA(0));
         pOutInfo->sharedEngFlag = pMIGComputeInstance->sharedEngFlag;
         pOutInfo->veidStartOffset = pMIGComputeInstance->resourceAllocation.veidOffset;
         pOutInfo->smCount = pMIGComputeInstance->resourceAllocation.smCount;
@@ -688,7 +790,7 @@ gisubscriptionCtrlCmdExecPartitionsGetActiveIds_IMPL
     KERNEL_MIG_GPU_INSTANCE *pKernelMIGGpuInstance = pGPUInstanceSubscription->pKernelMIGGpuInstance;
     NvU32 ciIdx;
 
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
 
     NV_ASSERT_OR_RETURN(pGpu->getProperty(pGpu, PDB_PROP_GPU_MIG_SUPPORTED),
                         NV_ERR_NOT_SUPPORTED);
@@ -738,13 +840,17 @@ gisubscriptionCtrlCmdExecPartitionsExport_IMPL
 
 {
     CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
+    RmClient     *pRmClient;
 
     NV_ASSERT_OR_RETURN(pCallContext != NULL, NV_ERR_INVALID_STATE);
 
+    pRmClient = dynamicCast(RES_GET_CLIENT(pGPUInstanceSubscription), RmClient);
+    NV_ASSERT_OR_RETURN(NULL != pRmClient, NV_ERR_INVALID_CLIENT);
+
     // An unprivileged client has no use case for import/export
-    if (!rmclientIsCapableOrAdminByHandle(RES_GET_CLIENT_HANDLE(pGPUInstanceSubscription),
-                                          NV_RM_CAP_SYS_SMC_CONFIG,
-                                          pCallContext->secInfo.privLevel))
+    if (!rmclientIsCapableOrAdmin(pRmClient,
+                                  NV_RM_CAP_SYS_SMC_CONFIG,
+                                  pCallContext->secInfo.privLevel))
     {
         return NV_ERR_INSUFFICIENT_PERMISSIONS;
     }
@@ -760,17 +866,14 @@ gisubscriptionCtrlCmdExecPartitionsExport_IMPL
     {
         CALL_CONTEXT *pCallContext  = resservGetTlsCallContext();
         RmCtrlParams *pRmCtrlParams = pCallContext->pControlParams;
-        NV_STATUS status = NV_OK;
+        RM_API       *pRmApi        = GPU_GET_PHYSICAL_RMAPI(pGpu);
 
-        NV_RM_RPC_CONTROL(pGpu,
-                          pRmCtrlParams->hClient,
-                          pRmCtrlParams->hObject,
-                          pRmCtrlParams->cmd,
-                          pRmCtrlParams->pParams,
-                          pRmCtrlParams->paramsSize,
-                          status);
-
-        return status;
+        return pRmApi->Control(pRmApi,
+                               pRmCtrlParams->hClient,
+                               pRmCtrlParams->hObject,
+                               pRmCtrlParams->cmd,
+                               pRmCtrlParams->pParams,
+                               pRmCtrlParams->paramsSize);
     }
 
     if (pParams->id >= NV_ARRAY_ELEMENTS(pGPUInstance->MIGComputeInstance))
@@ -784,6 +887,7 @@ gisubscriptionCtrlCmdExecPartitionsExport_IMPL
     portMemCopy(pParams->info.uuid, sizeof(pParams->info.uuid),
                 pMIGComputeInstance->uuid.uuid, sizeof(pMIGComputeInstance->uuid.uuid));
     pParams->info.sharedEngFlags = pMIGComputeInstance->sharedEngFlag;
+    pParams->info.gfxGpcCount    = pMIGComputeInstance->resourceAllocation.gfxGpcCount;
     pParams->info.veidOffset     = pMIGComputeInstance->resourceAllocation.veidOffset;
     pParams->info.veidCount      = pMIGComputeInstance->resourceAllocation.veidCount;
     pParams->info.smCount        = pMIGComputeInstance->resourceAllocation.smCount;
@@ -816,13 +920,17 @@ gisubscriptionCtrlCmdExecPartitionsImport_IMPL
 
 {
     CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
+    RmClient     *pRmClient;
 
     NV_ASSERT_OR_RETURN(pCallContext != NULL, NV_ERR_INVALID_STATE);
 
+    pRmClient = dynamicCast(RES_GET_CLIENT(pGPUInstanceSubscription), RmClient);
+    NV_ASSERT_OR_RETURN(NULL != pRmClient, NV_ERR_INVALID_CLIENT);
+
     // An unprivileged client has no use case for import/export
-    if (!rmclientIsCapableOrAdminByHandle(RES_GET_CLIENT_HANDLE(pGPUInstanceSubscription),
-                                          NV_RM_CAP_SYS_SMC_CONFIG,
-                                          pCallContext->secInfo.privLevel))
+    if (!rmclientIsCapableOrAdmin(pRmClient,
+                                  NV_RM_CAP_SYS_SMC_CONFIG,
+                                  pCallContext->secInfo.privLevel))
     {
         return NV_ERR_INSUFFICIENT_PERMISSIONS;
     }
@@ -838,14 +946,14 @@ gisubscriptionCtrlCmdExecPartitionsImport_IMPL
     {
         CALL_CONTEXT *pCallContext  = resservGetTlsCallContext();
         RmCtrlParams *pRmCtrlParams = pCallContext->pControlParams;
+        RM_API       *pRmApi        = GPU_GET_PHYSICAL_RMAPI(pGpu);
 
-        NV_RM_RPC_CONTROL(pGpu,
-                          pRmCtrlParams->hClient,
-                          pRmCtrlParams->hObject,
-                          pRmCtrlParams->cmd,
-                          pRmCtrlParams->pParams,
-                          pRmCtrlParams->paramsSize,
-                          status);
+        status = pRmApi->Control(pRmApi,
+                                 pRmCtrlParams->hClient,
+                                 pRmCtrlParams->hObject,
+                                 pRmCtrlParams->cmd,
+                                 pRmCtrlParams->pParams,
+                                 pRmCtrlParams->paramsSize);
 
         if (status != NV_OK)
             return status;
@@ -868,7 +976,8 @@ gisubscriptionCtrlCmdExecPartitionsImport_IMPL
         {
             KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
             NV_CHECK_OK_OR_GOTO(status, LEVEL_ERROR,
-                kmigmgrCreateComputeInstances_HAL(pGpu, pKernelMIGManager, pGPUInstance, NV_FALSE, restore, &pParams->id, NV_FALSE),
+                kmigmgrCreateComputeInstances_HAL(pGpu, pKernelMIGManager,
+                 pGPUInstance, NV_FALSE, restore, &pParams->id, pParams->bCreateCap),
                 cleanup_rpc);
         }
         else
@@ -1032,142 +1141,28 @@ gisubscriptionCtrlCmdExecPartitionsGetProfileCapacity_IMPL
     OBJGPU *pGpu = GPU_RES_GET_GPU(pGPUInstanceSubscription);
     KERNEL_MIG_GPU_INSTANCE *pKernelMIGGpuInstance = pGPUInstanceSubscription->pKernelMIGGpuInstance;
     KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
-    NvBool bCtsRequired = kmigmgrIsCTSAlignmentRequired(pGpu, pKernelMIGManager);
 
-    NV_CHECK_OR_RETURN(LEVEL_ERROR,
-        pParams->computeSize < NV2080_CTRL_GPU_PARTITION_FLAG_COMPUTE_SIZE__SIZE,
-        NV_ERR_INVALID_ARGUMENT);
+    return kmigmgrComputeProfileGetCapacity(pGpu, pKernelMIGManager, pKernelMIGGpuInstance->pProfile,
+                                            pKernelMIGGpuInstance, pParams);
+}
 
-    if (bCtsRequired)
-    {
-        NV_RANGE totalRange = kmigmgrComputeProfileSizeToCTSIdRange(pParams->computeSize);
-        NvU32 slotBasisComputeSize = kmigmgrSmallestComputeProfileSize(pGpu, pKernelMIGManager);
-        NvU64 slotBasisMask;
-        NvU64 validQueryMask;
-        NvU64 inUseIdMask;
-        NvU64 ctsId;
-        NvU32 totalSpanCount;
-        NvU32 availableSpanCount;
-        NV_RANGE slotBasisIdRange;
+NV_STATUS
+gisubscriptionCtrlCmdGetUuid_IMPL
+(
+    GPUInstanceSubscription *pGPUInstanceSubscription,
+    NVC637_CTRL_GET_UUID_PARAMS *pParams
+)
+{
+    NV_CHECK_OR_RETURN(LEVEL_ERROR, !pGPUInstanceSubscription->bDeviceProfiling,
+                       NV_ERR_NOT_SUPPORTED);
 
-        NV_CHECK_OR_RETURN(LEVEL_ERROR, slotBasisComputeSize != KMIGMGR_COMPUTE_SIZE_INVALID, NV_ERR_INVALID_STATE);
+    portMemCopy(pParams->uuid, sizeof(pParams->uuid),
+                pGPUInstanceSubscription->pKernelMIGGpuInstance->uuid.uuid,
+                sizeof(pGPUInstanceSubscription->pKernelMIGGpuInstance->uuid.uuid));
 
-        slotBasisIdRange = kmigmgrComputeProfileSizeToCTSIdRange(slotBasisComputeSize);
-
-        NV_CHECK_OR_RETURN(LEVEL_ERROR, !rangeIsEmpty(totalRange), NV_ERR_INVALID_ARGUMENT);
-        NV_CHECK_OR_RETURN(LEVEL_ERROR, !rangeIsEmpty(slotBasisIdRange), NV_ERR_INVALID_ARGUMENT);
-
-        slotBasisMask = DRF_SHIFTMASK64(slotBasisIdRange.hi:slotBasisIdRange.lo);
-        validQueryMask = DRF_SHIFTMASK64(totalRange.hi:totalRange.lo) & pKernelMIGGpuInstance->pProfile->validCTSIdMask;
-
-        // Find mask of un-usable IDs due to current in-use CTS Ids
-        inUseIdMask = 0x0;
-        FOR_EACH_INDEX_IN_MASK(64, ctsId, pKernelMIGGpuInstance->ctsIdsInUseMask)
-        {
-            NvU64 invalidMask;
-
-            NV_ASSERT_OK(kmigmgrGetInvalidCTSIdMask(pGpu, pKernelMIGManager, ctsId, &invalidMask));
-
-            inUseIdMask |= invalidMask;
-        }
-        FOR_EACH_INDEX_IN_MASK_END;
-
-        //
-        // The slot basis defines the smallest divison of the GPU instance.
-        // CTS IDs from this range are used as a means to specify span placements
-        // for compute profiles.
-        //
-        totalSpanCount = 0;
-        availableSpanCount = 0;
-
-        FOR_EACH_INDEX_IN_MASK(64, ctsId, validQueryMask)
-        {
-            NvU64 invalidMask;
-
-            NV_ASSERT_OK(kmigmgrGetInvalidCTSIdMask(pGpu, pKernelMIGManager, ctsId, &invalidMask));
-
-            invalidMask &= slotBasisMask;
-            pParams->totalSpans[totalSpanCount].lo = portUtilCountTrailingZeros64(invalidMask) - slotBasisIdRange.lo;
-            pParams->totalSpans[totalSpanCount].hi = nvPopCount64(invalidMask) + pParams->totalSpans[totalSpanCount].lo - 1;
-
-            if (!(NVBIT64(ctsId) & inUseIdMask))
-            {
-                pParams->availableSpans[availableSpanCount].lo = pParams->totalSpans[totalSpanCount].lo;
-                pParams->availableSpans[availableSpanCount].hi = pParams->totalSpans[totalSpanCount].hi;
-                availableSpanCount++;
-            }
-            totalSpanCount++;
-        }
-        FOR_EACH_INDEX_IN_MASK_END;
-
-        pParams->totalSpansCount     = totalSpanCount;
-        pParams->totalProfileCount   = totalSpanCount;
-        pParams->availableSpansCount = availableSpanCount;
-        pParams->profileCount        = availableSpanCount;
-    }
-    else
-    {
-        KernelGraphicsManager *pKernelGraphicsManager = GPU_GET_KERNEL_GRAPHICS_MANAGER(pGpu);
-        NV2080_CTRL_INTERNAL_MIGMGR_COMPUTE_PROFILE profile;
-        NvU64 veidMask;
-        NvU32 GPUInstanceVeidEnd;
-        NvU64 GPUInstanceVeidMask;
-        NvU64 GPUInstanceFreeVeidMask;
-        NvU64 GPUInstancePseudoMask;
-        NvU32 availableSpanCount;
-        NvU32 totalSpanCount;
-        NvU32 veidStepSize;
-        NvU32 veidSlotCount;
-        NvU32 count;
-        NvU32 i;
-
-        NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
-            kmigmgrGetComputeProfileFromSize(pGpu, pKernelMIGManager, pParams->computeSize, &profile));
-        NV_ASSERT_OK_OR_RETURN(
-            kgrmgrGetMaxVeidsPerGpc(pGpu, pKernelGraphicsManager, &veidStepSize));
-
-        // Create a mask for VEIDs associated with this GPU instance
-        veidMask = DRF_SHIFTMASK64(profile.veidCount - 1:0);
-        GPUInstanceVeidEnd = pKernelMIGGpuInstance->resourceAllocation.veidOffset + pKernelMIGGpuInstance->resourceAllocation.veidCount - 1;
-        GPUInstanceVeidMask = DRF_SHIFTMASK64(GPUInstanceVeidEnd:pKernelMIGGpuInstance->resourceAllocation.veidOffset);
-        GPUInstanceFreeVeidMask = GPUInstanceVeidMask & ~pKernelGraphicsManager->veidInUseMask;
-        GPUInstancePseudoMask = GPUInstanceFreeVeidMask;
-        veidSlotCount = 0;
-        availableSpanCount = 0;
-        totalSpanCount = 0;
-        count = 0;
-        for (i = pKernelMIGGpuInstance->resourceAllocation.veidOffset; i < GPUInstanceVeidEnd; i += veidStepSize)
-        {
-            // Determine max correctly sized VEID segments
-            if (((GPUInstanceFreeVeidMask >> i) & veidMask) == veidMask)
-            {
-                pParams->availableSpans[availableSpanCount].lo = count;
-                pParams->availableSpans[availableSpanCount].hi = count + (profile.veidCount / veidStepSize) - 1;
-                availableSpanCount++;
-            }
-
-            // Determine max correctly sized VEID segments
-            if (((GPUInstanceVeidMask >> i) & veidMask) == veidMask)
-            {
-                pParams->totalSpans[totalSpanCount].lo = count;
-                pParams->totalSpans[totalSpanCount].hi = count + (profile.veidCount / veidStepSize) - 1;
-                totalSpanCount++;
-            }
-
-            // Determine max correctly sized VEID segments
-            if (((GPUInstancePseudoMask >> i) & veidMask) == veidMask)
-            {
-                veidSlotCount++;
-                GPUInstancePseudoMask &= ~(veidMask << i);
-            }
-            count++;
-        }
-        pParams->totalProfileCount = NV_MIN(pKernelMIGGpuInstance->pProfile->virtualGpcCount / profile.gpcCount,
-                                                pKernelMIGGpuInstance->pProfile->veidCount / profile.veidCount);
-        pParams->totalSpansCount     = totalSpanCount;
-        pParams->profileCount        = veidSlotCount;
-        pParams->availableSpansCount = availableSpanCount;
-    }
+    nvGetSmcUuidString((void*)pParams->uuid, pParams->uuidStr);
+    pParams->uuidStr[1] = 'G';
+    pParams->uuidStr[2] = 'I';
 
     return NV_OK;
 }
