@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -25,6 +25,7 @@
 
 #include "gpu/mmu/kern_gmmu.h"
 #include "gpu/bus/kern_bus.h"
+#include "vgpu/rpc.h"
 
 #include "published/maxwell/gm107/dev_fb.h"
 #include "published/maxwell/gm107/dev_mmu.h"
@@ -36,12 +37,47 @@
  *
  * @returns NvU32
  */
-NvU32
+NvU64
 kgmmuGetBigPageSize_GM107(KernelGmmu *pKernelGmmu)
 {
     return pKernelGmmu->defaultBigPageSize;
 }
 
+NV_STATUS
+kgmmuCommitInvalidateTlbTest_GM107
+(
+    OBJGPU                              *pGpu,
+    KernelGmmu                          *pKernelGmmu,
+    COMMIT_TLB_INVALIDATE_TEST_PARAMS   *pTestParams
+)
+{
+    TLB_INVALIDATE_PARAMS params;
+    NvU32 regVal = 0;
+
+    if (pTestParams->invalidateAll != NV_TRUE)
+    {
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    portMemSet(&params, 0, sizeof(TLB_INVALIDATE_PARAMS));
+    params.gfid = pTestParams->gfid;
+    gpuSetTimeout(pGpu, GPU_TIMEOUT_DEFAULT, &params.timeout,
+                  GPU_TIMEOUT_FLAGS_BYPASS_THREAD_STATE |
+                  GPU_TIMEOUT_FLAGS_DEFAULT | GPU_TIMEOUT_FLAGS_BYPASS_CPU_YIELD);
+
+    NV_ASSERT_OK_OR_RETURN(kgmmuCheckPendingInvalidates_HAL(pGpu, pKernelGmmu,
+                                                            &params.timeout));
+
+    // Invalidate all VA and PDB
+    regVal = DRF_DEF(_PFB_PRI, _MMU_INVALIDATE, _ALL_VA, _TRUE) |
+             DRF_DEF(_PFB_PRI, _MMU_INVALIDATE, _ALL_PDB, _TRUE) |
+             DRF_DEF(_PFB_PRI, _MMU_INVALIDATE, _TRIGGER, _TRUE);
+
+    params.regVal = regVal;
+
+    return kgmmuCommitTlbInvalidate_HAL(pGpu, pKernelGmmu, &params);
+}
+
 /*!
  * @brief Unicast GMMU TLB invalidate.
  *
@@ -78,6 +114,8 @@ kgmmuInvalidateTlb_GM107
     NV_STATUS             status         = NV_OK;
     TLB_INVALIDATE_PARAMS params;
     NvU32                 flushCount     = 0;
+    NvBool                bDoVgpuRpc     = NV_FALSE;
+    OBJVGPU              *pVgpu          = NULL;
 
     //
     // Bail out early if
@@ -105,26 +143,41 @@ kgmmuInvalidateTlb_GM107
         return;
     }
 
-    //
-    // Originally the flag is 0, but to WAR bug 2909388, add flag
-    // GPU_TIMEOUT_FLAGS_BYPASS_THREAD_STATE to bypass using threadStateCheckTimeout,
-    // GPU_TIMEOUT_FLAGS_BYPASS_CPU_YIELD to not wait inside timeout with mutex held.
-    //
-    gpuSetTimeout(pGpu, GPU_TIMEOUT_DEFAULT, &params.timeout,
-                  GPU_TIMEOUT_FLAGS_BYPASS_THREAD_STATE |
-                  GPU_TIMEOUT_FLAGS_DEFAULT | GPU_TIMEOUT_FLAGS_BYPASS_CPU_YIELD);
-
-    //
-    // 2. Wait until we can issue an invalidate. On pre-Turing, wait for space
-    // in the PRI FIFO. On Turing, check if an invalidate is already in progress.
-    //
-    // Set the GFID.
-    params.gfid = gfid;
-
-    status = kgmmuCheckPendingInvalidates_HAL(pGpu, pKernelGmmu, &params.timeout, params.gfid);
-    if (status != NV_OK)
+    pVgpu = GPU_GET_VGPU(pGpu);
+    if (pVgpu && pVgpu->bGspBuffersInitialized)
     {
-       return;
+        VGPU_STATIC_INFO *pVSI = GPU_GET_STATIC_INFO(pGpu);
+
+        if  (pVSI &&
+             FLD_TEST_DRF(A080, _CTRL_CMD_VGPU_GET_CONFIG,
+                          _PARAMS_VGPU_DEV_CAPS_VF_INVALIDATE_TLB_TRAP_ENABLED,
+                          _TRUE, pVSI->vgpuConfig.vgpuDeviceCapsBits))
+            bDoVgpuRpc = NV_TRUE;
+    }
+
+    if (!bDoVgpuRpc)
+    {
+        //
+        // Originally the flag is 0, but to WAR bug 2909388, add flag
+        // GPU_TIMEOUT_FLAGS_BYPASS_THREAD_STATE to bypass using threadStateCheckTimeout,
+        // GPU_TIMEOUT_FLAGS_BYPASS_CPU_YIELD to not wait inside timeout with mutex held.
+        //
+        gpuSetTimeout(pGpu, GPU_TIMEOUT_DEFAULT, &params.timeout,
+                      GPU_TIMEOUT_FLAGS_BYPASS_THREAD_STATE |
+                      GPU_TIMEOUT_FLAGS_DEFAULT | GPU_TIMEOUT_FLAGS_BYPASS_CPU_YIELD);
+
+        //
+        // 2. Wait until we can issue an invalidate. On pre-Turing, wait for space
+        // in the PRI FIFO. On Turing, check if an invalidate is already in progress.
+        //
+        // Set the GFID.
+        params.gfid = gfid;
+
+        status = kgmmuCheckPendingInvalidates_HAL(pGpu, pKernelGmmu, &params.timeout);
+        if (status != NV_OK)
+        {
+           return;
+        }
     }
 
     // Trigger an invalidate.
@@ -133,7 +186,7 @@ kgmmuInvalidateTlb_GM107
     // Not using range-based invalidate.
     params.regVal = FLD_SET_DRF(_PFB_PRI, _MMU_INVALIDATE, _ALL_VA, _TRUE, params.regVal);
 
-    if ((NULL != pRootPageDir) && !pGpu->getProperty(pGpu, PDB_PROP_GPU_SRIOV_HEAVY_FORCE_INVALIDATE_ALL_PDBS_WAR_BUG3896322))
+    if (NULL != pRootPageDir)
     {
         // Invalidatating only one VAS.
         params.regVal = FLD_SET_DRF(_PFB_PRI, _MMU_INVALIDATE, _ALL_PDB, _FALSE, params.regVal);
@@ -178,16 +231,27 @@ kgmmuInvalidateTlb_GM107
     if (!(status == NV_OK || status == NV_ERR_NOT_SUPPORTED))
         return;
 
-    // 3 and 4. Commit the invalidate and wait for invalidate to complete.
-    status = kgmmuCommitTlbInvalidate_HAL(pGpu, pKernelGmmu, &params);
-    if (status != NV_OK)
+    if (bDoVgpuRpc)
     {
-       return;
+        NV_RM_RPC_INVALIDATE_TLB(pGpu, status, params.pdbAddress, params.regVal);
+        if (status != NV_OK)
+        {
+            return;
+        }
+    }
+    else
+    {
+        // 3 and 4. Commit the invalidate and wait for invalidate to complete.
+        status = kgmmuCommitTlbInvalidate_HAL(pGpu, pKernelGmmu, &params);
+        if (status != NV_OK)
+        {
+            return;
+        }
     }
 
     while (flushCount--)
     {
-        if (kbusFlush_HAL(pGpu, GPU_GET_KERNEL_BUS(pGpu), BUS_FLUSH_VIDEO_MEMORY) == NV_ERR_TIMEOUT)
+        if (kbusSendSysmembar(pGpu, GPU_GET_KERNEL_BUS(pGpu)) == NV_ERR_TIMEOUT)
         {
             break;
         }
@@ -221,7 +285,7 @@ kgmmuDetermineMaxVASize_GM107
             maxFmtVersionSupported = maxFmtVersionSupported < ver ? ver : maxFmtVersionSupported;
         }
     }
-    
+
     switch (maxFmtVersionSupported)
     {
         case GMMU_FMT_VERSION_1:
@@ -241,9 +305,6 @@ kgmmuDetermineMaxVASize_GM107
 /*
  * @brief Checks the system memory address against the PA capabilities of the
  *        GMMU.
- *
- * This will account for the DMA window, if applicable, and strip out any bits
- * that won't fit in the format (since the DMA window will be handling those).
  */
 void
 kgmmuEncodeSysmemAddrs_GM107
@@ -260,15 +321,8 @@ kgmmuEncodeSysmemAddrs_GM107
     for (i = 0; i < count; ++i)
     {
         NvU64 address = pAddresses[i];
-        NvU64 addressAdjusted = address;
 
-        NV_ASSERT(address >= pKernelGmmu->sysmemBaseAddress);
-        NV_ASSERT(address <= pKernelGmmu->sysmemBaseAddress + (NVBIT64(paWidth) - 1));
-
-        addressAdjusted &= (NVBIT64(paWidth) - 1);
-        NV_ASSERT(addressAdjusted == (address - pKernelGmmu->sysmemBaseAddress));
-
-        pAddresses[i] = addressAdjusted;
+        NV_ASSERT(address <= (NVBIT64(paWidth) - 1));
     }
 }
 
@@ -284,7 +338,7 @@ kgmmuEncodeSysmemAddrs_GM107
  *
  * @returns    The size of a large page in bytes
  */
-NvU32
+NvU64
 kgmmuGetMaxBigPageSize_GM107(KernelGmmu *pKernelGmmu)
 {
     if (!kgmmuIsPerVaspaceBigPageEn(pKernelGmmu))
@@ -317,6 +371,7 @@ kgmmuGetHwPteApertureFromMemdesc_GM107
             break;
         case ADDR_FABRIC_V2:
         case ADDR_FABRIC_MC:
+        case ADDR_EGM:
             aperture = NV_MMU_PTE_APERTURE_PEER_MEMORY;
             break;
         case ADDR_FBMEM:
