@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -29,15 +29,16 @@
 #include "kernel/gpu/fifo/kernel_channel_group.h"
 #include "kernel/gpu/fifo/kernel_channel_group_api.h"
 #include "kernel/gpu/mmu/kern_gmmu.h"
+#include "kernel/gpu/timer/objtmr.h"
 #include "kernel/os/os.h"
 #include "rmapi/client.h"
+#include "rmapi/rs_utils.h"
 
 #include "ctrl/ctrl506f.h"
 
 #include "libraries/utils/nvprintf.h"
 #include "nverror.h"
 #include "nvtypes.h"
-#include "objtmr.h"
 #include "vgpu/rpc.h"
 
 
@@ -60,12 +61,27 @@ _vgpuRcResetCallback
         {
             THREAD_STATE_NODE                             threadState;
             NV506F_CTRL_CMD_RESET_ISOLATED_CHANNEL_PARAMS params = {0};
+            RsClient      *pClient;
+            KernelChannel *pKernelChannel = NULL;
 
             threadStateInitISRAndDeferredIntHandler(
                 &threadState,
                 pRcErrorContext->pGpu,
                 THREAD_STATE_FLAGS_IS_DEFERRED_INT_HANDLER);
 
+            NV_ASSERT_OK_OR_GOTO(
+                status,
+                serverGetClientUnderLock(&g_resServ, hClient, &pClient),
+                error_cleanup);
+            NV_ASSERT_OK_OR_GOTO(
+                status,
+                CliGetKernelChannel(pClient, hChannel, &pKernelChannel),
+                error_cleanup);
+ 
+            NV_ASSERT_OR_ELSE(pKernelChannel != NULL,
+                              status = NV_ERR_INVALID_STATE;
+                              goto error_cleanup);
+
             params.engineID   = pRcErrorContext->EngineId;
             params.exceptType = pRcErrorContext->exceptType;
 
@@ -77,13 +93,14 @@ _vgpuRcResetCallback
                               sizeof params,
                               status);
 
+            rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
+
             threadStateFreeISRAndDeferredIntHandler(
                 &threadState,
                 pRcErrorContext->pGpu,
                 THREAD_STATE_FLAGS_IS_DEFERRED_INT_HANDLER);
 
             portMemFree(pRcErrorContext);
-            rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
         }
         else
         {
@@ -97,6 +114,11 @@ _vgpuRcResetCallback
     }
 
     return status;
+
+error_cleanup:
+    rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
+    osReleaseRmSema(pSys->pSema, NULL);
+    return status;
 }
 
 
@@ -147,6 +169,7 @@ krcResetCallback
             if (rmGpuLocksAcquire(GPUS_LOCK_FLAGS_NONE, RM_LOCK_MODULES_RC) ==
                 NV_OK)
             {
+                RsClient      *pClient;
                 KernelChannel *pKernelChannel = NULL;
 
                 threadStateInitISRAndDeferredIntHandler(
@@ -156,11 +179,11 @@ krcResetCallback
 
                 NV_ASSERT_OK_OR_GOTO(
                     status,
-                    serverGetClientUnderLock(&g_resServ, hClient, NULL),
+                    serverGetClientUnderLock(&g_resServ, hClient, &pClient),
                     error_cleanup);
                 NV_ASSERT_OK_OR_GOTO(
                     status,
-                    CliGetKernelChannel(hClient, hChannel, &pKernelChannel),
+                    CliGetKernelChannel(pClient, hChannel, &pKernelChannel),
                     error_cleanup);
 
                 NV_ASSERT_OR_ELSE(pKernelChannel != NULL,
@@ -180,13 +203,14 @@ krcResetCallback
                         sizeof params);
                 }
 
+                rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
+
                 threadStateFreeISRAndDeferredIntHandler(
                     &threadState,
                     pRcErrorContext->pGpu,
                     THREAD_STATE_FLAGS_IS_DEFERRED_INT_HANDLER);
 
                 portMemFree(pRcErrorContext);
-                rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
             }
             else
             {
@@ -230,26 +254,19 @@ krcErrorInvokeCallback_IMPL
 {
     OBJSYS             *pSys              = SYS_GET_INSTANCE();
     Journal            *pRcDB             = SYS_GET_RCDB(pSys);
-    OBJOS              *pOS               = SYS_GET_OS(pSys);
     KernelMIGManager   *pKernelMigManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
     RmClient           *pClient           = NULL;
-    RsClient           *pRsClient;
     RC_CALLBACK_STATUS  clientAction;
     RM_ENGINE_TYPE      localRmEngineType  = rmEngineType;
     NvU32               rcDiagRecOwner = RCDB_RCDIAG_DEFAULT_OWNER;
     NV_STATUS           status;
     NvBool              bReturn = NV_TRUE;
+    NvBool              bCheckCallback;
 
     NV_ASSERT_OR_RETURN(!gpumgrGetBcEnabledStatus(pGpu), bReturn);
     NV_CHECK_OR_RETURN(LEVEL_ERROR, pKernelChannel != NULL, bReturn);
 
-    status = serverGetClientUnderLock(&g_resServ,
-                                      RES_GET_CLIENT_HANDLE(pKernelChannel),
-                                      &pRsClient);
-    if (status != NV_OK)
-        return bReturn;
-
-    pClient = dynamicCast(pRsClient, RmClient);
+    pClient = dynamicCast(RES_GET_CLIENT(pKernelChannel), RmClient);
     if (pClient == NULL)
         return bReturn;
 
@@ -261,9 +278,9 @@ krcErrorInvokeCallback_IMPL
         kmigmgrIsEnginePartitionable(pGpu, pKernelMigManager, rmEngineType))
     {
         MIG_INSTANCE_REF ref;
-        status = kmigmgrGetInstanceRefFromClient(pGpu,
+        status = kmigmgrGetInstanceRefFromDevice(pGpu,
                                                  pKernelMigManager,
-                                                 pRsClient->hClient,
+                                                 GPU_RES_GET_DEVICE(pKernelChannel),
                                                  &ref);
         if (status != NV_OK)
             return bReturn;
@@ -286,17 +303,15 @@ krcErrorInvokeCallback_IMPL
             return bReturn;
     }
 
-    if (pOS->osCheckCallback(pGpu))
+    bCheckCallback =
+        IS_GSP_CLIENT(pGpu) ? osCheckCallback_v2(pGpu) : osCheckCallback(pGpu);
+
+    if (bCheckCallback)
     {
-        NvHandle          hDevice, hFifo;
         RC_ERROR_CONTEXT *pRcErrorContext = NULL;
-        Device           *pDevice;
-
-        NV_ASSERT_OK_OR_RETURN(
-            deviceGetByGpu(pRsClient, pGpu, NV_TRUE, &pDevice));
-
-        hDevice = RES_GET_HANDLE(pDevice);
-
+        Device           *pDevice = GPU_RES_GET_DEVICE(pKernelChannel);
+        NvHandle          hDevice = RES_GET_HANDLE(pDevice);
+        NvHandle          hFifo;
 
         if (!pKernelChannel->pKernelChannelGroupApi->pKernelChannelGroup
                  ->bAllocatedByRm)
@@ -330,17 +345,37 @@ krcErrorInvokeCallback_IMPL
                     GPU_GET_KERNEL_GMMU(pGpu),
                     pMmuExceptionData->faultType);
             }
+            else
+            {
+                // TODO: Set some default values
+            }
         }
 
-        clientAction = pOS->osRCCallback(pGpu,
-                                         RES_GET_CLIENT_HANDLE(pKernelChannel),
-                                         hDevice,
-                                         hFifo,
-                                         RES_GET_HANDLE(pKernelChannel),
-                                         exceptLevel,
-                                         exceptType,
-                                         (NvU32 *)pRcErrorContext,
-                                         &krcResetCallback);
+        if (IS_GSP_CLIENT(pGpu))
+        {
+            clientAction = osRCCallback_v2(pGpu,
+                                           RES_GET_CLIENT_HANDLE(pKernelChannel),
+                                           hDevice,
+                                           hFifo,
+                                           RES_GET_HANDLE(pKernelChannel),
+                                           exceptLevel,
+                                           exceptType,
+                                           NV_FALSE,
+                                           (NvU32 *)pRcErrorContext,
+                                           &krcResetCallback);
+        }
+        else
+        {
+            clientAction = osRCCallback(pGpu,
+                                        RES_GET_CLIENT_HANDLE(pKernelChannel),
+                                        hDevice,
+                                        hFifo,
+                                        RES_GET_HANDLE(pKernelChannel),
+                                        exceptLevel,
+                                        exceptType,
+                                        (NvU32 *)pRcErrorContext,
+                                        &krcResetCallback);
+        }
 
         if (clientAction == RC_CALLBACK_IGNORE ||
             clientAction == RC_CALLBACK_ISOLATE_NO_RESET)
@@ -357,6 +392,20 @@ krcErrorInvokeCallback_IMPL
             //
             portMemFree(pRcErrorContext);
         }
+        else if (IS_GSP_CLIENT(pGpu) && clientAction == RC_CALLBACK_ISOLATE)
+        {
+            NV506F_CTRL_CMD_RESET_ISOLATED_CHANNEL_PARAMS params = {0};
+            RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pRcErrorContext->pGpu);
+
+            // Client lock is already obtained above.
+            status = pRmApi->Control(pRmApi,
+                                     RES_GET_CLIENT_HANDLE(pKernelChannel),
+                                     RES_GET_HANDLE(pKernelChannel),
+                                     NV506F_CTRL_CMD_RESET_ISOLATED_CHANNEL,
+                                     &params,
+                                     sizeof params);
+        }
+
         bReturn = (clientAction != RC_CALLBACK_IGNORE);
     }
     else
@@ -380,10 +429,10 @@ krcErrorInvokeCallback_IMPL
                                &classInfo);
 
         // notify the Fifo channel based event listeners
-        kchannelNotifyGeneric(pKernelChannel,
-                              classInfo.rcNotifierIndex,
-                              &params,
-                              sizeof(params));
+        kchannelNotifyEvent(pKernelChannel,
+                            classInfo.rcNotifierIndex,
+                            0, 0, &params,
+                            sizeof(params));
     }
 
     // update RC diagnostic records with process id and owner
