@@ -22,7 +22,7 @@
  */
 
 
-/***************************** HW State Rotuines ***************************\
+/***************************** HW State Routines ***************************\
 *                                                                           *
 *         IOMMU Virtual Address Space Function Definitions.                 *
 *                                                                           *
@@ -156,6 +156,7 @@ iovaspaceApplyDefaultAlignment_IMPL
                                 NV_ERR_INVALID_ARGUMENT);
                 maxPageSize = RM_PAGE_SIZE_512M;
                 break;
+            case RM_ATTR_PAGE_SIZE_256GB:
             default:
                 break;
         }
@@ -208,7 +209,7 @@ iovaspaceGetVaLimit_IMPL(OBJIOVASPACE *pIOVAS)
     return NVBIT64(32) - 1;
 }
 
-#if (RMCFG_FEATURE_PLATFORM_UNIX || RMCFG_FEATURE_PLATFORM_MODS) && !NVCPU_IS_ARM
+#if (RMCFG_FEATURE_PLATFORM_UNIX || RMCFG_FEATURE_MODS_FEATURES) && !NVCPU_IS_ARM
 static PIOVAMAPPING
 _iovaspaceCreateMappingDataFromMemDesc
 (
@@ -601,4 +602,4 @@ void iovaMappingDestroy(PIOVAMAPPING pIovaMapping)
     iovaspaceDestroyMapping(pIOVAS, pIovaMapping);
 }
 
-#endif // (RMCFG_FEATURE_PLATFORM_UNIX || RMCFG_FEATURE_PLATFORM_MODS) && !NVCPU_IS_ARM
+#endif // (RMCFG_FEATURE_PLATFORM_UNIX || RMCFG_FEATURE_MODS_FEATURES) && !NVCPU_IS_ARM
