@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -39,9 +39,7 @@
 #include "mem_mgr/vaspace.h"
 #include "gpu/mem_mgr/mem_mgr.h"
 #include "gpu/mem_mgr/mem_utils.h"
-#include "gpu/mem_mgr/heap.h"
 #include "gpu/device/device.h"
-#include "gpu/subdevice/subdevice.h"
 #include "os/os.h"
 #include "compute/fabric.h"
 #include "gpu/mem_mgr/mem_desc.h"
@@ -52,6 +50,7 @@
 #include "gpu/bus/kern_bus.h"
 #include "gpu/bus/p2p_api.h"
 #include "kernel/gpu/nvlink/kernel_nvlink.h"
+#include "kernel/gpu/gpu_fabric_probe.h"
 #include "ctrl/ctrl0041.h"
 
 static NvU32
@@ -64,7 +63,7 @@ _memoryfabricMemDescGetNumAddr
     NvU64   pageSize = 0;
 
     // Get the page size from the memory descriptor.
-    pageSize = memdescGetPageSize64(pMemDesc,
+    pageSize = memdescGetPageSize(pMemDesc,
                             VAS_ADDRESS_TRANSLATION(pGpu->pFabricVAS));
 
     // Get the number of addresses associated with this memory descriptor.
@@ -82,47 +81,42 @@ _memoryfabricMemDescGetNumAddr
 static NV_STATUS
 _memoryfabricValidatePhysMem
 (
-    NvHandle           hClient,
-    NvHandle           hPhysMem,
-    OBJGPU            *pOwnerGpu,
-    MEMORY_DESCRIPTOR **ppPhysMemDesc
+    RsClient           *pRsClient,
+    NvHandle            hPhysMem,
+    OBJGPU             *pOwnerGpu,
+    MEMORY_DESCRIPTOR **ppPhysMemDesc,
+    Memory            **ppPhysMemory
 )
 {
     MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pOwnerGpu);
-    RsResourceRef *pPhysmemRef;
     MEMORY_DESCRIPTOR *pPhysMemDesc;
     NvU64 physPageSize;
     NV_STATUS status;
+    Memory *pMemory;
 
-    if (hPhysMem == 0)
-    {
-        NV_PRINTF(LEVEL_ERROR, "Invalid physmem handle\n");
-
-        return NV_ERR_INVALID_ARGUMENT;
-    }
-
-    status = serverutilGetResourceRef(hClient, hPhysMem, &pPhysmemRef);
+    status = memGetByHandle(pRsClient, hPhysMem, &pMemory);
     if (status != NV_OK)
     {
-        NV_PRINTF(LEVEL_ERROR,
-                  "Failed to get resource in resserv for physmem handle\n");
-
+        NV_PRINTF(LEVEL_ERROR, "Invalid object handle passed\n");
         return status;
     }
 
-    pPhysMemDesc = (dynamicCast(pPhysmemRef->pResource, Memory))->pMemDesc;
+    pPhysMemDesc = pMemory->pMemDesc;
 
-    if (!memmgrIsApertureSupportedByFla_HAL(pOwnerGpu, pMemoryManager, memdescGetAddressSpace(pPhysMemDesc)) ||
-        (pOwnerGpu != pPhysMemDesc->pGpu))
+    if ((pOwnerGpu != pPhysMemDesc->pGpu) ||
+        !memmgrIsMemDescSupportedByFla_HAL(pOwnerGpu,
+                                           pMemoryManager,
+                                           pPhysMemDesc))
     {
         NV_PRINTF(LEVEL_ERROR, "Invalid physmem handle passed\n");
 
         return NV_ERR_INVALID_ARGUMENT;
     }
 
-    physPageSize = memdescGetPageSize64(pPhysMemDesc, AT_GPU);
-    if ((physPageSize != NV_MEMORY_FABRIC_PAGE_SIZE_2M) &&
-        (physPageSize != NV_MEMORY_FABRIC_PAGE_SIZE_512M))
+    physPageSize = memdescGetPageSize(pPhysMemDesc, AT_GPU);
+    if (
+        (physPageSize != RM_PAGE_SIZE_HUGE) &&
+        (physPageSize != RM_PAGE_SIZE_512M))
     {
         NV_PRINTF(LEVEL_ERROR, "Physmem page size should be 2MB\n");
 
@@ -131,21 +125,26 @@ _memoryfabricValidatePhysMem
 
     *ppPhysMemDesc = pPhysMemDesc;
 
+    if (ppPhysMemory != NULL)
+        *ppPhysMemory = pMemory;
+
     return NV_OK;
 }
 
 static NV_STATUS
 _memoryFabricDetachMem
 (
-    MEMORY_DESCRIPTOR   *pFabricMemDesc,
-    NvU64                offset
+    MemoryFabric        *pMemoryFabric,
+    NvU64                offset,
+    NvBool               bRemoveInterMapping
 )
 {
     NV_STATUS status;
-    RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
     FABRIC_ATTCH_MEM_INFO_NODE *pAttachMemInfoNode;
     NODE *pNode = NULL;
     FABRIC_VASPACE *pFabricVAS;
+    Memory *pMemory = staticCast(pMemoryFabric, Memory);
+    MEMORY_DESCRIPTOR *pFabricMemDesc = pMemory->pMemDesc;
     MEMORY_DESCRIPTOR *pPhysMemDesc;
     FABRIC_MEMDESC_DATA *pMemdescData;
 
@@ -163,8 +162,11 @@ _memoryFabricDetachMem
                                   pPhysMemDesc,
                                   pAttachMemInfoNode->physMapLength);
 
-    NV_ASSERT_OK(pRmApi->Free(pRmApi, pFabricVAS->hClient,
-                              pAttachMemInfoNode->hDupedPhysMem));
+    if (bRemoveInterMapping)
+    {
+        refRemoveInterMapping(RES_GET_REF(pMemoryFabric),
+                              pAttachMemInfoNode->pInterMapping);
+    }
 
     btreeUnlink(&pAttachMemInfoNode->node, &pMemdescData->pAttachMemInfoTree);
 
@@ -173,29 +175,6 @@ _memoryFabricDetachMem
     return NV_OK;
 }
 
-static void
-_memoryFabricBatchDetachMem
-(
-    MEMORY_DESCRIPTOR *pFabricMemDesc
-)
-{
-    FABRIC_MEMDESC_DATA *pMemdescData;
-    NODE *pNode = NULL;
-    NvU64 offset;
-    pMemdescData = (FABRIC_MEMDESC_DATA *)memdescGetMemData(pFabricMemDesc);
-
-    if (pMemdescData == NULL)
-        return;
-
-    btreeEnumStart(0, &pNode, pMemdescData->pAttachMemInfoTree);
-    while (pNode != NULL)
-    {
-        offset = pNode->keyStart;
-        btreeEnumNext(&pNode, pMemdescData->pAttachMemInfoTree);
-        NV_ASSERT_OK(_memoryFabricDetachMem(pFabricMemDesc, offset));
-    }
-}
-
 static NV_STATUS
 _memoryFabricAttachMem
 (
@@ -209,10 +188,10 @@ _memoryFabricAttachMem
     MEMORY_DESCRIPTOR *pPhysMemDesc;
     MEMORY_DESCRIPTOR *pFabricMemDesc = pMemory->pMemDesc;
     FABRIC_MEMDESC_DATA *pMemdescData;
-    NvHandle hDupedPhysMem = 0;
-    RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
     FABRIC_VASPACE *pFabricVAS = dynamicCast(pGpu->pFabricVAS, FABRIC_VASPACE);
     FABRIC_ATTCH_MEM_INFO_NODE *pNode;
+    RsInterMapping *pInterMapping;
+    Memory *pPhysMemory;
 
     pMemdescData = (FABRIC_MEMDESC_DATA *)memdescGetMemData(pFabricMemDesc);
 
@@ -222,18 +201,25 @@ _memoryFabricAttachMem
         return NV_ERR_NOT_SUPPORTED;
     }
 
-    status = _memoryfabricValidatePhysMem(RES_GET_CLIENT_HANDLE(pMemory),
-                                          pAttachInfo->hMemory,
-                                          pGpu, &pPhysMemDesc);
+    if (gpuIsCCFeatureEnabled(pGpu) && !gpuIsCCMultiGpuProtectedPcieModeEnabled(pGpu))
+    {
+        NV_PRINTF(LEVEL_ERROR,
+                  "Unsupported when Confidential Computing is enabled in SPT\n");
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    status = _memoryfabricValidatePhysMem(RES_GET_CLIENT(pMemory), pAttachInfo->hMemory,
+                                          pGpu, &pPhysMemDesc, &pPhysMemory);
 
     NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, status);
 
-    status = pRmApi->DupObject(pRmApi, pFabricVAS->hClient, pFabricVAS->hDevice,
-                               &hDupedPhysMem, RES_GET_CLIENT_HANDLE(pMemory),
-                               pAttachInfo->hMemory, 0);
+    status = refAddInterMapping(RES_GET_REF(pMemoryFabric),
+                                RES_GET_REF(pPhysMemory),
+                                RES_GET_REF(pMemoryFabric)->pParentRef,
+                                &pInterMapping);
     if (status != NV_OK)
     {
-        NV_PRINTF(LEVEL_ERROR, "Failed to dup physmem handle\n");
+        NV_PRINTF(LEVEL_ERROR, "Failed to setup inter mapping\n");
         return status;
     }
 
@@ -247,7 +233,7 @@ _memoryFabricAttachMem
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "Failed to map FLA\n");
-        goto freeDupedMem;
+        goto freeInterMapping;
     }
 
     pNode = portMemAllocNonPaged(sizeof(*pNode));
@@ -264,9 +250,15 @@ _memoryFabricAttachMem
     pNode->physMapOffset = pAttachInfo->mapOffset;
     pNode->physMapLength = pAttachInfo->mapLength;
     pNode->pPhysMemDesc  = pPhysMemDesc;
-    pNode->hDupedPhysMem = hDupedPhysMem;
+    pNode->pInterMapping = pInterMapping;
     pNode->node.Data     = pNode;
 
+    pInterMapping->dmaOffset = pAttachInfo->offset;
+    pInterMapping->pMemDesc = pPhysMemDesc;
+
+    // No partial unmap supported
+    pInterMapping->size = 0;
+
     status = btreeInsert(&pNode->node, &pMemdescData->pAttachMemInfoTree);
     if (status != NV_OK)
     {
@@ -280,11 +272,12 @@ freeNode:
     portMemFree(pNode);
 
 unmapVas:
-    fabricvaspaceUnmapPhysMemdesc(pFabricVAS, pFabricMemDesc, pAttachInfo->offset,
+    fabricvaspaceUnmapPhysMemdesc(pFabricVAS, pFabricMemDesc,
+                                  pAttachInfo->offset,
                                   pPhysMemDesc, pAttachInfo->mapLength);
 
-freeDupedMem:
-    NV_ASSERT_OK(pRmApi->Free(pRmApi, pFabricVAS->hClient, hDupedPhysMem));
+freeInterMapping:
+    refRemoveInterMapping(RES_GET_REF(pMemoryFabric), pInterMapping);
 
     return status;
 }
@@ -303,30 +296,35 @@ _memoryfabricMemDescDestroyCallback
     FABRIC_MEMDESC_DATA *pMemdescData;
     NvU32                numAddr;
     NvU64                pageSize;
+    NODE                *pNode;
 
     NV_ASSERT_OR_RETURN_VOID(pGpu->pFabricVAS != NULL);
 
     pFabricVAS = dynamicCast(pGpu->pFabricVAS, FABRIC_VASPACE);
     pMemdescData = (FABRIC_MEMDESC_DATA *)memdescGetMemData(pMemDesc);
-    pteArray = memdescGetPteArrayForGpu(pMemDesc, pGpu, VAS_ADDRESS_TRANSLATION(pGpu->pFabricVAS));
+    pteArray = memdescGetPteArrayForGpu(pMemDesc, pGpu,
+                                    VAS_ADDRESS_TRANSLATION(pGpu->pFabricVAS));
     numAddr = _memoryfabricMemDescGetNumAddr(pMemDesc);
     // Get the page size from the memory descriptor.
-    pageSize = memdescGetPageSize64(pMemDesc, VAS_ADDRESS_TRANSLATION(pGpu->pFabricVAS));
+    pageSize = memdescGetPageSize(pMemDesc,
+                                    VAS_ADDRESS_TRANSLATION(pGpu->pFabricVAS));
 
     // Remove the fabric memory allocations from the map.
     fabricvaspaceVaToGpaMapRemove(pFabricVAS, pteArray[0]);
 
-    // Detach any pending memory...
-    _memoryFabricBatchDetachMem(pMemDesc);
+    // Every attached memory should have been unmapped by now..
+    btreeEnumStart(0, &pNode, pMemdescData->pAttachMemInfoTree);
+    NV_ASSERT(pNode == NULL);
 
     if (!pFabricVAS->bRpcAlloc)
     {
         //
         // Call fabricvaspaceBatchFree to free the FLA allocations.
-        // _pteArray in memdesc is RM_PAGE_SIZE whereas page size for memory fabric
-        // allocations is either 2MB or 512MB. Pass stride accordingly.
+        // _pteArray in memdesc is RM_PAGE_SIZE whereas page size for memory
+        // fabric allocations is either 2MB or 512MB. Pass stride accordingly.
         //
-        fabricvaspaceBatchFree(pFabricVAS, pteArray, numAddr, (pageSize >> RM_PAGE_SHIFT));
+        fabricvaspaceBatchFree(pFabricVAS, pteArray, numAddr,
+                               (pageSize >> RM_PAGE_SHIFT));
     }
 
     if (pMemdescData != NULL)
@@ -398,12 +396,14 @@ _memoryfabricAllocFabricVa_VGPU
                            pParams->hResource,
                            pParams->externalClassId,
                            pAllocParams,
+                           sizeof(*pAllocParams),
                            status);
 
     if (status != NV_OK)
     {
-        NV_PRINTF(LEVEL_ERROR, "Alloc NV_MEMORY_FABRIC RPC failed, status: %x\n",
-                    status);
+        NV_PRINTF(LEVEL_ERROR,
+                  "Alloc NV_MEMORY_FABRIC RPC failed, status: %x\n",
+                  status);
         return status;
     }
 
@@ -427,14 +427,16 @@ _memoryfabricAllocFabricVa_VGPU
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_ERROR, "CTRL_CMD_DESCRIBE failed, status: 0x%x, "
-                        "numPfns: 0x%x, totalPfns: 0x%llx, readSoFar: 0x%x \n",
-                        status, pDescribeParams->numPfns, pDescribeParams->totalPfns, idx);
+                      "numPfns: 0x%x, totalPfns: 0x%llx, readSoFar: 0x%x \n",
+                      status, pDescribeParams->numPfns,
+                      pDescribeParams->totalPfns, idx);
             goto cleanup;
         }
 
         if (pAddr == NULL)
         {
-            pAddr = portMemAllocNonPaged(sizeof(NvU64) * pDescribeParams->totalPfns);
+            pAddr = portMemAllocNonPaged(sizeof(NvU64) *
+                                         pDescribeParams->totalPfns);
             if (pAddr == NULL)
             {
                 status = NV_ERR_NO_MEMORY;
@@ -444,7 +446,8 @@ _memoryfabricAllocFabricVa_VGPU
 
         for (i=0; i < pDescribeParams->numPfns; i++)
         {
-            pAddr[idx + i] = (NvU64)((NvU64)pDescribeParams->pfnArray[i] << RM_PAGE_SHIFT_HUGE);
+            pAddr[idx + i] =
+                (NvU64)((NvU64)pDescribeParams->pfnArray[i] << RM_PAGE_SHIFT_HUGE);
         }
 
         idx += pDescribeParams->numPfns;
@@ -504,25 +507,25 @@ memoryfabricConstruct_IMPL
     RS_RES_ALLOC_PARAMS_INTERNAL *pParams
 )
 {
-    Memory                       *pMemory        = staticCast(pMemoryFabric, Memory);
-    OBJGPU                       *pGpu           = pMemory->pGpu;
-    FABRIC_VASPACE               *pFabricVAS     = dynamicCast(pGpu->pFabricVAS, FABRIC_VASPACE);
+    Memory *pMemory = staticCast(pMemoryFabric, Memory);
+    OBJGPU *pGpu = pMemory->pGpu;
+    FABRIC_VASPACE *pFabricVAS  = dynamicCast(pGpu->pFabricVAS, FABRIC_VASPACE);
     NV00F8_ALLOCATION_PARAMETERS *pAllocParams   = pParams->pAllocParams;
-    RM_API                       *pRmApi         = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
-    MemoryManager                *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
-    MEMORY_DESCRIPTOR            *pPhysMemDesc    = NULL;
-    NV_STATUS                     status         = NV_OK;
-    MEMORY_DESCRIPTOR            *pMemDesc       = NULL;
-    FABRIC_MEMDESC_DATA          *pMemdescData   = NULL;
-    MEM_DESC_DESTROY_CALLBACK    *pCallback      = NULL;
-    VAS_ALLOC_FLAGS               flags          = {0};
-    NvU64                        *pAddr          = NULL;
-    NvU32                         numAddr        = 0;
-    NvU32                         pteKind        = 0;
-    NvBool                        bReadOnly      = NV_FALSE;
-    NvHandle                      hPhysMem;
-    NvBool                        bFlexible      = NV_FALSE;
-    NvU32                         mapFlags       = 0;
+    RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+    MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
+    MEMORY_DESCRIPTOR *pPhysMemDesc = NULL;
+    NV_STATUS status = NV_OK;
+    MEMORY_DESCRIPTOR *pMemDesc = NULL;
+    FABRIC_MEMDESC_DATA *pMemdescData = NULL;
+    MEM_DESC_DESTROY_CALLBACK *pCallback = NULL;
+    VAS_ALLOC_FLAGS flags = {0};
+    NvU64   *pAddr = NULL;
+    NvU32    numAddr = 0;
+    NvU32    pteKind = 0;
+    NvBool   bReadOnly = NV_FALSE;
+    NvHandle hPhysMem;
+    NvBool   bFlexible = NV_FALSE;
+    NvU32    mapFlags = 0;
 
     if (RS_IS_COPY_CTOR(pParams))
     {
@@ -541,18 +544,20 @@ memoryfabricConstruct_IMPL
         return NV_ERR_NOT_SUPPORTED;
     }
 
-    // initialize Fabric VAS Unicast range if not already setup 
+    // initialize Fabric VAS Unicast range if not already setup
     if (fabricvaspaceGetUCFlaLimit(pFabricVAS) == 0)
     {
-        NV_PRINTF(LEVEL_ERROR, "UC FLA ranges should be initialized by this time!\n");
+        NV_PRINTF(LEVEL_ERROR,
+                  "UC FLA ranges should be initialized by this time!\n");
         return NV_ERR_INVALID_STATE;
     }
 
-    // Only page size 512MB and 2MB supported.
-    if ((pAllocParams->pageSize != NV_MEMORY_FABRIC_PAGE_SIZE_512M) &&
+    if (
+        (pAllocParams->pageSize != NV_MEMORY_FABRIC_PAGE_SIZE_256G) &&
+        (pAllocParams->pageSize != NV_MEMORY_FABRIC_PAGE_SIZE_512M) &&
         (pAllocParams->pageSize != NV_MEMORY_FABRIC_PAGE_SIZE_2M))
     {
-        NV_PRINTF(LEVEL_ERROR, "Unsupported pageSize: 0x%x\n",
+        NV_PRINTF(LEVEL_ERROR, "Unsupported pageSize: 0x%llx\n",
                   pAllocParams->pageSize);
 
         return NV_ERR_INVALID_ARGUMENT;
@@ -599,8 +604,8 @@ memoryfabricConstruct_IMPL
     }
     else if (!bFlexible)
     {
-        status = _memoryfabricValidatePhysMem(pCallContext->pClient->hClient,
-                                              hPhysMem, pGpu, &pPhysMemDesc);
+        status = _memoryfabricValidatePhysMem(pCallContext->pClient, hPhysMem,
+                                              pGpu, &pPhysMemDesc, NULL);
         if (status != NV_OK)
             return status;
     }
@@ -621,7 +626,7 @@ memoryfabricConstruct_IMPL
     {
         NV_PRINTF(LEVEL_ERROR,
                   "VA Space alloc failed! Status Code: 0x%x Size: 0x%llx "
-                  "RangeLo: 0x%llx, RangeHi: 0x%llx, page size: 0x%x\n",
+                  "RangeLo: 0x%llx, RangeHi: 0x%llx, page size: 0x%llx\n",
                   status, pAllocParams->allocSize,
                   fabricvaspaceGetUCFlaStart(pFabricVAS),
                   fabricvaspaceGetUCFlaLimit(pFabricVAS),
@@ -700,7 +705,7 @@ memoryfabricConstruct_IMPL
     // Associate the memdescDestroy callback function.
     pCallback->pObject         = (void *)pCallback;
     pCallback->destroyCallback =
-                   (MemDescDestroyCallBack*) &_memoryfabricMemDescDestroyCallback;
+                (MemDescDestroyCallBack*) &_memoryfabricMemDescDestroyCallback;
 
     memdescAddDestroyCallback(pMemDesc, pCallback);
 
@@ -713,9 +718,11 @@ memoryfabricConstruct_IMPL
     if (hPhysMem != 0)
     {
         // Dup the physical memory handle and cache it in memfabric memdesc.
-        status = pRmApi->DupObject(pRmApi, pFabricVAS->hClient, pFabricVAS->hDevice,
-                            &pMemdescData->hDupedPhysMem, pCallContext->pClient->hClient,
-                            hPhysMem, 0);
+        status = pRmApi->DupObject(pRmApi, pFabricVAS->hClient,
+                                   pFabricVAS->hDevice,
+                                   &pMemdescData->hDupedPhysMem,
+                                   pCallContext->pClient->hClient,
+                                   hPhysMem, 0);
 
         if (status != NV_OK)
         {
@@ -727,7 +734,7 @@ memoryfabricConstruct_IMPL
         NV0041_CTRL_GET_SURFACE_INFO_PARAMS surfaceInfoParam = {0};
 
         surfaceInfo[0].index = NV0041_CTRL_SURFACE_INFO_INDEX_ADDR_SPACE_TYPE;
-        surfaceInfo[1].index = NV0041_CTRL_SURFACE_INFO_INDEX_COMPR_COVERAGE;
+        surfaceInfo[1].index = NV0041_CTRL_SURFACE_INFO_INDEX_ATTRS;
         surfaceInfoParam.surfaceInfoListSize = 2;
         surfaceInfoParam.surfaceInfoList = NvP64_VALUE(&surfaceInfo);
 
@@ -744,7 +751,14 @@ memoryfabricConstruct_IMPL
         }
 
         pMemdescData->physAttrs.addressSpace = surfaceInfo[0].data;
-        pMemdescData->physAttrs.compressionCoverage = surfaceInfo[1].data;
+        //
+        // TODO: Bug 4322867: use NV0041_CTRL_SURFACE_INFO_ATTRS_COMPR
+        // instead of NV0041_CTRL_SURFACE_INFO_INDEX_COMPR_COVERAGE.
+        // NV0041_CTRL_SURFACE_INFO_INDEX_COMPR_COVERAGE is buggy and
+        // will be removed soon.
+        //
+        pMemdescData->physAttrs.compressionCoverage =
+            (surfaceInfo[1].data & NV0041_CTRL_SURFACE_INFO_ATTRS_COMPR) ?  0x1 : 0x0;
 
         mapFlags |= bReadOnly ? FABRIC_VASPACE_MAP_FLAGS_READ_ONLY : 0;
 
@@ -769,7 +783,8 @@ memoryfabricConstruct_IMPL
         // No need to unmap on failure. Unmap happens implicitly when fabric VA
         // would be freed.
         //
-        status = fabricvaspaceVaToGpaMapInsert(pFabricVAS, pAddr[0], pPhysMemDesc,
+        status = fabricvaspaceVaToGpaMapInsert(pFabricVAS, pAddr[0],
+                                               pPhysMemDesc,
                                                pAllocParams->map.offset);
         if (status != NV_OK)
             goto freeDupedMem;
@@ -905,15 +920,54 @@ memoryfabricCtrlCmdDescribe_IMPL
     NV00F8_CTRL_DESCRIBE_PARAMS   *pParams
 )
 {
+    NV_STATUS status;
     Memory *pMemory = staticCast(pMemoryFabric, Memory);
     NvU64  *pFabricArray;
     NvU64   offset;
     NvU64   pageSize;
     NvU32   i;
+    CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
+    FABRIC_MEMDESC_DATA *pMemdescData;
+    OBJGPU *pGpu;
+
+    RmClient *pRmClient = dynamicCast(pCallContext->pClient, RmClient);
+    NV_ASSERT_OR_RETURN(pRmClient != NULL, NV_ERR_INVALID_CLIENT);
+
+    if (
+        !rmclientIsCapableOrAdmin(pRmClient, NV_RM_CAP_SYS_FABRIC_IMEX_MGMT, pCallContext->secInfo.privLevel)
+    )
+    {
+        return NV_ERR_INSUFFICIENT_PERMISSIONS;
+    }
 
     if (pMemory->pMemDesc == NULL)
         return NV_ERR_INVALID_ARGUMENT;
 
+    pGpu = pMemory->pMemDesc->pGpu;
+    pMemdescData = (FABRIC_MEMDESC_DATA *)memdescGetMemData(pMemory->pMemDesc);
+
+    pParams->memFlags = pMemdescData->allocFlags;
+    pParams->physAttrs = pMemdescData->physAttrs;
+
+    pParams->attrs.pageSize = memdescGetPageSize(pMemory->pMemDesc, AT_GPU);
+    pParams->attrs.kind = memdescGetPteKind(pMemory->pMemDesc);
+    pParams->attrs.size = memdescGetSize(pMemory->pMemDesc);
+
+    if (gpuFabricProbeIsSupported(pGpu))
+    {
+        status = gpuFabricProbeGetFabricCliqueId(pGpu->pGpuFabricProbeInfoKernel,
+                                                 &pParams->attrs.cliqueId);
+        if (status != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR, "unable to query cliqueId 0x%x\n", status);
+            return status;
+        }
+    }
+    else
+    {
+        pParams->attrs.cliqueId = 0;
+    }
+
     pageSize = memdescGetPageSize(pMemory->pMemDesc, AT_GPU);
 
     if (memdescGetContiguity(pMemory->pMemDesc, AT_GPU))
@@ -923,8 +977,8 @@ memoryfabricCtrlCmdDescribe_IMPL
 
     if (pParams->offset >= pParams->totalPfns)
     {
-        NV_PRINTF(LEVEL_ERROR, "offset: %llx is out of range: %llx \n", pParams->offset,
-                  pParams->totalPfns);
+        NV_PRINTF(LEVEL_ERROR, "offset: 0x%llx is out of range: 0x%llx\n",
+                  pParams->offset, pParams->totalPfns);
         return NV_ERR_OUT_OF_RANGE;
     }
 
@@ -937,9 +991,8 @@ memoryfabricCtrlCmdDescribe_IMPL
         return NV_ERR_NO_MEMORY;
 
     offset = pParams->offset * pageSize;
-    memdescGetPhysAddrsForGpu(pMemory->pMemDesc, pMemory->pMemDesc->pGpu, AT_GPU, offset,
-                              pageSize, pParams->numPfns,
-                              pFabricArray);
+    memdescGetPhysAddrsForGpu(pMemory->pMemDesc, pGpu, AT_GPU, offset,
+                              pageSize, pParams->numPfns, pFabricArray);
 
     for (i = 0; i < pParams->numPfns; i++)
     {
@@ -991,7 +1044,6 @@ memoryfabricCtrlDetachMem_IMPL
 {
     NvU32 i;
     NV_STATUS status;
-    Memory *pMemory = staticCast(pMemoryFabric, Memory);
 
     pParams->numDetached = 0;
 
@@ -1004,7 +1056,9 @@ memoryfabricCtrlDetachMem_IMPL
 
     for (i = 0; i < pParams->numOffsets; i++)
     {
-        status = _memoryFabricDetachMem(pMemory->pMemDesc, pParams->offsets[i]);
+        // Remove inter mapping on explicit detach by the control call.
+        status = _memoryFabricDetachMem(pMemoryFabric, pParams->offsets[i],
+                                        NV_TRUE);
         if (status != NV_OK)
             return status;
 
@@ -1029,7 +1083,9 @@ memoryfabricCtrlGetNumAttachedMem_IMPL
 
     pParams->numMemInfos = 0;
 
-    btreeEnumStart(pParams->offsetStart, &pNode, pMemdescData->pAttachMemInfoTree);
+    btreeEnumStart(pParams->offsetStart, &pNode,
+                   pMemdescData->pAttachMemInfoTree);
+
     while ((pNode != NULL) && (pNode->keyStart <= pParams->offsetEnd))
     {
         pParams->numMemInfos++;
@@ -1039,70 +1095,122 @@ memoryfabricCtrlGetNumAttachedMem_IMPL
     return NV_OK;
 }
 
-NV_STATUS
-memoryfabricCtrlGetAttachedMem_IMPL
+static NV_STATUS
+_memoryfabricGetPhysAttrsUsingFabricMemdesc
 (
-    MemoryFabric                        *pMemoryFabric,
-    NV00F8_CTRL_GET_ATTACHED_MEM_PARAMS *pParams
+    OBJGPU            *pGpu,
+    FABRIC_VASPACE    *pFabricVAS,
+    MEMORY_DESCRIPTOR *pFabricMemDesc,
+    NvU64              offset,
+    NvU64             *pPhysPageSize
 )
 {
     NV_STATUS status;
-    Memory *pMemory = staticCast(pMemoryFabric, Memory);
-    MEMORY_DESCRIPTOR *pFabricMemDesc = pMemory->pMemDesc;
-    FABRIC_MEMDESC_DATA *pMemdescData = \
-                    (FABRIC_MEMDESC_DATA *)memdescGetMemData(pFabricMemDesc);
-    OBJGPU *pGpu = pMemory->pGpu;
-    RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
-    FABRIC_VASPACE *pFabricVAS = dynamicCast(pGpu->pFabricVAS, FABRIC_VASPACE);
-    FABRIC_ATTCH_MEM_INFO_NODE *pAttachMemInfoNode;
+    MEMORY_DESCRIPTOR *pPhysMemDesc;
+    FABRIC_MEMDESC_DATA *pMemdescData;
     NODE *pNode = NULL;
-    NvU16 i, count = 0;
+    FABRIC_ATTCH_MEM_INFO_NODE *pAttachMemInfoNode;
 
-    if ((pParams->numMemInfos == 0) ||
-        (pParams->numMemInfos > NV00F8_MAX_ATTACHED_MEM_INFOS))
-        return NV_ERR_INVALID_ARGUMENT;
-
-    btreeEnumStart(pParams->offsetStart, &pNode, pMemdescData->pAttachMemInfoTree);
-    while (count < pParams->numMemInfos)
+    if ((pFabricMemDesc == NULL) || (pPhysPageSize == NULL))
     {
-        if (pNode == NULL)
-        {
-            status = NV_ERR_INVALID_ARGUMENT;
-            goto cleanup;
-        }
+        return NV_ERR_INVALID_ARGUMENT;
+    }
 
-        pAttachMemInfoNode = (FABRIC_ATTCH_MEM_INFO_NODE *)pNode->Data;
+    status = fabricvaspaceGetGpaMemdesc(pFabricVAS, pFabricMemDesc, pGpu,
+                                        &pPhysMemDesc);
+    if (status == NV_OK)
+    {
+        *pPhysPageSize = memdescGetPageSize(pPhysMemDesc, AT_GPU);
+        if (pPhysMemDesc != pFabricMemDesc)
+            fabricvaspacePutGpaMemdesc(pFabricVAS, pPhysMemDesc);
+        return NV_OK;
+    }
 
-        status = pRmApi->DupObject(pRmApi, RES_GET_CLIENT_HANDLE(pMemory),
-                                   RES_GET_HANDLE(pMemory->pDevice),
-                                   &pParams->memInfos[count].hMemory,
-                                   pFabricVAS->hClient,
-                                   pAttachMemInfoNode->hDupedPhysMem, 0);
-        if (status != NV_OK)
-        {
-            NV_PRINTF(LEVEL_ERROR, "Failed to dup physmem handle\n");
-            goto cleanup;
-        }
+    if (status != NV_ERR_OBJECT_NOT_FOUND)
+        return status;
 
-        pParams->memInfos[count].offset = pNode->keyStart;
-        pParams->memInfos[count].mapOffset = pAttachMemInfoNode->physMapOffset;
-        pParams->memInfos[count].mapLength = pAttachMemInfoNode->physMapLength;
+    // If the object is flexible, check the attach mem info tree.
+    pMemdescData = (FABRIC_MEMDESC_DATA *)memdescGetMemData(pFabricMemDesc);
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, btreeSearch(offset, &pNode,
+                          pMemdescData->pAttachMemInfoTree));
+    pAttachMemInfoNode = (FABRIC_ATTCH_MEM_INFO_NODE*)pNode->Data;
 
-        btreeEnumNext(&pNode, pMemdescData->pAttachMemInfoTree);
-        count++;
+    *pPhysPageSize = memdescGetPageSize(pAttachMemInfoNode->pPhysMemDesc,
+                                          AT_GPU);
+
+    return NV_OK;
+}
+
+NV_STATUS
+memoryfabricCtrlGetPageLevelInfo_IMPL
+(
+    MemoryFabric                           *pMemoryFabric,
+    NV00F8_CTRL_GET_PAGE_LEVEL_INFO_PARAMS *pParams
+)
+{
+    Memory *pMemory = staticCast(pMemoryFabric, Memory);
+    OBJGPU *pGpu  = pMemory->pGpu;
+    FABRIC_VASPACE *pFabricVAS = dynamicCast(pGpu->pFabricVAS, FABRIC_VASPACE);
+    NV90F1_CTRL_VASPACE_GET_PAGE_LEVEL_INFO_PARAMS pageLevelInfoParams = {0};
+    MEMORY_DESCRIPTOR *pFabricMemDesc = pMemory->pMemDesc;
+    NvU64 fabricAddr;
+    NvU64 mappingPageSize;
+    NvU64 physPageSize;
+    NvU32 i;
+
+    NV_ASSERT_OR_RETURN(pFabricMemDesc != NULL, NV_ERR_INVALID_ARGUMENT);
+
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+                          _memoryfabricGetPhysAttrsUsingFabricMemdesc(pGpu,
+                          pFabricVAS, pFabricMemDesc, pParams->offset,
+                          &physPageSize));
+
+    mappingPageSize = NV_MIN(physPageSize,
+                             memdescGetPageSize(pFabricMemDesc, AT_GPU));
+
+    NV_CHECK_OR_RETURN(LEVEL_ERROR,
+                       NV_IS_ALIGNED64(pParams->offset, mappingPageSize),
+                       NV_ERR_INVALID_ARGUMENT);
+
+    NV_CHECK_OR_RETURN(LEVEL_ERROR,
+                       pParams->offset < memdescGetSize(pFabricMemDesc),
+                       NV_ERR_INVALID_ARGUMENT);
+
+    memdescGetPhysAddrsForGpu(pFabricMemDesc, pGpu, AT_GPU, pParams->offset,
+                              mappingPageSize, 1, &fabricAddr);
+
+    pageLevelInfoParams.virtAddress = fabricAddr;
+    pageLevelInfoParams.pageSize = mappingPageSize;
+
+    NV_ASSERT_OK_OR_RETURN(fabricvaspaceGetPageLevelInfo(pFabricVAS, pGpu,
+                           &pageLevelInfoParams));
+
+    pParams->numLevels = pageLevelInfoParams.numLevels;
+    for (i = 0; i < pParams->numLevels; i++)
+    {
+        portMemCopy(&pParams->levels[i], sizeof(pParams->levels[i]),
+                    &pageLevelInfoParams.levels[i],
+                    sizeof(pageLevelInfoParams.levels[i]));
     }
 
     return NV_OK;
+}
 
-cleanup:
-    for (i = 0; i < count; i++)
-    {
-        NV_ASSERT_OK(pRmApi->Free(pRmApi, RES_GET_CLIENT_HANDLE(pMemory),
-                                  pParams->memInfos[i].hMemory));
-        pParams->memInfos[i].offset = 0;
-        pParams->memInfos[i].mapOffset = 0;
-        pParams->memInfos[i].mapLength = 0;
-    }
-
-    return status;
+NV_STATUS
+memoryfabricUnmapFrom_IMPL
+(
+    MemoryFabric             *pMemoryFabric,
+    RS_RES_UNMAP_FROM_PARAMS *pParams
+)
+{
+    OBJGPU *pGpu = pParams->pGpu;
+    NvU32 gpuMask = NVBIT(gpuGetInstance(pGpu));
+
+    NV_ASSERT_OR_RETURN(rmGpuGroupLockIsOwner(0, GPU_LOCK_GRP_MASK, &gpuMask),
+                        NV_ERR_INVALID_STATE);
+
+    NV_ASSERT_OK_OR_RETURN(_memoryFabricDetachMem(pMemoryFabric,
+                                                  pParams->dmaOffset, NV_FALSE));
+
+    return NV_OK;
 }
