@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1999-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -34,9 +34,6 @@
 #include <gpu_mgr/gpu_mgr.h>
 #include "kernel/gpu/intr/intr.h"
 #include <gpu/bif/kernel_bif.h>
-#include <objtmr.h>
-
-#include "g_os_private.h"
 
 /*!
  * @brief Wait for interrupt
@@ -111,7 +108,7 @@ NV_STATUS osSanityTestIsr(
 
     osWaitForInterrupt(pGpu, &serviced);
 
-    return (serviced) ? NV_OK : NV_ERR_GENERIC;
+    return (serviced) ? NV_OK : NV_ERR_INVALID_STATE;
 }
 
 //
@@ -123,10 +120,8 @@ static NV_STATUS _osVerifyInterrupts(
     OBJGPU *pGpu
 )
 {
-#if !defined(NV_UNIX) || defined(NV_MODS)
-    OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
-#endif
     Intr *pIntr = GPU_GET_INTR(pGpu);
+    KernelBif *pKernelBif = GPU_GET_KERNEL_BIF(pGpu);
     OBJGPU *pGpuSaved = pGpu;
     NvU32  *pIntrEn0, *pIntrEn1;
     MC_ENGINE_BITVECTOR intrMask;
@@ -239,13 +234,9 @@ static NV_STATUS _osVerifyInterrupts(
 
     while (!interrupt_triggered)
     {
-#if defined(NV_UNIX) && !defined(NV_MODS)
-        osDelay(50);
-        Bailout += 50 * 1000;
-#else
-        tmrDelay(pTmr, 5 * 1000);
+        osDelayUs(5);
         Bailout += 5;
-#endif
+
         if (Bailout > pGpu->timeoutData.defaultus)
             break;
     }
@@ -254,7 +245,6 @@ static NV_STATUS _osVerifyInterrupts(
     // Message Signalled Interrupt (MSI) support
     // This call checks if MSI is enabled and if it is, we need re-arm it.
     //
-    KernelBif *pKernelBif = GPU_GET_KERNEL_BIF(pGpu);
     kbifCheckAndRearmMSI(pGpu, pKernelBif);
 
     pGpu->testIntr = NV_FALSE;
