@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2015-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2015-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -36,7 +36,6 @@
 
 #if PORT_MEM_TRACK_USE_LIMIT
 #include "os/os.h"
-#define PORT_MEM_LIMIT_MAX_PIDS 32
 #endif
 
 #if NVOS_IS_LIBOS
@@ -174,8 +173,7 @@ static struct PORT_MEM_GLOBALS
     NvU32 totalAllocators;
 #if PORT_MEM_TRACK_USE_LIMIT
     NvBool bLimitEnabled;
-    NvLength limitPid[PORT_MEM_LIMIT_MAX_PIDS];
-    NvLength counterPid[PORT_MEM_LIMIT_MAX_PIDS];
+    PORT_MEM_ALLOCATOR_TRACKING *pGfidTracking[PORT_MEM_LIMIT_MAX_GFID];
 #endif
 } portMemGlobals;
 
@@ -190,16 +188,7 @@ _portMemExTrackingGetAllocUsableSizeWrapper
     void *pMem
 )
 {
-#if PORT_IS_FUNC_SUPPORTED(portMemExTrackingGetAllocUsableSize)
-    //
-    // blockSize in PORT_MEM_HEADER does not include the size of the header/
-    // footer, but the underlying malloc implementation doesn't know about
-    // those. Account for them here.
-    //
-    NvLength allocSize = portMemExTrackingGetAllocUsableSize(PORT_MEM_SUB_HEADER_PTR(pMem));
-    PORT_ASSERT_CHECKED(allocSize > PORT_MEM_STAGING_SIZE);
-    return allocSize - PORT_MEM_STAGING_SIZE;
-#elif PORT_MEM_HEADER_HAS_BLOCK_SIZE
+#if PORT_MEM_HEADER_HAS_BLOCK_SIZE
     return PORT_MEM_SUB_HEADER_PTR(pMem)->blockSize;
 #endif
 }
@@ -459,7 +448,7 @@ _portMemCallerInfoInitTracking
     _portMemCallerInfoInitMem(pMem, PORT_MEM_CALLERINFO_PARAM)
 
 #if PORT_MEM_TRACK_USE_CALLERINFO_IP
-#if NVCPU_IS_RISCV64
+#if NVOS_IS_LIBOS
 //
 // Libos has custom %a format specifier that decodes an instruction pointer into
 // a function / file / line reference when the binary output is decoded.
@@ -467,7 +456,7 @@ _portMemCallerInfoInitTracking
 #define PORT_MEM_CALLERINFO_PRINT_ARGS(x)  "@ %a\n", x
 #else
 #define PORT_MEM_CALLERINFO_PRINT_ARGS(x)  "@ 0x%016x\n", x
-#endif // NVCPU_IS_RISCV64
+#endif // NVOS_IS_LIBOS
 #else
 #define PORT_MEM_CALLERINFO_PRINT_ARGS(x)  "@ %s:%u (%s)\n", x.file, x.line, x.func
 #endif // PORT_MEM_TRACK_USE_CALLERINFO_IP
@@ -559,7 +548,13 @@ static void    *_portMemAllocatorAllocExistingWrapper(PORT_MEM_ALLOCATOR *pAlloc
 static void     _portMemAllocatorFreeExistingWrapper(PORT_MEM_ALLOCATOR *pAlloc, void *pMem);
 
 static void _portMemTrackingRelease(PORT_MEM_ALLOCATOR_TRACKING *pTracking, NvBool bReportLeaks);
-static void _portMemTrackAlloc(PORT_MEM_ALLOCATOR_TRACKING *pTracking, void *pMem, NvLength size, NvU32 pid PORT_MEM_CALLERINFO_COMMA_TYPE_PARAM);
+static void _portMemTrackAlloc(
+    PORT_MEM_ALLOCATOR_TRACKING *pTracking,
+    void *pMem,
+    NvLength size,
+    NvU32 gfid
+    PORT_MEM_CALLERINFO_COMMA_TYPE_PARAM
+);
 static void _portMemTrackFree(PORT_MEM_ALLOCATOR_TRACKING *pTracking, void *pMem);
 
 
@@ -588,17 +583,36 @@ static void _portMemTrackFree(PORT_MEM_ALLOCATOR_TRACKING *pTracking, void *pMem
 // Per-process heap limiting implementation
 //
 #if PORT_MEM_TRACK_USE_LIMIT
+static inline NvBool isGfidValid(NvU32 gfid)
+{
+    return (gfid > 0) && (gfid <= PORT_MEM_LIMIT_MAX_GFID);
+}
+
+static void _portMemTrackingGfidRelease(void)
+{
+    int gfidIdx;
+
+    for (gfidIdx = 0; gfidIdx < PORT_MEM_LIMIT_MAX_GFID; ++gfidIdx)
+    {
+        if (portMemGlobals.pGfidTracking[gfidIdx] != NULL)
+        {
+            _portMemTrackingRelease(portMemGlobals.pGfidTracking[gfidIdx], NV_FALSE);
+            portMemGlobals.pGfidTracking[gfidIdx] = NULL;
+        }
+    }
+}
+
 static NV_INLINE void
-_portMemLimitInc(NvU32 pid, void *pMem, NvLength size)
+_portMemLimitInc(NvU32 gfid, void *pMem, NvLength size)
 {
     PORT_MEM_HEADER *pMemHeader = PORT_MEM_SUB_HEADER_PTR(pMem);
-    pMemHeader->pid = pid;
+    pMemHeader->gfid = gfid;
     if (portMemGlobals.bLimitEnabled)
     {
-        if ((pid > 0) && (pid <= PORT_MEM_LIMIT_MAX_PIDS))
+        if (isGfidValid(gfid))
         {
-            NvU32 pidIdx = pid - 1;
-            PORT_MEM_ATOMIC_ADD_SIZE(&portMemGlobals.counterPid[pidIdx], size);
+            NvU32 gfidIdx = gfid - 1;
+            PORT_MEM_ATOMIC_ADD_SIZE(&portMemGlobals.pGfidTracking[gfidIdx]->counterGfid, size);
         }
     }
 }
@@ -609,38 +623,44 @@ _portMemLimitDec(void *pMem, NvLength size)
     if (portMemGlobals.bLimitEnabled)
     {
         PORT_MEM_HEADER *pMemHeader = PORT_MEM_SUB_HEADER_PTR(pMem);
-        NvU32 pid = pMemHeader->pid;
+        NvU32 gfid = pMemHeader->gfid;
 
-        if ((pid > 0) && (pid <= PORT_MEM_LIMIT_MAX_PIDS))
+        if (isGfidValid(gfid))
         {
-            NvU32 pidIdx = pid - 1;
-            if (portMemGlobals.counterPid[pidIdx] < size)
+            NvU32 gfidIdx = gfid - 1;
+            if (portMemGlobals.pGfidTracking[gfidIdx]->counterGfid < size)
             {
                 PORT_MEM_PRINT_ERROR("memory free error: counter underflow\n");
                 PORT_BREAKPOINT_CHECKED();
             }
             else
             {
-                PORT_MEM_ATOMIC_SUB_SIZE(&portMemGlobals.counterPid[pidIdx], size);
+                PORT_MEM_ATOMIC_SUB_SIZE(
+                    &portMemGlobals.pGfidTracking[gfidIdx]->counterGfid,
+                    size);
             }
         }
     }
 }
 
 static NV_INLINE NvBool
-_portMemLimitExceeded(NvU32 pid, NvLength size)
+_portMemLimitExceeded(NvU32 gfid, NvLength size)
 {
     NvBool bExceeded = NV_FALSE;
 
     if (portMemGlobals.bLimitEnabled)
     {
-        if ((pid > 0) && (pid <= PORT_MEM_LIMIT_MAX_PIDS))
+        if (isGfidValid(gfid))
         {
-            NvU32 pidIdx = pid - 1;
-            if ((size + portMemGlobals.counterPid[pidIdx]) > portMemGlobals.limitPid[pidIdx])
+            NvU32 gfidIdx = gfid - 1;
+            if ((size + portMemGlobals.pGfidTracking[gfidIdx]->counterGfid) >
+                portMemGlobals.pGfidTracking[gfidIdx]->limitGfid)
             {
-                PORT_MEM_PRINT_ERROR("memory allocation denied; PID %d exceeded per-process heap limit of %"NvUPtr_fmtu"\n",
-                                      pid, portMemGlobals.limitPid[pidIdx]);
+                PORT_MEM_PRINT_ERROR(
+                    "memory allocation denied; GFID %d exceeded per-process heap limit of "
+                    "%"NvUPtr_fmtu"\n",
+                    gfid, portMemGlobals.pGfidTracking[gfidIdx]->limitGfid
+                );
                 bExceeded = NV_TRUE;
             }
         }
@@ -648,13 +668,13 @@ _portMemLimitExceeded(NvU32 pid, NvLength size)
     return bExceeded;
 }
 
-#define PORT_MEM_LIMIT_INC(pid, pMem, size) _portMemLimitInc(pid, pMem, size)
+#define PORT_MEM_LIMIT_INC(gfid, pMem, size) _portMemLimitInc(gfid, pMem, size)
 #define PORT_MEM_LIMIT_DEC(pMem, size)      _portMemLimitDec(pMem, size)
-#define PORT_MEM_LIMIT_EXCEEDED(pid, size)  _portMemLimitExceeded(pid, size)
+#define PORT_MEM_LIMIT_EXCEEDED(gfid, size)  _portMemLimitExceeded(gfid, size)
 #else
-#define PORT_MEM_LIMIT_INC(pid, pMem, size) \
+#define PORT_MEM_LIMIT_INC(gfid, pMem, size) \
     do {                                    \
-        PORT_UNREFERENCED_VARIABLE(pid);    \
+        PORT_UNREFERENCED_VARIABLE(gfid);    \
         PORT_UNREFERENCED_VARIABLE(pMem);   \
         PORT_UNREFERENCED_VARIABLE(size);   \
     } while (0)
@@ -663,7 +683,7 @@ _portMemLimitExceeded(NvU32 pid, NvLength size)
         PORT_UNREFERENCED_VARIABLE(pMem);   \
         PORT_UNREFERENCED_VARIABLE(size);   \
     } while (0)
-#define PORT_MEM_LIMIT_EXCEEDED(pid, size)  (NV_FALSE)
+#define PORT_MEM_LIMIT_EXCEEDED(gfid, size)  (NV_FALSE)
 #endif // PORT_MEM_TRACK_USE_LIMIT
 
 static NV_INLINE PORT_MEM_ALLOCATOR_TRACKING *
@@ -698,8 +718,6 @@ portMemInitialize(void)
 #if PORT_MEM_TRACK_USE_LIMIT
     // Initialize process heap limit to max int (i.e. no limit)
     portMemGlobals.bLimitEnabled = NV_FALSE;
-    portMemSet(&portMemGlobals.limitPid, NV_U8_MAX, sizeof(portMemGlobals.limitPid));
-    portMemSet(&portMemGlobals.counterPid, 0, sizeof(portMemGlobals.counterPid));
 #endif
 
     portMemGlobals.alloc.paged._portAlloc      = _portMemAllocatorAllocPagedWrapper;
@@ -731,7 +749,7 @@ portMemInitialize(void)
         // Use the same impl for both paged and nonpaged.
         portMemGlobals.alloc.paged.pImpl    = &portMemGlobals.alloc.pagedImpl;
         portMemGlobals.alloc.nonPaged.pImpl = &portMemGlobals.alloc.pagedImpl;
-        portMemInitializeAllocatorTracking(NULL,
+        portMemInitializeAllocatorTracking(&portMemGlobals.alloc.paged,
                              &portMemGlobals.alloc.pagedImpl.tracking
                              PORT_MEM_CALLERINFO_COMMA_PARAM);
         portMemGlobals.alloc.paged.pTracking    = &portMemGlobals.alloc.pagedImpl.tracking;
@@ -763,7 +781,9 @@ portMemShutdown(NvBool bForceSilent)
     {
         _portMemTrackingRelease(&portMemGlobals.alloc.pagedImpl.tracking, NV_FALSE);
     }
-
+#if PORT_MEM_TRACK_USE_LIMIT
+    _portMemTrackingGfidRelease();
+#endif
     PORT_MEM_LOCK_DESTROY(portMemGlobals.trackingLock);
     PORT_MEM_LIST_DESTROY(&portMemGlobals.mainTracking);
     portMemSet(&portMemGlobals, 0, sizeof(portMemGlobals));
@@ -948,12 +968,76 @@ portMemInitializeAllocatorTracking
 
 #if PORT_MEM_TRACK_USE_LIMIT
 void
-portMemInitializeAllocatorTrackingLimit(NvU32 pid, NvLength limit, NvBool bLimitEnabled)
+portMemInitializeAllocatorTrackingLimit(NvU32 gfid, NvLength limit, NvBool bLimitEnabled)
 {
-    NvU32 pidIdx = pid - 1;
-    portMemGlobals.limitPid[pidIdx] = limit;
+    if (!isGfidValid(gfid))
+        return;
+
+    NvU32 gfidIdx = gfid - 1;
+
+    portMemGlobals.pGfidTracking[gfidIdx]->limitGfid = limit;
     portMemGlobals.bLimitEnabled = bLimitEnabled;
 }
+
+void portMemGfidTrackingInit(NvU32 gfid)
+{
+    if (!isGfidValid(gfid))
+    {
+        PORT_BREAKPOINT_CHECKED();
+        return;
+    }
+
+    NvU32 gfidIdx = gfid - 1;
+
+    if (portMemGlobals.pGfidTracking[gfidIdx] != NULL)
+        return;
+
+    PORT_MEM_ALLOCATOR_TRACKING *pTracking =
+        _portMemAllocNonPagedUntracked(sizeof(PORT_MEM_ALLOCATOR_TRACKING));
+
+    if (pTracking == NULL)
+    {
+        portDbgPrintf("!!! Failed memory allocation for pTracking !!!\n");
+        PORT_BREAKPOINT_CHECKED();
+        return;
+    }
+
+    portMemSet(pTracking, 0, sizeof(*pTracking));
+    pTracking->limitGfid = NV_U64_MAX;
+    pTracking->counterGfid = 0;
+    pTracking->gfid = gfid;
+    PORT_LOCKED_LIST_LINK(&portMemGlobals.mainTracking, pTracking, portMemGlobals.trackingLock);
+    PORT_MEM_COUNTER_INIT(&pTracking->counter);
+    portMemGlobals.pGfidTracking[gfidIdx] = pTracking;
+}
+
+void portMemGfidTrackingFree(NvU32 gfid)
+{
+    if (!isGfidValid(gfid))
+    {
+        PORT_BREAKPOINT_CHECKED();
+        return;
+    }
+
+    NvU32 gfidIdx = gfid - 1;
+    PORT_MEM_ALLOCATOR_TRACKING *pTracking = portMemGlobals.pGfidTracking[gfidIdx];
+
+    if (pTracking == NULL)
+    {
+        PORT_BREAKPOINT_CHECKED();
+        return;
+    }
+
+    if (pTracking->counter.activeAllocs != 0)
+    {
+        portDbgPrintf("  !!! MEMORY LEAK DETECTED (%u blocks) !!!\n",
+                      pTracking->counter.activeAllocs);
+
+    }
+
+    portMemPrintTrackingInfo(pTracking);
+}
+
 #endif
 
 void *
@@ -964,7 +1048,7 @@ _portMemAllocatorAlloc
     PORT_MEM_CALLERINFO_COMMA_TYPE_PARAM
 )
 {
-    NvU32 pid = 0;
+    NvU32 gfid = 0;
     void *pMem = NULL;
     if (pAlloc == NULL)
     {
@@ -975,7 +1059,7 @@ _portMemAllocatorAlloc
 #if PORT_MEM_TRACK_USE_LIMIT
     if (portMemGlobals.bLimitEnabled)
     {
-        if (osGetCurrentProcessGfid(&pid) != NV_OK)
+        if (osGetCurrentProcessGfid(&gfid) != NV_OK)
         {
             PORT_BREAKPOINT_CHECKED();
             return NULL;
@@ -984,7 +1068,7 @@ _portMemAllocatorAlloc
 #endif
 
     // Check if per-process memory limit will be exhausted by this allocation
-    if (PORT_MEM_LIMIT_EXCEEDED(pid, length))
+    if (PORT_MEM_LIMIT_EXCEEDED(gfid, length))
         return NULL;
 
     if (length > 0)
@@ -1009,7 +1093,7 @@ _portMemAllocatorAlloc
     if (pMem != NULL)
     {
         pMem = PORT_MEM_ADD_HEADER_PTR(pMem);
-        _portMemTrackAlloc(_portMemGetTracking(pAlloc), pMem, length, pid
+        _portMemTrackAlloc(_portMemGetTracking(pAlloc), pMem, length, gfid
                            PORT_MEM_CALLERINFO_COMMA_PARAM);
     }
     return pMem;
@@ -1044,16 +1128,20 @@ portMemPrintTrackingInfo
         pTracking = &portMemGlobals.mainTracking;
 
     if (pTracking == &portMemGlobals.mainTracking)
-        portDbgPrintf("[NvPort] ******** Aggregate Memory Tracking ********\n");
+        portDbgPrintf("[NvPort] ======== Aggregate Memory Tracking ========\n");
     else if ((pTracking == portMemGlobals.alloc.nonPaged.pTracking) &&
              (pTracking == portMemGlobals.alloc.paged.pTracking))
-        portDbgPrintf("[NvPort] ******** Global Allocator Tracking ********\n");
+        portDbgPrintf("[NvPort] ======== Global Allocator Tracking ========\n");
     else if (pTracking == portMemGlobals.alloc.nonPaged.pTracking)
-        portDbgPrintf("[NvPort] ******** Global Non-Paged Memory Allocator Tracking ********\n");
+        portDbgPrintf("[NvPort] ======== Global Non-Paged Memory Allocator Tracking ========\n");
     else if (pTracking == portMemGlobals.alloc.paged.pTracking)
-        portDbgPrintf("[NvPort] ******** Global Paged Memory Allocator Tracking ********\n");
+        portDbgPrintf("[NvPort] ======== Global Paged Memory Allocator Tracking ========\n");
+#if PORT_MEM_TRACK_USE_LIMIT
+    else if (isGfidValid(pTracking->gfid))
+        portDbgPrintf("[NvPort] ======== GFID %u Tracking ========\n", pTracking->gfid);
+#endif
     else
-        portDbgPrintf("[NvPort] ******** Memory Allocator %p Tracking ******** \n", pTracking->pAllocator);
+        portDbgPrintf("[NvPort] ======== Memory Allocator %p Tracking ======== \n", pTracking->pAllocator);
 
     if (pTracking->counter.activeAllocs != 0)
         portDbgPrintf("  !!! MEMORY LEAK DETECTED (%u blocks) !!!\n",
@@ -1080,8 +1168,16 @@ portMemPrintTrackingInfo
         PORT_MEM_TRACK_ALLOCATOR_STATS stats;
 
         portMemSet(&stats, 0, sizeof(stats));
-
-        portMemExTrackingGetActiveStats(pTracking->pAllocator, &stats);
+#if PORT_MEM_TRACK_USE_LIMIT
+        if (isGfidValid(pTracking->gfid))
+        {
+            portMemExTrackingGetGfidActiveStats(pTracking->gfid, &stats);
+        }
+        else
+#endif
+        {
+            portMemExTrackingGetActiveStats(pTracking->pAllocator, &stats);
+        }
 
         //
         // rmtest_gsp test script (dvs_gsp_sanity.sh) depends on this print, so do not change
@@ -1100,8 +1196,16 @@ portMemPrintTrackingInfo
         PORT_MEM_TRACK_ALLOCATOR_STATS stats;
 
         portMemSet(&stats, 0, sizeof(stats));
-
-        portMemExTrackingGetTotalStats(pTracking->pAllocator, &stats);
+#if PORT_MEM_TRACK_USE_LIMIT
+        if (isGfidValid(pTracking->gfid))
+        {
+            portMemExTrackingGetGfidTotalStats(pTracking->gfid, &stats);
+        }
+        else
+#endif
+        {
+            portMemExTrackingGetTotalStats(pTracking->pAllocator, &stats);
+        }
         portDbgPrintf("  TOTAL:  %u allocations, %"NvUPtr_fmtu" bytes allocated (%"NvUPtr_fmtu" useful, %"NvUPtr_fmtu" meta)\n",
                     stats.numAllocations,
                     stats.allocatedSize,
@@ -1115,8 +1219,16 @@ portMemPrintTrackingInfo
         PORT_MEM_TRACK_ALLOCATOR_STATS stats;
 
         portMemSet(&stats, 0, sizeof(stats));
-
-        portMemExTrackingGetPeakStats(pTracking->pAllocator, &stats);
+#if PORT_MEM_TRACK_USE_LIMIT
+        if (isGfidValid(pTracking->gfid))
+        {
+            portMemExTrackingGetGfidPeakStats(pTracking->gfid, &stats);
+        }
+        else
+#endif
+        {
+            portMemExTrackingGetPeakStats(pTracking->pAllocator, &stats);
+        }
         portDbgPrintf("  PEAK:   %u allocations, %"NvUPtr_fmtu" bytes allocated (%"NvUPtr_fmtu" useful, %"NvUPtr_fmtu" meta)\n",
                     stats.numAllocations,
                     stats.allocatedSize,
@@ -1185,6 +1297,33 @@ portMemExTrackingGetActiveStats
     pStats->allocatedSize  = pStats->usefulSize + pStats->metaSize;
     return NV_OK;
 }
+#if PORT_MEM_TRACK_USE_LIMIT
+NV_STATUS
+portMemExTrackingGetGfidActiveStats
+(
+    NvU32                           gfid,
+    PORT_MEM_TRACK_ALLOCATOR_STATS *pStats
+)
+{
+    if (!isGfidValid(gfid))
+    {
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+
+    NvU32 gfidIdx = gfid - 1;
+    PORT_MEM_ALLOCATOR_TRACKING *pTracking = portMemGlobals.pGfidTracking[gfidIdx];
+
+    if (pTracking == NULL)
+    {
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+    pStats->numAllocations = pTracking->counter.activeAllocs;
+    pStats->usefulSize     = pTracking->counter.activeSize;
+    pStats->metaSize       = pStats->numAllocations * PORT_MEM_STAGING_SIZE;
+    pStats->allocatedSize  = pStats->usefulSize + pStats->metaSize;
+    return NV_OK;
+}
+#endif
 #endif
 
 #if portMemExTrackingGetTotalStats_SUPPORTED
@@ -1206,6 +1345,33 @@ portMemExTrackingGetTotalStats
     pStats->allocatedSize  = pStats->usefulSize + pStats->metaSize;
     return NV_OK;
 }
+#if PORT_MEM_TRACK_USE_LIMIT
+NV_STATUS
+portMemExTrackingGetGfidTotalStats
+(
+    NvU32                           gfid,
+    PORT_MEM_TRACK_ALLOCATOR_STATS *pStats
+)
+{
+    if (!isGfidValid(gfid))
+    {
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+
+    NvU32 gfidIdx = gfid - 1;
+    PORT_MEM_ALLOCATOR_TRACKING *pTracking = portMemGlobals.pGfidTracking[gfidIdx];
+
+    if (pTracking == NULL)
+    {
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+    pStats->numAllocations = pTracking->counter.totalAllocs;
+    pStats->usefulSize     = pTracking->counter.totalSize;
+    pStats->metaSize       = pStats->numAllocations * PORT_MEM_STAGING_SIZE;
+    pStats->allocatedSize  = pStats->usefulSize + pStats->metaSize;
+    return NV_OK;
+}
+#endif
 #endif
 
 #if portMemExTrackingGetPeakStats_SUPPORTED
@@ -1227,6 +1393,33 @@ portMemExTrackingGetPeakStats
     pStats->allocatedSize  = pStats->usefulSize + pStats->metaSize;
     return NV_OK;
 }
+#if PORT_MEM_TRACK_USE_LIMIT
+NV_STATUS
+portMemExTrackingGetGfidPeakStats
+(
+    NvU32                           gfid,
+    PORT_MEM_TRACK_ALLOCATOR_STATS *pStats
+)
+{
+    if (!isGfidValid(gfid))
+    {
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+
+    NvU32 gfidIdx = gfid - 1;
+    PORT_MEM_ALLOCATOR_TRACKING *pTracking = portMemGlobals.pGfidTracking[gfidIdx];
+
+    if (pTracking == NULL)
+    {
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+    pStats->numAllocations = pTracking->counter.peakAllocs;
+    pStats->usefulSize     = pTracking->counter.peakSize;
+    pStats->metaSize       = pStats->numAllocations * PORT_MEM_STAGING_SIZE;
+    pStats->allocatedSize  = pStats->usefulSize + pStats->metaSize;
+    return NV_OK;
+}
+#endif
 #endif
 
 #if portMemExTrackingGetNext_SUPPORTED
@@ -1295,7 +1488,16 @@ _portMemTrackingRelease
 
     PORT_LOCKED_LIST_UNLINK(&portMemGlobals.mainTracking, pTracking, portMemGlobals.trackingLock);
     PORT_MEM_LIST_DESTROY(pTracking);
-    PORT_MEM_ATOMIC_DEC_U32(&portMemGlobals.totalAllocators);
+#if PORT_MEM_TRACK_USE_LIMIT
+    if (isGfidValid(pTracking->gfid))
+    {
+        _portMemFreeUntracked(pTracking);
+    }
+    else
+#endif
+    {
+        PORT_MEM_ATOMIC_DEC_U32(&portMemGlobals.totalAllocators);
+    }
 }
 
 static void
@@ -1304,7 +1506,7 @@ _portMemTrackAlloc
     PORT_MEM_ALLOCATOR_TRACKING *pTracking,
     void                        *pMem,
     NvLength                     size,
-    NvU32                        pid
+    NvU32                        gfid
     PORT_MEM_CALLERINFO_COMMA_TYPE_PARAM
 )
 {
@@ -1325,12 +1527,20 @@ _portMemTrackAlloc
 
     PORT_MEM_COUNTER_INC(&pTracking->counter, size);
     PORT_MEM_COUNTER_INC(&portMemGlobals.mainTracking.counter, size);
-    PORT_MEM_LIMIT_INC(pid, pMem, size);
+    PORT_MEM_LIMIT_INC(gfid, pMem, size);
 
     PORT_MEM_FENCE_INIT(pTracking->pAllocator, pMem, size);
     PORT_MEM_LIST_ADD(pTracking, pMem);
     PORT_MEM_CALLERINFO_INIT_MEM(pMem);
     PORT_MEM_LOG_ALLOC(pTracking->pAllocator, pMem, size);
+#if PORT_MEM_TRACK_USE_LIMIT
+    if (isGfidValid(gfid))
+    {
+        NvU32 gfidIdx = gfid - 1;
+
+        PORT_MEM_COUNTER_INC(&portMemGlobals.pGfidTracking[gfidIdx]->counter, size);
+    }
+#endif
 }
 
 static void
@@ -1358,6 +1568,16 @@ _portMemTrackFree
     PORT_MEM_FENCE_CHECK(pTracking->pAllocator, pMem, size);
     PORT_MEM_LIST_REMOVE(pTracking, pMem);
     PORT_MEM_LOG_FREE(pTracking->pAllocator, pMem);
+#if PORT_MEM_TRACK_USE_LIMIT
+    PORT_MEM_HEADER *pMemHeader = PORT_MEM_SUB_HEADER_PTR(pMem);
+
+    if (isGfidValid(pMemHeader->gfid)) 
+    {
+        NvU32 gfidIdx = pMemHeader->gfid - 1;
+
+        PORT_MEM_COUNTER_DEC(&portMemGlobals.pGfidTracking[gfidIdx]->counter, size);
+    }
+#endif
 }
 
 
